(function($) {
    
    window.RAVEN = window.RAVEN || {};

    RAVEN.namespace("RAVEN.ExecuteTest");

    function getTestRunId() {
        return $('#test-run-id').val();
    }
    
    function getReadonly() {
    	return $("#test-run-readonly").val() == 'true';
    }

    /******************************
    /*********** MODELS ***********
    /******************************/

    var RequirementStatusModel = Backbone.Model.extend({
        urlRoot: function() {
            return AJS.contextPath() + '/rest/raven/1.0/testrun/' + getTestRunId() + '/requirements';
        },
        defaults: function() {
            return {}
        }
    });

    var RequirementStatusCollection = Backbone.Collection.extend({
        model: RequirementStatusModel,
        url: function() {
            return AJS.contextPath() + '/rest/raven/1.0/testrun/' + getTestRunId() + '/requirements';
        }
    });

    /******************************
    /************ VIEWS ***********
    /******************************/

    RAVEN.ExecuteTest.RequirementStatusesView = Backbone.View.extend({
        collection: new RequirementStatusCollection(),
        el: '#execution-requirement-status-content',
        activityStreamView: null,
        isStatusEditable: false,

        currentRequirementStatusView: null,
        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be 
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.req.status.raven.execute .requirement-status .transition' : '_onRequirementStatusClick'
        },

        initialize: function(options) {
            var self = this;

            if (options) {
                _.extend(this, _.pick(options, "activityStreamView", "isStatusEditable"));
            }

            this._delegateUnscopedEvents();

            this.collection.on('reset', this.render, this);

            this.$el.block();
            this.collection.fetch({
                success: function() {
                    self.render();
                    self.$el.unblock(); // unmask on complete was being unmasked before render
                },
                error: function(model, response) {
                    AJS.messages.error(self.$el, {
                        title: AJS.I18n.getText('execute-test-action.requirement-status.fetch-error'),
                        body: response.responseText,
                        closeable: false
                    });
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            var self = this;

            // clear view
            this.$el.empty();

            // empty table
            if (!this.collection || this.collection.length == 0) {
                this.$el.append('<span>None</span>');
            } else {
                // render table skeleton
                var $reqStatusList = $('<ul class="links-list requirement-status-list"></ul>');
                
                this.collection.each(function(result) {
                    $reqStatusList.append((new RAVEN.ExecuteTest.RequirementStatusView({
                        model: result,
                        parentView: self
                    })).render().$el);
                });

                this.$el.append($reqStatusList);

                this.applyTipsy();

                if (this.isStatusEditable === true) {
                    $('.requirement-status-bar').removeAttr('aria-disabled');
                } else {
                    $('.requirement-status-bar').attr('aria-disabled', true);
                }
            }

            return this;
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.req.status.raven.execute');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

         /**
         * Event handler for the click event any requirement status option.
         */
        _onRequirementStatusClick : function(e) {
            var self = this;

             if (this.currentRequirementStatusView) {
                 var view = this.currentRequirementStatusView;

                 var status = $(e.target).data('status');
                 var modelStatus = view.model.get('status');

                 if (modelStatus !== status) {
                     this.$el.block();
                     view.model.save({
                         'status': status
                     }, {
                         success: function (model, response) {
                             view.render();
                             self.$el.unblock();

                             if (self.activityStreamView) {
                                 self.activityStreamView.refresh();
                             }

                             self.applyTipsy();
                         },
                         error: function () {
                             self.$el.unblock();
                         }
                     });
                 }
             }

            e.preventDefault();
            e.stopPropagation();
        },

        applyTipsy: function() {
            $(".tipsy.tipsy-n").remove();
            $(".status-tipsy").tipsy();
        }

    });

    RAVEN.ExecuteTest.RequirementStatusView = Backbone.View.extend({
        tagName: 'li',
        styleClass: 'requirement-status-result',
        parentView: null,
        requirementStatuses: null,

        events: {
            'mouseover  .requirement-status': '_onStatusMouseover'
        },

        initialize: function(options) {

            if (options) {
                _.extend(this, _.pick(options, "parentView"));
            }

            this.requirementStatuses = JSON.parse($('#requirement-statuses').val());

            this.render();
        },

        render: function() {
            this.$el.empty();

            // render table skeleton
            var itemHtml = RAVEN.ExecuteTest.renderRequirementStatusResultItem({
                'item': this.model.attributes,
                'requirementStatuses': this.requirementStatuses,
                'contextPath': AJS.contextPath(),
                readonly: getReadonly()
            });
            this.$el.html(itemHtml);
            this.$el.addClass(this.styleClass);

            return this;
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _onStatusMouseover: function() {
            this.parentView.currentRequirementStatusView = this;
        }
    });

    $(document).on('click.req.status.raven', 'a.raven-requirement-status-acc-option', function() {
        return false;
    });



})(AJS.$);