(function($) {
    $(document).ready(function () {
        var PARAM_COVERAGE_ENABLED = 'coverageEnabled';
        var PARAM_COVERAGE_DISABLED = 'coverageDisabled';
        var PARAM_SHOW_CONFIG = 'showXrayConfig';
        var PARAM_NEEDS_REINDEX = 'needsReindex';
        var PARAM_TYPE_ADDED = 'XrayTypeAdded';

        var currentProject = null;

        if (JIRA.ProjectConfig && JIRA.ProjectConfig.getKey) {
            currentProject = JIRA.ProjectConfig.getKey();
        } else {
            return; // disable if there is no project context
        }

        var pageURL = decodeURIComponent(window.location.search.substring(1));
        var urlVariables = pageURL.split('&');

        for (var i = 0; i < urlVariables.length; i++) {
            var parameter = urlVariables[i].split('=');

            if (parameter[0] === PARAM_COVERAGE_ENABLED) {
                notifyEnablingResult(
                    urlVariables.filter(function(variable){
                        return variable === PARAM_SHOW_CONFIG + '=true';
                    }).length > 0
                );
            } else if (parameter[0] === PARAM_COVERAGE_DISABLED) {
                notifyDisablingResult(
                    urlVariables.filter(function(variable){
                        return variable === PARAM_SHOW_CONFIG + '=true';
                    }).length > 0
                );
            } else if (parameter[0] === PARAM_NEEDS_REINDEX) {
                notifyReindex();
            } else if (parameter[0] === PARAM_TYPE_ADDED) {
                notifyXrayIssueTypesAdded();
            }
        }

        function notifyEnablingResult(showXrayConfig){
            var self = this;
            $('#project-config-panel-summary').before('<div class="message"></div>');

            self.configMessage = showXrayConfig ?
                AJS.I18n.getText('raven-admin-config.requirement-project-dialog.message.success.show-configuration'
                                            ,'/secure/admin/views/XrayRequirementProjectsConfiguration.jspa') : "";

            if(parameter[1] === "true") {
                AJS.messages.success('.message', {
                    body: AJS.I18n.getText('raven-admin-config.enable-requirement-project-dialog.message.success', currentProject)+self.configMessage
                });
            } else if (parameter[1] === "false") {
                AJS.messages.error('.message', {
                    body: AJS.I18n.getText('raven-admin-config.enable-requirement-project-dialog.message.error', currentProject)
                });
            }
        }

        function notifyDisablingResult(showXrayConfig){
            var self = this;
            $('#project-config-panel-summary').before('<div class="xray-coverage-config-message"></div>');

            self.configMessage = showXrayConfig ?
                AJS.I18n.getText('raven-admin-config.requirement-project-dialog.message.success.show-configuration'
                    ,'/secure/admin/views/XrayRequirementProjectsConfiguration.jspa') : "";

            if(parameter[1] === "true") {
                AJS.messages.success('.xray-coverage-config-message', {
                    body: AJS.I18n.getText('raven-admin-config.disable-requirement-project-dialog.message.success', currentProject)+self.configMessage
                });
            } else if (parameter[1] === "false") {
                AJS.messages.error('.xray-coverage-config-message', {
                    body: AJS.I18n.getText('raven-admin-config.disable-requirement-project-dialog.message.error', currentProject)
                });
            }
        }

        function notifyReindex(){
            $('#project-config-panel-summary').before('<div class="reindex-message"></div>');
            if(parameter[1] === "true") {
                AJS.messages.warning('.reindex-message', {
                    body: AJS.I18n.getText('raven-admin-config.message.reindex-warning')
                });
            }
        }

        function notifyXrayIssueTypesAdded(){
            $('#project-config-panel-summary').before('<div class="issue-types-added-message"></div>');
            if(parameter[1] === "true") {
                AJS.messages.success('.issue-types-added-message', {
                    body: AJS.I18n.getText('raven-admin-config.project.xray-tests-projects-enable.message.success')
                });
            }else if(parameter[1] === "false"){
                AJS.messages.error('.xray-coverage-config-message', {
                    body: AJS.I18n.getText('raven-admin-config.project.xray-tests-projects-enable.message.error', currentProject)
                });
            }
        }

    });
})(AJS.$);
