(function($) {

    RAVEN.namespace("RAVEN.Settings");

    RAVEN.Settings.ProjectsView = Backbone.View.extend({
        el: '#raven-requirement-projects',

        /**
         * DataTable Instance
         */
        dataTable: null,

        bulkEditView: null,

        availableProjects: [],

        requirementProjectSelect: null,

        events: {
            'click #raven-add-projects-link': '_addProjects',
            'click .delete-requirement-project': '_onProjectDeleteClick',
            'click #requirement-project-clear-filter': '_onClearFilter',
            'submit #requirement-project-form': '_onFormSubmit'
        },
        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.requirementProject .bulk-remove-command': '_onBulkProjectDeleteClick',
            'click #dialog-submit-button': '_saveProjects',
            'click #dialog-submit-all-button': '_saveAllProjects',
            'keyup.raven.requirementProject input#requirement-project-search-input': '_onSearchInputKeyUp',
            'click #raven-search-submit': '_onSearchInputKeyUp',
            'click #dialog-close-button': '_closeDialog'
        },

        initialize: function() {
            AJS.messages.setup();

            this._delegateUnscopedEvents();

            if (this.$el.closest(document.documentElement).length > 0) {
                this.render();
            }
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.requirementProject');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _onSearchInputKeyUp: function(event) {

            var instance = $(event.target);
            if (!event.keyCode || (event.keyCode != null && event.keyCode === 13)) {
                event.preventDefault();

                this.dataTable.fnDraw(false);
                return false;
            }
            event.preventDefault();
            event.stopPropagation();

        },
        render: function() {
            // create the dataTable
            this._createTestsDataTable();

            // render bulk mode button
            if (!this.bulkEditView) {
                this._renderBulkModeButton();
            }

            this.$el.show();

            return this;
        },


        formSubmitListener: function() {
            prepareAvailableIds();
            prepareRequirementIds();

            return true;
        },

        /**
         * Delete test click handler
         */
        _onProjectDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var $row = $(e.target).closest('tr');
            var projectData = this.dataTable.fnGetData($row[0]);

            this._confirmationDeleteProject([projectData.pid], this._deleteProject);
        },

        /**
         * Delete Bulk test click handler
         */
        _onBulkProjectDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var rowIds = this.bulkEditView.getSelectedRowIds();

            if (rowIds && rowIds.length > 0) {
                this._confirmationDeleteProject(rowIds, this._deleteProject, true);
            }
        },

        prepareAvailableIds: function() {
            var ids = $('#available-projects li').map(function() {
                return $(this).data('id');
            }).get();

            $('#availableProjectIds').val(JSON.stringify(ids));
        },

        prepareRequirementIds: function() {
            var arrayIds = $("#select2-requirements-project").val().split(',');
            $('#requirementProjectIds').val(JSON.stringify(arrayIds));
        },

        formatSelection: function(o) {
            if (!o.id) {
                return o.text;
            } else {
                var span = '<a href="' + AJS.contextPath() + '/projects/' + o.key + '"><div style="padding: 4px;"><img style="width: 20px;height: 20px;padding: 0px;" src="' + o.icon + '" width="16" height="16"><div style="float: right;padding: 4px 4px 4px 4px;color:#3b73af">' + o.text + '</div></div></a>';
                return span;
            }
        },

        formatResult: function(o) {
            if (!o.id) {
                return o.text;
            } else {
                var span = '<div style="padding: 2px;"><img style="float: left;width: 20px;height: 20px;padding: 4px;" src="' + o.icon + '" width="16" height="16"><div style="float: left;padding: 4px;">' + o.text + '</div></div>';
                return span;
            }
        },

        _renderBulkModeButton: function() {
            var $wrapper = $('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.bulkEditView = new RAVEN.DataTable.BulkEditView({
                el: $wrapper
            });
        },

        _createTestsDataTable: function() {

            if (this.dataTable) {
                this.dataTable.fnDestroy();
            }

            var path = AJS.contextPath() + '/rest/raven/1.0/preferences/requirementProjects';
            var self = this;
            this.dataTable = $('#project-entries-table').xrayDataTable({
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.infoempty'),
                    "sEmptyTable": AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.tableempty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "sDom": "l<t>ip",
                "bServerSide": true,
                "sAjaxSource": path,
                "sAjaxDataProp": "entries",
                "bProcessing": false,
                "bAutoWidth": false,
                "bDefaultFontSize": true,
                "sPaginationType": "full_numbers",
                "bSort": true,
                "aaSorting": [],
                "fnServerData": function(sSource, aoData, fnCallback, oSettings) {
                    self._fnServerData(sSource, aoData, fnCallback, oSettings);
                    if (self.bulkEditView) {
                        self.bulkEditView.setCheckBoxRowsSelectionValue(false);
                    }
                },
                "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    nRow.id = aData["pid"];
                    return nRow;
                },
                "aoColumnDefs": this._createDataTableColumnDefs(),
                "fnDrawCallback": function(oSettings) {

                    if (self.bulkEditView) {
                        self.bulkEditView.showHideBulkActions();
                    }
                }
            });

            // self._initSortableColumns();

        },

        /**
         * Creates the DataTable column def descriptors
         */
        _createDataTableColumnDefs: function() {
            var self = this;

            var bulkToogleColumnTitle = null;
            var bulkActionColumnTitle = null;

            bulkToogleColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkSelectAllInput();

            var bulkActions = '<a href="javascript:void(0);" data-key="bulkKey" class="bulk-remove-command active aui-dropdown2-active" title="' + AJS.I18n.getText('common.action.remove') + '">' +
                '<span class="icon icon-delete menu-icon"></span>' + AJS.I18n.getText('common.action.remove') + '</a>';
            bulkActionColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkActionTrigger({ id: "projects-bulk-action", actions: bulkActions });

            return [{
                "mData": null,
                "sTitle": bulkToogleColumnTitle,
                "mDataProp": null,
                "bSortable": false,
                "sWidth": "20px",
                "sDefaultContent": "<input type='checkbox' ></input>",
                "bSortable": false,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return RAVEN.DataTable.BulkEdit.renderBulkSelectInput({ id: row["alias"] });
                },
                "aTargets": [0]
            }, {
                "mData": null,
                "sWidth": "24px",
                "bSortable": false,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<img style="float: left;width: 20px;height: 20px;padding: 4px;" src="' + AJS.contextPath() + "/secure/projectavatar?size=xsmall&pid=" + row.pid + "&avatarId=" + row.avatarId + '" width="16" height="16">';
                },
                "aTargets": [1]
            }, {
                "mData": "alias",
                "sWidth": "114px",
                "bSortable": true,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<a class="link-title" style="padding-left: 2px" href="' + AJS.contextPath() + '/projects/' + row.alias + '">' + data + '</a>';
                },
                "aTargets": [2]
            }, {
                "mData": "type",
                "bSortable": true,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<span class="link-title" style="padding-left: 2px" >' + data + '</span>';
                },
                "aTargets": [4]
            }, {
                "mData": "name",
                "bSortable": true,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<a class="link-title" style="padding-left: 2px" href="' + AJS.contextPath() + '/projects/' + row.alias + '">' + data + '</a>';
                },
                "aTargets": [3]
            }, {
                "mData": null,
                "bSortable": false,
                "sTitle": bulkActionColumnTitle,
                "sClass": "project-lastcol-actions",
                "mRender": function(data, type, row) {
                    return "<a class='aui-button aui-button-link delete-requirement-project' id='test_row_'" + new Date().getTime() + ">" + AJS.I18n.getText('common.action.delete') + "</a>";
                },
                "aTargets": [5]
            }];
        },

        _saveAllProjects: function(e) {
            e.preventDefault();
            var self = this;
            var dialog = new JIRA.FormDialog({
                id: 'add-allprojects',
                trigger: '#reset-test-run',
                content: function(callback) {
                    var title = AJS.I18n.getText('raven-admin-config.project.xray-requirement-projects');
                    var message = AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.add-all.label');
                    var innerHtml = RAVEN.Dialog.confirmationDialog({
                        title: title,
                        message: message,
                        severity: 'info'
                    });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    dialog.hide();
                    e.preventDefault();
                    self._saveAllProjectSubmitHandler();
                },
                autoClose: true
            });

            dialog.show();
        },

        _saveAllProjectSubmitHandler: function() {
            var self = this;
            $.ajax({
                type: "POST",
                url: AJS.contextPath() + '/rest/raven/1.0/preferences/allAsRequirementProjects',
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    AJS.messages.error('#raven-config-message-bar', {
                        title: "Error",
                        body: errorThrown,
                        closeable: true
                    });
                },
                success: function(data, textStatus, jqXHR) {

                    self.dataTable.fnDraw(false);

                    self._clearMessage();
                    AJS.messages.success('#raven-config-message-bar', {
                        title: "Success",
                        body: AJS.I18n.getText('raven-admin-config.requirement-project.message.success'),
                        closeable: true
                    });
                    AJS.messages.warning('#raven-config-message-bar', {
                        body: AJS.I18n.getText('raven-admin-config.message.reindex-warning'),
                        closeable: true
                    });
                },
                complete: function() {

                }
            });

            //hide modal
            $("#project-dialog").css("display", "none");
        },

        _saveProjects: function(e) {
            e.preventDefault();
            var self = this;
            var ids = $("#select2-requirements-project").val();

            if (ids !== null && ids.length > 0) {
                var arrayIds = ids.split(',');
                var data = JSON.stringify(arrayIds);
                $.ajax({
                    type: "POST",
                    url: AJS.contextPath() + '/rest/raven/1.0/preferences/requirementProjects',
                    processData: true,
                    async: true,
                    data: data,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        AJS.messages.error('#raven-config-message-bar', {
                            title: "Error",
                            body: errorThrown,
                            closeable: true
                        });
                    },
                    success: function(data, textStatus, jqXHR) {

                        self.dataTable.fnDraw(false);

                        self._clearMessage();
                        AJS.messages.success('#raven-config-message-bar', {
                            title: "Success",
                            body: AJS.I18n.getText('raven-admin-config.requirement-project.message.success'),
                            closeable: true
                        });
                        AJS.messages.warning('#raven-config-message-bar', {
                            body: AJS.I18n.getText('raven-admin-config.message.reindex-warning'),
                            closeable: true
                        });
                    },
                    complete: function() {

                    }
                });
            }
            //hide modal
            $("#project-dialog").css("display", "none");
        },

        _closeDialog: function(e) {
            e.preventDefault();
            //hide modal
            $("#project-dialog").css("display", "none");
        },

        _onFormSubmit: function(e) {
            e.preventDefault();
            return false;
        },

        _onClearFilter: function(e) {
            $("#requirement-project-search-input").val('');

            this.dataTable.fnDraw(false);
            return false;
        },

        _addProjects: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: AJS.contextPath() + '/rest/raven/1.0/preferences/nonRequirementProjects',
                processData: false,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    AJS.messages.error('#raven-config-message-bar', {
                        title: "Error",
                        body: errorThrown,
                        closeable: true
                    });
                },
                success: function(data, textStatus, jqXHR) {
                    this.availableProjects = data;
                    var html = RAVEN.Settings.selectProject();
                    $(".project-choice-container").html(html);

                    $("#project-dialog").css("display", "block");

                    this.requirementProjectSelect = $("#select2-requirements-project").auiSelect2({
                        data: $.map(this.availableProjects, function(obj) {
                            obj.id = obj.id;
                            obj.text = obj.name;
                            obj.icon = obj.avatarUrls['16x16'];
                            obj.url = obj.self;
                            obj.key = obj.key;
                            return obj;
                        }),
                        multiple: true,
                        formatResult: self.formatResult,
                        formatSelection: self.formatSelection
                    });
                }
            });
        },

        _fnServerData: function(sSource, aoData, fnCallback, oSettings) {
            var self = this;

            if (this.filters) {
                aoData = aoData.concat(this.filters);
            }

            var versions = $('#requirement-project-search-input').val();
            if (versions) {
                aoData.push({ "name": "filter_text", "value": versions });
            }

            $('#raven-requirement-projects').block();
            oSettings.jqXHR = $.ajax({
                "dataType": 'json',
                "type": "GET",
                "url": sSource,
                "data": aoData,
                "error": function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    AJS.messages.error('#raven-config-message-bar', {
                        title: "Error",
                        body: errorThrown,
                        closeable: true
                    });
                },
                "complete": function() {
                    $('#raven-requirement-projects').unblock();
                },
                "success": fnCallback
            });
        },

        _confirmationDeleteProject: function(projectIds, yesFunction, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                id: 'delete-project-dialog',
                content: function(callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.delete.confirm.title');
                        message = AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.bulk.delete.confirm.message');
                    } else {
                        title = AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.delete.confirm.title', projectIds[0]);
                        message = AJS.I18n.getText('raven-admin-config.project.xray-requirement-project.delete.confirm.message');
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({ title: title, message: message, severity: severity });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(projectIds);
                    dialog.hide();
                },
                autoClose: true
            });
            dialog.show();
        },

        /**
         * Build query param
         */
        _buildQueryParamsFromSelectedRows: function(selectedIds) {
            var projectKeys = "";
            var i;
            for (i = 0; i < selectedIds.length; ++i) {
                if (projectKeys.length > 0) {
                    projectKeys += "&";
                }
                projectKeys += "projectKeys=" + selectedIds[i];
            }

            return projectKeys;
        },

        _deleteProjectsCallback: function(projectIds, data) {
            var self = this;
            if (projectIds && projectIds.length > 0) {
                var allData = this.dataTable.fnGetData();
                if (allData.length >= 1) {
                    _.each(_.initial(projectIds), function(testKey) {
                        self.dataTable.fnDeleteRow(testKey, null, false);
                    });
                    this.dataTable.fnDeleteRow(_.last(projectIds));
                } else {
                    // don't refresh the table if the last element is removed because it will request more data from server
                    this.dataTable.fnDeleteRow(_.last(projectIds), null, false);
                }
            }
        },

        _deleteProject: function(projectKeys) {
            var self = this;

            $('#raven-requirement-projects').block();

            //if we have records to delete
            if (projectKeys && projectKeys.length > 0) {
                //Build query param
                var projectKeysParam = this._buildQueryParamsFromSelectedRows(projectKeys);

                $.ajax({
                    type: "DELETE",
                    url: AJS.contextPath() + '/rest/raven/1.0/preferences/requirementProjects/delete?' + projectKeysParam,
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        if (projectKeys.length == 1) {
                            self.message(AJS.I18n.getText('view-issue-testexec-tests-section.delete.error', projectKeys[0]), errorThrown);
                        } else {
                            self.message(AJS.I18n.getText('view-issue-testexec-tests-section.bulk.delete.error'), errorThrown);
                        }
                        $('#raven-requirement-projects').unblock();
                    },
                    success: function(data, textStatus, jqXHR) {
                        $('#raven-requirement-projects').unblock();
                        self._deleteProjectsCallback(projectKeys, data);

                        AJS.messages.warning('#raven-config-message-bar', {
                            body: AJS.I18n.getText('raven-admin-config.message.reindex-warning'),
                            closeable: true
                        });
                    },
                    complete: function() {}
                });
            }
        },

        _clearMessage: function() {
            $('#raven-config-message-bar').empty();
        },
    });
})(AJS.$);
