(function ($) {

	window.RAVEN = window.RAVEN || {};
	
	RAVEN.Dialogs = RAVEN.Dialogs || {};

	RAVEN.Dialogs.createSimpleTestExecDialog = function (labels, summary , projectKey, assigneeKey, fixVersion, customField, submit, cancel) {
			var self = this; 
			var project;
			var assignee;
			this.remoteJiraSearchDialog = null;

            var revisionDeferred = $.Deferred();
            hasRevision(projectKey,revisionDeferred);

            $.when(
                revisionDeferred,
                $.ajax({
                    type: "GET",
                    url: contextPath +'/rest/api/2/project/' + projectKey,
                    contentType: "application/json",
                    success: function(data) {
                        project = data;
                    }
                })
                ,
                $.ajax({
                    type: "GET",
                    url: contextPath + '/rest/api/2/user?key=' + assigneeKey,
                    contentType: "application/json",
                    success: function (data) {
                        assignee = data;
                    }
                })
			).then( function (hasRevision) {
                var createTestExecDialog = new JIRA.FormDialog({
                    id: 'raven-create-testExec-dialog',
                    width: 680,
                    height: "250px",
                    stacked: true,
                    content: function (render) {
                        var content = RAVEN.Dialogs.createTestExec({
                            dialogMode: true,
                            title: labels.title,
                            summary: summary,
                            assignee: assignee,
                            project: project,
                            versions: fixVersion,
                            hasRevision: hasRevision});
                        var $dialogWrapper = $(content);
                        render($dialogWrapper);
                    },
                    submitHandler: function (e) {
                        e.preventDefault();

                        var numberFixVersions = 0;
                        if(getFixVersions() != null) {
                            numberFixVersions = getFixVersions().length;
                        }
                        var numberSelectedVersions = $("#fix-version-filter-multi-select button .value-text").length;
                        if(numberFixVersions > numberSelectedVersions) {
                            var invalidVersion = getFixVersions()[numberFixVersions-1];
                            $(".fix-version-error").remove();
                            $("#fix-version-filter-multi-select").after("<div class='fix-version-error' style='color:red'>" + AJS.I18n.getText('view-issue-testplan-section.create-testexec.fix-version.error', invalidVersion)+"</div>");
                            $("#raven-create-testExec-dialog").find("span.icon.throbber.loading").remove();
                            $("#raven-create-testExec-dialog").find("#issue-selector-form-submit").removeAttr("disabled");
                        } else {
                            submit(getSelectedProjectKey(), getSummary(), getAssignee(), getCanRedirectToExecute(), getFixVersions(), getEnvironment(), getRevision());
                            createTestExecDialog.hide();
                        }
                    },
                    autoClose : true
                });

                createTestExecDialog.show();
                createProjectPicker($('#projects-filter-te'));
                //createAssigneePicker($('#assignee-filter'),projectKey);
                createFixVersionPicker($('#fix-version-filter'),fixVersion,projectKey);
                createTestEnvironmentPicker($('#test-environment-filter'), customField);

            });
	}
	
	RAVEN.Dialogs.createSimpleTestExecDialog.prototype = Object.create(RAVEN.Dialogs.createSimpleTestExecDialog.prototype);
	
	function getSelectedProjectKey(){
		return AJS.$('#projects-filter-te').selected().val()[0];
	}
	
	function getSummary(){
		return AJS.$('#create-testexec-summary').val();
	}
	
	function getAssignee(){
		return AJS.$('#assignee-filter').val()[0];
	}
	
	function getCanRedirectToExecute(){
		return AJS.$("#can-redirect-execute-page").is(':checked');
	}
	
    function getFixVersions() {
    	return AJS.$("#fix-version-filter").val();
    }
    
    function getEnvironment() {
    	return AJS.$("#test-environment-filter").val();
    }

    function getRevision() {
        return AJS.$("#revision-field").val();
    }
	
    function createProjectPicker($selectField) {
        var self = this;
        projectFilter = new AJS.SingleSelect({
            element: $selectField,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + '/rest/api/2/project',
                formatResponse: function (data) {
                    var optgroup = new AJS.GroupDescriptor({
                        weight: 0,
                        id: "all-projects-opt-group",
                        replace: true,
                        label: "All Projects"
                    });
                    
                    $(data).each(function(i, proj) {
                        var item = new AJS.ItemDescriptor({
                            value: proj.key,
                            label: proj.name,
                            icon: proj.avatarUrls['16x16'],
                            allowDuplicate: false,
                            selected: null
                        });
                        optgroup.addItem(item);
                    });
                    return [optgroup];
                }
            }
        });
        $('#projects-filter-te').off('change.raven.test.project');
        $('#projects-filter-te').on('change.raven.test.project',function(){
            var ele = $("#revision-field-group");

            $("#raven-create-testExec-dialog").block();
            var revisionDeferred = $.Deferred();
            hasRevision(getSelectedProjectKey(), revisionDeferred);
            $.when(revisionDeferred).then(function(rev) {
                if(rev) {
                    ele.show();
                } else {
                    ele.hide();
                }
            });
            $("#raven-create-testExec-dialog").unblock();
        });
    }
    
    function createAssigneePicker($selectField, projectKey) {
        assigneeFilter = new AJS.SingleSelect({
            element: $selectField,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + '/rest/api/2/user/assignable/search?username=&project=' + projectKey + '&maxResults=1000&startAt=0',
                formatResponse: function (data) {
                	var suggestionsOptGroup = new AJS.GroupDescriptor({
                		weight: 0,
                        id: "suggestions-assignee-opt-group",
                        replace: true,
                        label: "Suggestions"
                	});
                	
                	var item = new AJS.ItemDescriptor({
                        value: "",
                        label: AJS.I18n.getText('common.label.unassigned'),
                        html:  "<div><span>"+AJS.I18n.getText('common.label.unassigned')+"</span></div>",
                        icon: contextPath + "/secure/useravatar?size=xsmall&avatarId=10123",
                        allowDuplicate: false,
                        selected: null
                    });
                	suggestionsOptGroup.addItem(item);
                	
                	
                    var optgroup = new AJS.GroupDescriptor({
                        weight: 1,
                        id: "all-assignee-opt-group",
                        replace: true,
                        label: "All Users"
                    });
                    
                    $(data).each(function(i, assignee) {
                        var item = new AJS.ItemDescriptor({
                            value: assignee.key,
                            label: assignee.displayName,
                            html:  "<div><span>" + assignee.displayName + " - " + assignee.emailAddress + "</span></div>",
                            icon: assignee.avatarUrls['16x16'],
                            allowDuplicate: false,
                            selected: null
                        });
                        optgroup.addItem(item);
                    });
                    return [suggestionsOptGroup,optgroup];
                }
            }
        });
        AJS.$('#assignee-filter-single-select').addClass("long-field");
    }
    
    function createFixVersionPicker($selectField, fixversions, project) {
    	new RAVEN.VersionPicker({
			element: $selectField,
			loadOnInit: true,
			errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.versions.error"),
			maxInlineResultsDisplayed: 15,
			projectKey: project
		});
    }
    
    function createTestEnvironmentPicker($selectField, customField) {
        projectFilter = new AJS.LabelPicker({
            element: $selectField,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + '/rest/api/1.0/labels/suggest?customFieldId='+customField,
                formatResponse: function (data) {
                    var optgroup = new AJS.GroupDescriptor({
                        weight: 0,
                        id: "suggestions-opt-group",
                        replace: true,
                        label: "Suggestions"
                    });
                    console.log(data)
                    console.log(data.suggestions)
                    $(data.suggestions).each(function(i, env) {
                        var item = new AJS.ItemDescriptor({
                            value: env.label,
                            label: env.label,
                            allowDuplicate: false,
                            selected: null
                        });
                        optgroup.addItem(item);
                    });
                    return [optgroup];
                }
            }
        });
    }

    function hasRevision(projectKey, deferred) {
        var hasRevision = false;
        $.ajax({
            type: "GET",
            url: contextPath +'/rest/raven/1.0/customField/isInCreateScreen?project=' + projectKey + '&field=' + $("#raven-testexec-test-revisionId").val(),
            contentType: "application/json",
            success: function(data) {
                hasRevision = data;
                deferred.resolve(hasRevision);
            }
        });
    }

})(AJS.$);