(function($) {

    window.RAVEN = window.RAVEN || {};

    RAVEN.Dialogs = RAVEN.Dialogs || {};

    /*
     * Constants
     */
    var SIMPLE_MODE = 0;
    var ADVANCED_MODE = 1;

    var SELECT_PANEL_ID = 0;
    var SEARCH_PANEL_ID = 1;

    var SELECT = "1";
    var SEARCH = "2";

    var ISSUE_SELECTOR_REQUIRED = 1;

    RAVEN.Dialogs.FILTER_TESTTYPE = 'testtype';
    RAVEN.Dialogs.FILTER_PROJECT = 'project';
    RAVEN.Dialogs.FILTER_FOLDER = 'folder';
    RAVEN.Dialogs.FILTER_ASSIGNEE = 'assignee';

    var selectedValues = [];

    /*
     * Utility functions for the selected values
     */
    function resetSelectedValues() {
        selectedValues = [];
    }

    function removeFromSelectedValues(input) {
        var index = selectedValues.indexOf(input.name);
        if (index > -1) {
            selectedValues.splice(index, 1);
        }
    }

    function addToSelectedValues(input) {
        var index = selectedValues.indexOf(input.name);
        if (index < 0) {
            selectedValues.push(input.name);
        }
    }

    function checkIfAllSelected() {
        var entries = $("#raven-search-result-table tbody tr").find('input[type="checkbox"]');
        var countChecked = 0;
        for (var i = 0; i < entries.length; i++) {
            if ($(entries[i]).is(':checked')) {
                countChecked++;
            }
        }
        return entries.length === countChecked;
    }

    RAVEN.Dialogs.getDefaultTestLabels = function(addToItem) {
        var labels = {};
        labels.label = AJS.I18n.getText('dialogs.issueselector.test.label');
        labels.title = AJS.I18n.getText('dialogs.issueselector.test.title', addToItem);
        labels.description = AJS.I18n.getText('dialogs.issueselector.test.description-generic');
        labels.enter = AJS.I18n.getText('dialogs.issueselector.test.enter');

        return labels;
    };

    /*
     * Issue Selector base prototype
     */
    RAVEN.Dialogs.IssueSelectorBase = function(mode, type, issueKey, labels, appLinks, redirectOption, projectKey) {
        this.type = type;
        this.issueKey = issueKey;
        this.labels = labels;

        this.mode = mode;
        this.appLinks = appLinks;

        this.redirectOption = redirectOption;
        this.projectKey = projectKey;
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.initIssuePicker = function(issueSelectorRequired) {
        var enter = '';
        if (this.labels && this.labels.enter) {
            enter = this.labels.enter;
        } else {
            enter = AJS.I18n.getText('dialogs.issueselector.default.enter');
        }

        new JIRA.IssuePicker({
            element: $('#raven-field-issuepicker'),
            userEnteredOptionsMsg: enter,
            uppercaseUserEnteredOnSelect: true
        });

        if (issueSelectorRequired == ISSUE_SELECTOR_REQUIRED) {
            new AJS.SingleSelect({
                element: $('#raven-field-singleissuepicker'),
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/1.0/issues/picker',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-parent-issues-opt-group",
                            replace: true,
                            label: data.sections[0].sub
                        });

                        $(data.sections[0].issues).each(function(i, issue) {
                            var item = new AJS.ItemDescriptor({
                                value: issue.key,
                                label: issue.key + ' - ' + issue.summary,
                                icon: contextPath + issue.img,
                                allowDuplicate: false,
                                selected: 'selected'
                            });
                            optgroup.addItem(item);
                        });
                        return [optgroup];
                    }
                }
            });
        }

        // resize picker div
        $('#issue-selector-dialog').find('.jira-issue-picker').css("width", 350);

        $('#issue-selector-dialog textarea').focus();
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.getIssueSelectorContent = function() {
        var title, label, description;
        if (this.labels && this.labels.title) {
            title = this.labels.title;
        } else {
            title = AJS.I18n.getText('dialogs.issueselector.default.title', this.issueKey);
        }

        if (this.labels && this.labels.label) {
            label = this.labels.label;
        } else {
            label = AJS.I18n.getText('dialogs.issueselector.default.label', this.issueKey);
        }

        if (this.labels && this.labels.description) {
            description = this.labels.description;
        } else {
            description = AJS.I18n.getText('dialogs.issueselector.default.description');
        }

        return RAVEN.Dialogs.issueSelectorDialog({
            dialogMode: this.mode === SIMPLE_MODE,
            issueKey: this.issueKey,
            issueType: this.type,
            jql: this.jql,
            title: title,
            label: label,
            description: description,
            contextPath: contextPath,
            appLinks: this.appLinks,
            redirectOption: this.redirectOption
        });
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.getParentIssueSelectorContent = function() {

        var title = AJS.I18n.getText('dialogs.issueselector.testexec.subtask-defect.title');

        var label = AJS.I18n.getText('dialogs.issueselector.testexec.subtask-defect.label');

        var description = AJS.I18n.getText('dialogs.issueselector.testexec.subtask-defect.description');

        return RAVEN.Dialogs.parentIssueSelectorDialog({
            dialogMode: this.mode === SIMPLE_MODE,
            title: title,
            label: label,
            description: description,
            appLinks: this.appLinks,
            redirectOption: this.redirectOption
        });
    }

    /**
     * Search Issue Dialog functions. Copied from the jira v5.2.7 remote link plugin and adapted to raven's issue selector dialog.
     */
    RAVEN.Dialogs.IssueSelectorBase.prototype.initSearchDialog = function($dialog) {
        // Gather the JQL auto complete data
        var appId = $("#jira-app-link").val();
        var autoCompletePromise = getJqlAutoCompleteData(appId);

        var self = this;

        // catch form submission
        $("#raven-issue-simple-search-form", $dialog).submit(function(e) {
            self.performSimpleSearch($dialog);

            e.preventDefault();
            return false;
        });

        // cancel JIRA stuff because search button is disabled
        $("#raven-issue-simple-search-form", $dialog).on('before-submit', function(e) {
            e.preventDefault();
            return false;
        });

        // Bind the simple search button
        /*$("#simple-search-panel-button", $dialog).click(function () {
         self.performSimpleSearch($dialog);
         });*/

        // Bind the advanced search button
        $("#advanced-search-panel-button", $dialog).click(function() {
            try {
                advancedSearchButtonClick($dialog, self);
            } catch (err) {
                console.log(err);
            } finally {
                return false;
            }
        });

        $("#simple-search-toggle", $dialog).click(function() {
            $("#raven-issue-simple-search-form", $dialog).show();
            $("#raven-issue-advanced-search-form", $dialog).hide();
            return false;
        });

        $("#advanced-search-toggle", $dialog).click(function() {
            $("#raven-issue-advanced-search-form", $dialog).show();
            $("#raven-issue-simple-search-form", $dialog).hide();

            // Initialise the JQL auto complete once we have the data
            // Ensure that we only initialise it once only
            var $jqlSearchText = $("#jql-search-text");
            if (!$jqlSearchText.attr("jql-initialized")) {
                setAutoCompleteLoadingIconVisible(true, $dialog);
                autoCompletePromise.done(function(smartAjaxResult) {
                    if (smartAjaxResult.successful) {
                        // Enable JQL AutoComplete
                        IssueLinkJQLAutoComplete.initialize({
                            fieldID: "jql-search-text",
                            errorID: "jql-search-error",
                            autoCompleteUrl: getAutoCompleteUrl(appId),
                            autoCompleteData: smartAjaxResult.data,
                            formSubmitFunction: function() {
                                advancedSearchButtonClick($dialog, self);
                            }
                        });
                    } else {
                        setJQLErrorVisible(false, $dialog);
                        setAutoCompleteFailedIconVisible(true, $dialog);
                    }
                    setAutoCompleteLoadingIconVisible(false, $dialog);
                    $jqlSearchText.attr("jql-initialized", 1);
                });
            }
            $jqlSearchText.focus();

            return false;
        });
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.performSimpleSearch = function($dialog) {
        var self = this;
        try {
            $("#search-results-table", $dialog).empty();
            var searchText = $("#raven-issue-search-text", $dialog).val();
            searchText = $.trim(searchText);
            if (self.mode === ADVANCED_MODE || searchText) {
                self.doSimpleSearch(searchText, $dialog);
            } else {
                AJS.messages.info("#search-results-table", {
                    body: AJS.I18n.getText("dialogs.issueselector.search.value.required"),
                    closeable: false
                });
            }
        } catch (err) {
            console.log(err);
        } finally {
            return false;
        }
    }

    function getAutoCompleteUrl(appId) {
        if (appId && appId !== "") {
            // Remote JIRA instance
            return contextPath + "/rest/remoteJiraIssueLink/1/remoteJira/autocomplete?appId=" + appId;
        }
        // Local JIRA instance - will use the default URL in JQLAutoComplete
        return "";
    }

    function advancedSearchButtonClick($dialog, self) {
        $("#search-results-table", $dialog).empty();
        var searchText = $("#jql-search-text", $dialog).val();
        searchText = $.trim(searchText);
        if (searchText) {
            self.doAdvancedSearch(encodeURIComponent(searchText), $dialog);
        } else {
            AJS.messages.info("#search-results-table", {
                body: AJS.I18n.getText("dialogs.issueselector.search.value.required"),
                closeable: false
            });
        }
    }

    function setLoadingIconVisible(visible, $context) {
        $("#raven-issue-search-loading", $context).toggleClass("hidden", !visible);
    }

    function setAutoCompleteLoadingIconVisible(visible, $context) {
        $("#autocomplete-loading", $context).toggleClass("hidden", !visible);
    }

    function setAutoCompleteFailedIconVisible(visible, $context) {
        $("#autocomplete-failed", $context).toggleClass("hidden", !visible);
    }

    function setJQLErrorVisible(visible, $context) {
        $("#jql-search-error", $context).toggleClass("hidden", !visible);
    }

    function getProjectJql(projectKey) {
        if (!projectKey) {
            return null;
        }
        return '(project = "' + projectKey + '")';
    }

    function getIssueTypeJql(issueTypeId) {
        if (!issueTypeId) {
            return null;
        }

        if (issueTypeId.constructor === Array) {
            var query = '(';
            for (var i = 0; i < issueTypeId.length; i++) {
                query += '(issuetype = ' + issueTypeId[i] + ')'
                if (i < issueTypeId.length - 1) {
                    query = query += ' OR ';
                }
            }
            query += ')';
            return query
        }

        return '(issuetype = ' + issueTypeId + ')';
    }

    function getProjectJqlClause() {
        var projKey = $('#project-filter option:selected').attr('value');
        if (projKey) {
            return 'project = "' + projKey + '"';
        }
        return null;
    }

    function getFolderPath() {
        if(this.testRepositoryFolderPicker) {
            return this.testRepositoryFolderPicker.buildJql();
        }
        return null;
    }

    function getVersionJqlClause() {
        var versions = $('#version-filter option:selected');
        if (versions.length > 0) {
            var joinedVersions = versions.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'fixVersion in (' + joinedVersions + ')';
        }
        return null;
    }

    function getFilterJqlClause() {
        var filters = $('#filter-filter option:selected');
        if (filters.length > 0) {
            var joinedFilters = filters.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'filter = ' + joinedFilters;
        }
        return null;
    }

    function getLabelJqlClause() {
        var labels = $('#label-filter option:selected');
        if (labels.length > 0) {
            var joinedLabels = labels.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'labels in (' + joinedLabels + ')';
        }
        return null;
    }

    function getStatusJqlClause() {
        var status = $('#workflow-status-filter option:selected');
        if (status.length > 0) {
            var joinedStatus = status.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'status in (' + joinedStatus + ')';
        }
        return null;
    }

    function getTestTypeJqlClause(testTypeCFMeta) {
        var testTypes = $('#test-type-filter option:selected');
        if (testTypes.length > 0) {
            var joinedTestTypes = testTypes.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'cf[' + testTypeCFMeta.id + '] in (' + joinedTestTypes + ')';
        }
        return null;
    }

    function getTestSetJqlClause() {
        var testSets = $('#testset-filter option:selected');
        if (testSets.length > 0) {
            var joinedTestSets = testSets.map(function() {
                return 'issueKey in testSetTests("' + $(this).val() + '")';
            }).get().join(' or ');

            return '(' + joinedTestSets + ')';
        }
        return null;
    }

    function getPreConditionTypeJqlClause(preConditionTypeCFMeta) {
        var preConditionTypes = $('#precondition-type-filter option:selected');
        if (preConditionTypes.length > 0) {
            var joinedPreConditionTypes = preConditionTypes.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'cf[' + preConditionTypeCFMeta.id + '] in (' + joinedPreConditionTypes + ')';
        }
        return null;
    }

    function getTestSetVersionJqlClause() {
        var testSets = $('#testset-filter option:selected');
        if (testSets.length == 0) {
            var versions = $('#testset-version-filter option:selected');
            if (versions.length > 0) {
                var joinedVersions = versions.map(function() { return '"' + $(this).val() + '"' }).get().join();
                var projKey = $('#project-filter option:selected').attr('value');
                if (!projKey) {
                    projKey = '""';
                }
                return 'key in testsWithTestSetVersion(' + projKey + ', ' + joinedVersions + ')';
            }
        }
        return null;
    }

    function getRequirementJqlClause() {
        var requirements = $('#requirement-filter option:selected');
        if (requirements.length > 0) {
            var joinedRequirements = requirements.map(function() {
                return 'issueKey in requirementTests("' + $(this).val() + '")';
            }).get().join(' or ');

            return '(' + joinedRequirements + ')';
        }
        return null;
    }

    function getRequirementVersionJqlClause() {
        var requirements = $('#requirement-filter option:selected');
        if (requirements.length == 0) {
            var versions = $('#requirement-version-filter option:selected');
            if (versions.length > 0) {
                var joinedVersions = versions.map(function() { return '"' + $(this).val() + '"' }).get().join();
                var projKey = $('#project-filter option:selected').attr('value');
                if (!projKey) {
                    projKey = '""';
                }
                return 'key in testsWithReqVersion(' + projKey + ', ' + joinedVersions + ')';
            }
        }
        return null;
    }

    function getComponentJqlClause() {
        var component = $('#component-filter option:selected');
        if (component.length > 0) {
            var joinedComponents = component.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'component in (' + joinedComponents + ')';
        }
        return null;
    }

    function getTestEnvironmentJqlClause(environmentId) {
        var environments = $('#environment-filter option:selected');
        if (environments.length > 0) {
            var joinedEnvironments = environments.map(function() { return '"' + $(this).val() + '"' }).get().join();
            return 'cf[' + environmentId + '] in (' + joinedEnvironments + ')';
        }
        return null;
    }
    RAVEN.Dialogs.IssueSelectorBase.prototype.doSimpleSearch = function(searchText, $context) {
        setLoadingIconVisible(true, $context);
        var appId = $("#jira-app-link").val();

        var self = this;

        var issueKeyJql = '';
        var plainTextJql = '';

        if (searchText) {
            issueKeyJql = 'key = "' + searchText + '"';
            plainTextJql = '(summary ~ "' + searchText + '" OR description ~ "' + searchText + '" OR comment ~ "' + searchText + '")';
        }

        var jql = '';

        // add issue type constraint
        var issueTypeCond = getIssueTypeJql(this.issueTypeId);
        if (issueTypeCond !== null) {
            jql = issueTypeCond;
        }

        if (this.mode === ADVANCED_MODE) {
            var projectJql = getProjectJqlClause();
            if (projectJql) {
                jql += jql ? ' and ' + projectJql : projectJql;
            }
            var folderPath = getFolderPath();
            if(folderPath) {
                jql += jql ? ' and '  + folderPath : folderPath;
            }
            var labelsJql = getLabelJqlClause();
            if (labelsJql) {
                jql += jql ? ' and ' + labelsJql : labelsJql;
            }
            var versionJql = getVersionJqlClause();
            if (versionJql) {
                jql += jql ? ' and ' + versionJql : versionJql;
            }
            var filterJql = getFilterJqlClause();
            if (filterJql) {
                jql += jql ? ' and ' + filterJql : filterJql;
            }

            var statusJql = getStatusJqlClause();
            if (statusJql) {
                jql += jql ? ' and ' + statusJql : statusJql;
            }

            var componentJql = getComponentJqlClause();
            if (componentJql) {
                jql += jql ? ' and ' + componentJql : componentJql;
            }


            if (this.type === 'TEST') {
                var testTypeJql = getTestTypeJqlClause(self.testTypeCFMeta);
                if (testTypeJql) {
                    jql += jql ? ' and ' + testTypeJql : testTypeJql;
                }
                var testSetJql = getTestSetJqlClause();
                if (testSetJql) {
                    jql += jql ? ' and ' + testSetJql : testSetJql;
                }
                var testSetVersionJql = getTestSetVersionJqlClause();
                if (testSetVersionJql) {
                    jql += jql ? ' and ' + testSetVersionJql : testSetVersionJql;
                }
                var requirementJql = getRequirementJqlClause();
                if (requirementJql) {
                    jql += jql ? ' and ' + requirementJql : requirementJql;
                }
                var requirementVersionJql = getRequirementVersionJqlClause();
                if (requirementVersionJql) {
                    jql += jql ? ' and ' + requirementVersionJql : requirementVersionJql;
                }
            }

            if (this.type === 'PRECONDITION') {
                var preConditionTypeJql = getPreConditionTypeJqlClause(self.testTypeCFMeta);
                if (preConditionTypeJql) {
                    jql += jql ? ' and ' + preConditionTypeJql : preConditionTypeJql;
                }
            }
            if (this.type === 'TESTEXECUTION') {
                var testEnvironmentJql = getTestEnvironmentJqlClause(self.testEnvironmentsCFMeta.id);
                if (testEnvironmentJql) {
                    jql += jql ? ' and ' + testEnvironmentJql : testEnvironmentJql;
                }
            }
        }

		if(self.jql !== null){jql += jql ? ' and ' + self.jql : self.jql;}

        if (jql) {
            if (issueKeyJql) {
                issueKeyJql += issueKeyJql ? ' and ' + jql : jql;
                issueKeyJql = encodeURIComponent(issueKeyJql);
            }
            plainTextJql += plainTextJql ? ' and ' + jql : jql;
            plainTextJql = encodeURIComponent(plainTextJql);
        }
        
		// First, check if search text is an issue key
		// We need to do this because the search will fail if it is not an issue key,
		// even if it is OR'd with a condition that returns results!
        if (this.mode === ADVANCED_MODE) {
            if (issueKeyJql) {
                self.advancedJqlSearch(issueKeyJql).done(function(smartAjaxResult) {
                    if (smartAjaxResult.successful && smartAjaxResult.data) {
                        setLoadingIconVisible(false, $context);
                        self.showResults(smartAjaxResult, $context);
                    } else if (searchText) {
                        // Finally, a plain text search
                        self.createServerSearchDataTable(plainTextJql, $context);
                    } else {
                        setLoadingIconVisible(false, $context);
                        if (smartAjaxResult.successful) {
                            self.showResults(smartAjaxResult, $context);
                        } else {
                            showResultsError(smartAjaxResult);
                        }
                    }
                });
            } else {
                self.createServerSearchDataTable(plainTextJql, $context);
            }
        } else {
            self.jqlSearch(issueKeyJql, appId).done(function(smartAjaxResult) {
                if (smartAjaxResult.successful && smartAjaxResult.data) {
                    setLoadingIconVisible(false, $context);
                    self.showResults(smartAjaxResult, $context);
                } else if (searchText) {
                    // Finally, a plain text search
                    self.jqlSearch(plainTextJql, appId).done(function(smartAjaxResult) {
                        setLoadingIconVisible(false, $context);
                        if (smartAjaxResult.successful) {
                            self.showResults(smartAjaxResult, $context);
                        } else {
                            showResultsError(smartAjaxResult);
                        }
                    });
                } else {
                    setLoadingIconVisible(false, $context);
                    if (smartAjaxResult.successful) {
                        self.showResults(smartAjaxResult, $context);
                    } else {
                        showResultsError(smartAjaxResult);
                    }
                }
            });
        }
	}

    RAVEN.Dialogs.IssueSelectorBase.prototype.doAdvancedSearch = function(jql, $context) {
        var self = this;

        jql = '('+jql+')';
        // add issue type constraint

        var projecteCond = getProjectJql(this.projectKey);
        if (projecteCond !== null) {
            jql += ' and ' + projecteCond + ' ';
        }

        var issueTypeCond = getIssueTypeJql(this.issueTypeId);
        if (issueTypeCond !== null) {
            jql += ' and ' + issueTypeCond;
        }
        if(self.jql !== null){
            jql += ' and ' + self.jql;
		}
        if (this.mode === ADVANCED_MODE) {
            this.createServerSearchDataTable(jql, $context);
        } else {
            setLoadingIconVisible(true, $context);
            var appId = $("#jira-app-link").val();

            this.jqlSearch(jql, appId).done(function(smartAjaxResult) {
                setLoadingIconVisible(false, $context);
                if (smartAjaxResult.successful) {
                    self.showResults(smartAjaxResult, $context);
                } else {
                    if (smartAjaxResult.status === 400) {
                        AJS.messages.warning("#search-results-table", {
                            body: AJS.I18n.getText("dialogs.issueselector.search.invalid.jql"),
                            closeable: false
                        });
                    } else {
                        showResultsError(smartAjaxResult);
                    }
                }
            });
        }
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.jqlSearch = function(jql, appId) {
        var deferred = $.Deferred();
        var url;
        if (appId && appId !== "") {
            // Remote JIRA instance
            url = contextPath + "/rest/remoteJiraIssueLink/1/remoteJira/search?jql=" + jql + "&appId=" + appId;
        } else {
            // Local JIRA instance
            // Filter out current issue from results
            if (jql) {
                jql = "(" + jql + ")";
            }
            jql += " and key != " + this.issueKey;
            url = contextPath + "/rest/api/2/search?jql=" + jql;
        }

        JIRA.SmartAjax.makeRequest({
            url: url,
            complete: function(xhr, textStatus, smartAjaxResult) {
                deferred.resolve(smartAjaxResult);
            }
        });
        return deferred.promise();
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.advancedJqlSearch = function(jql) {
        var deferred = $.Deferred();
        // Local JIRA instance
        // Filter out current issue from results
        if (jql) {
            jql = "(" + jql + ")";
        }
        jql += " and key != " + this.issueKey;
        var url = contextPath + "/rest/raven/1.0/issue/search?jql=" + jql;

        JIRA.SmartAjax.makeRequest({
            url: url,
            complete: function(xhr, textStatus, smartAjaxResult) {
                deferred.resolve(smartAjaxResult);
            }
        });
        return deferred.promise();
    }

    function getJqlAutoCompleteData(appId) {
        var deferred = $.Deferred();
        var remote;
        var url;
        if (appId && appId !== "") {
            // Remote JIRA instance
            url = contextPath + "/rest/remoteJiraIssueLink/1/remoteJira/autocompletedata?appId=" + appId;
            remote = true;
        } else {
            // Local JIRA instance
            url = contextPath + "/rest/api/2/jql/autocompletedata";
            remote = false;
        }
        JIRA.SmartAjax.makeRequest({
            url: url,
            complete: function(xhr, textStatus, smartAjaxResult) {
                if (!smartAjaxResult.successful && remote) {
                    // If a remote JIRA request fails, it probably doesn't have the autocompletedata REST endpoint (added in JIRA v5.1)
                    // Get the auto complete data by parsing the issue navigator page
                    JIRA.SmartAjax.makeRequest({
                        url: contextPath + "/rest/remoteJiraIssueLink/1/remoteJira/autocompletedata/legacy?appId=" + appId,
                        complete: function(xhr, textStatus, smartAjaxResult) {
                            deferred.resolve(smartAjaxResult);
                        }
                    });
                } else {
                    deferred.resolve(smartAjaxResult);
                }
            }
        });
        return deferred.promise();
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.showResults = function(smartAjaxResult, $context) {
        if (this.mode === SIMPLE_MODE) {
            this.showSimpleResults(smartAjaxResult, $context);
        } else {
            this.showAdvancedResults(smartAjaxResult, $context);
        }
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.showSimpleResults = function(smartAjaxResult, $context) {
        var resultHtml = RAVEN.Dialogs.IssueSearch.linkResultsTable({ result: smartAjaxResult.data });
        $("#search-results-table", $context).html(resultHtml);

        var self = this;

        $("#raven-search-result-table tbody tr", $context).click(function() {
            var selectedIssueKey = $(this).children().first().data("key");
            // close search dialog
            self.remoteJiraSearchDialog.hide();

            // update stacked dialog
            $("#raven-field-issuepicker").trigger("selectOption", [{ 'value': selectedIssueKey }]);
            $('#issue-selector-dialog textarea').focus().select();
        });
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.showAdvancedResults = function(smartAjaxResult, $context) {
        var testTypeCFKey = this.type === 'TEST' || this.type === 'PRECONDITION' ? this.testTypeCFMeta.key : null;
        var resultHtml = RAVEN.Dialogs.IssueSearch.resultTable({ result: smartAjaxResult.data, contextPath: AJS.contextPath(), testTypeCF: testTypeCFKey });
        $("#search-results-table", $context).html(resultHtml);

        if ($('#raven-search-result-table').length > 0) {
            var table = createSearchResultsDataTable();

            $('#search-results-table th input[name="all"]').change(function() {
                if ($(this).is(':checked')) {
                    $('input[type="checkbox"]', table.fnGetNodes()).attr("checked", "checked");
                } else {
                    $('input[type="checkbox"]', table.fnGetNodes()).removeAttr("checked");
                }
            });

            // toogle issue selection uppon row click
            $(table.fnGetNodes()).click(function(e) {
                var input = $(this).find('input[type="checkbox"]');
                if (input.is(':checked')) {
                    input.removeAttr("checked");
                } else {
                    input.attr("checked", "checked");
                }
            });

            $('input[type="checkbox"]', table.fnGetNodes()).click(function(e) {
                e.stopPropagation();
            });
        }
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.getSelectedIssues = function() {
        var value = [];
        $(document.body).find('#issue-selector-dialog ul.items li.item-row').each(function() {
            var val = $(this).find("button.value-item span.value-text");
            value.push(val.text());
        });
        return value;
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.createServerSearchDataTable = function(jql, $context) {
        resetSelectedValues();
        setLoadingIconVisible(false, $context);
        var self = this;

        var testTypeCFKey = this.type === 'TEST' ? this.testTypeCFMeta.key : null;

        var resultHtml = RAVEN.Dialogs.IssueSearch.renderSearchResultsTable({ testTypeCF: testTypeCFKey });

        $("#search-results-table").hide();
        $("#search-results-table", $context).html(resultHtml);

        if ($('#raven-search-result-table').length > 0) {
            setLoadingIconVisible(true, $context);

            var path = contextPath + "/rest/raven/1.0/issue/search?jql=" + jql;

            if (testTypeCFKey) {
                path += "&hideUnexecutableTests=" + $('#cbHideForbiddenTests').is(":checked");
            }

            var table = $('#raven-search-result-table').xrayDataTable({
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('view-issue-precondition-tests-section.infoempty'),
                    "sEmptyTable": AJS.I18n.getText('view-issue-precondition-tests-section.emptytable'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bServerSide": true,
                "sAjaxSource": path,
                "sAjaxDataProp": "entries",
                "bProcessing": false,
                "bFilter": false,
                "bLengthChange": false,
                "iDisplayLength": 10,
                "bDestroy": true,
                "bAutoWidth": false,
                "sPaginationType": "full_numbers",
                "aaSorting": [],
                "bSort": true,
                "aoColumnDefs": this._createServerDataTableColumnDefs(this.type === 'TEST'),
                "fnServerData": function(sSource, aoData, fnCallback) {
                    $.ajax({
                        "dataType": 'json',
                        "contentType": "application/json",
                        "type": "GET",
                        "url": sSource,
                        "data": aoData,
                        "success": function(msg) {
                            fnCallback(msg);
                        },
                        error: function(xhr, textStatus, error) {
                            self.showError(xhr, textStatus, error);

                            setLoadingIconVisible(false, $context);

                            // At the end of the draw Check if all lines are selected and if so tick the all selected box
                            // this is called after all RowCall are done.
                            if (checkIfAllSelected()) {
                                $('#page-select-all').attr("checked", "checked");
                            }
                        }
                    });
                },
                "fnDrawCallback": function(oSettings) {
                    self.toogleSearchResultsTable(oSettings);

                    setLoadingIconVisible(false, $context);

                    // At the end of the draw Check if all lines are selected and if so tick the all selected box
                    // this is called after all RowCall are done.
                    if (checkIfAllSelected()) {
                        $('#page-select-all').attr("checked", "checked");
                    }

                },
                "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    nRow.id = aData["key"];
                    $(nRow).data("key", aData["key"]);

                    // when page changes, select all is unchecked
                    $('#page-select-all').removeAttr("checked");

                    // Check the previous selected values
                    if (selectedValues.indexOf(aData["key"]) > -1) {
                        var input = $(nRow).find('input[type="checkbox"]')
                        if (input.length > 0) {
                            input.attr("checked", "checked");
                        }
                    }
                    return nRow;
                }
            });
        }

        $('#raven-search-result-table th input[name="all"]').change(function() {
            if ($(this).is(':checked')) {
                $('input[type="checkbox"]', table.fnGetNodes()).attr("checked", "checked");
                var nodes = table.fnGetNodes();
                for (var i = 0; i < nodes.length; i++) {
                    addToSelectedValues({ name: nodes[i].id });
                }
            } else {
                $('input[type="checkbox"]', table.fnGetNodes()).removeAttr("checked");
                var nodes = table.fnGetNodes();
                for (var i = 0; i < nodes.length; i++) {
                    removeFromSelectedValues({ name: nodes[i].id });
                }
            }
        });


        // need to remove the event to prevent repetition of events
        $(document).off('click', '#raven-search-result-table tbody tr');
        // toogle issue selection uppon row click
        $(document).on('click', '#raven-search-result-table tbody tr', function(e) {
            var input = $(this).find('input[type="checkbox"]');
            if (input.is(':checked')) {
                removeFromSelectedValues(input[0]);
                input.removeAttr("checked");
                $('#page-select-all').removeAttr("checked");
            } else {
                addToSelectedValues(input[0]);
                input.attr("checked", "checked");
                if (checkIfAllSelected()) {
                    $('#page-select-all').attr("checked", "checked");
                }
            }
        });

        $(document).off('click', '#raven-search-result-table tbody tr input[type="checkbox"]');
        $(document).on('click', '#raven-search-result-table tbody tr input[type="checkbox"]', function(e) {
            var input = $(this);
            if (!input.is(':checked')) {
                removeFromSelectedValues(input[0]);
                $('#page-select-all').removeAttr("checked");
            } else {
                addToSelectedValues(input[0]);
                if (checkIfAllSelected()) {
                    $('#page-select-all').attr("checked", "checked");
                }
            }
        });

        $(document).off('click', '#raven-search-result-table tr input[type="checkbox"]');
        $(document).on('click', '#raven-search-result-table tr input[type="checkbox"]', function(e) {
            e.stopPropagation();
        });
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.toogleSearchResultsTable = function(oSettings) {
        var dataTable = $('#raven-search-result-table').xrayDataTable();
        var $addAllButton = $('#raven-issue-selector-dialog .dialog-button-panel .button-panel-button.button-panel-submit-button').last();

        if (!dataTable || dataTable.fnRecordsTotal() == 0) {
            dataTable.fnDestroy(true);
            this.allSearchResultKeys = null;
            $('#search-results-table')
                .empty()
                .append('<div class="aui-message info">\
                                    <span class="aui-icon icon-info"></span>\
                                    <p>' + AJS.I18n.getText('dialogs.issueselector.search.result.empty') + '</p>\
                                    </div>');
            $addAllButton.hide();
        } else {
            if (oSettings.meta) {
                this.allSearchResultKeys = oSettings.meta;
            }

            $addAllButton.text(AJS.I18n.getText('dialogs.issueselector.label.add-all') + ' (' + dataTable.fnRecordsTotal() + ')');
            $addAllButton.show();
        }

        $("#search-results-table").show();
    }

    RAVEN.Dialogs.IssueSelectorBase.prototype.showError = function(xhr, textStatus, error) {
        //  For Invalid JQL
        if (xhr.status == 400) {
            $('#search-results-table')
                .empty()
                .append('<div class="aui-message error">\
	                                <span class="aui-icon icon-error"></span>\
	            					<p>' + fixErrorMessage(xhr.responseText) + '</p>\
	                                </div>');
            $("#search-results-table").show();
        }
    }

    function fixErrorMessage(errorMessage) {
        var clean = errorMessage.replace("Errors: [", " ");
        clean = clean.replace("], Warnings: []", "");
        return clean;
    }

    /**
     * Creates the DataTable column def descriptors
     */
    RAVEN.Dialogs.IssueSelectorBase.prototype._createServerDataTableColumnDefs = function(isTest) {
        var self = this;

        var columnDefs = [{
                "mData": null,
                "mDataProp": null,
                "sWidth": "20px",
                "sDefaultContent": "<input type='checkbox' ></input>",
                "bSortable": false,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<input type="checkbox" name="' + row["key"] + '" value="' + row["key"] + '"/>';
                },
                "aTargets": [0]
            },
            {
                "mData": "issueType",
                "bSortable": false,
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return RAVEN.Dialogs.IssueSearch.issueType({ issueType: data, contextPath: AJS.contextPath() });
                },
                "aTargets": [1]
            },
            {
                "mData": "key",
                "bSortable": true,
                "mRender": function(data, type, row) {
                    return data;
                },
                "aTargets": [2]
            },
            {
                "mData": "summary",
                "bSortable": true,
                "mRender": function(data, type, row) {
                    return data;
                },
                "aTargets": [3]
            }
        ];

        if (isTest === true) {
            columnDefs.push({
                "mData": "testType",
                "bSortable": true,
                "mRender": function(data, type, row) {
                    return data;
                },
                "aTargets": [4]
            });
        }

        return columnDefs;
    }

    function createSearchResultsDataTable() {
        return $('#raven-search-result-table').xrayDataTable({
            "bProcessing": false,
            "bFilter": false,
            "bLengthChange": false,
            "iDisplayLength": 8,
            "bDestroy": true,
            "bAutoWidth": false,
            "sPaginationType": "full_numbers",
            "aaSorting": [],
            "bSort": true,
            "aoColumnDefs": [
                { "bSortable": false, "aTargets": [0, 1] }
            ]
        });
    }

    function showResultsError(smartAjaxResult) {
        AJS.messages.error("#search-results-table", {
            body: JIRA.SmartAjax.buildSimpleErrorContent(smartAjaxResult),
            closeable: false
        });
    }

    /*
     * Select simple dialog
     */
    RAVEN.Dialogs.createIssueSelectorDialog = function(type, jql, issueTypeId, issueKey, labels, submit, cancel, appLinks, redirectOption, projectKey) {
        // call parent constructor
        RAVEN.Dialogs.IssueSelectorBase.call(this, SIMPLE_MODE, type, issueKey, labels, appLinks, redirectOption, projectKey);
        this.issueTypeId = issueTypeId;
        this.projectKey = projectKey;
        var self = this;
        this.remoteJiraSearchDialog = null;
        this.jql = jql;

        var issueSelectorDialog = new JIRA.FormDialog({
            id: 'raven-issue-selector-dialog',
            width: 680,
            height: "250px",
            stacked: true,
            content: function(render) {
                var innerHtml = self.getIssueSelectorContent();
                initIssueSelector($(innerHtml).filter('#issue-selector-dialog'));
                render($(innerHtml));
                self.initIssuePicker();
            },
            submitHandler: function(e) {
                e.preventDefault();
                submit(self.getSelectedIssues(), getCanRedirectToExecute());
                issueSelectorDialog.hide();
            },
            autoClose: true
        });

        function getCanRedirectToExecute() {
            return AJS.$("#can-redirect-execute-page").is(':checked');
        }

        // manual trigger
        $(document).off('click.raven.issue.selector').on('click.raven.issue.selector', '#raven-issue-selector-search-trigger', function(e) {
            e.stopPropagation();
            e.preventDefault();
            self.remoteJiraSearchDialog.show();
        });

        issueSelectorDialog.show();

        function initIssueSelector(dialog) {
            self.remoteJiraSearchDialog = new JIRA.FormDialog({
                id: "remote-jira-search-dialog", //"raven-issue-search-dialog",
                height: "565px",
                widthClass: "large",
                content: function(render) {
                    var content = RAVEN.Dialogs.IssueSearch.dialog();
                    var $dialogWrapper = $(content);
                    self.initSearchDialog($dialogWrapper);
                    render($dialogWrapper);
                    $("#remote-jira-search-dialog").addClass("raven-issue-search-dialog");
                },
                submitHandler: function(e, callback) {
                    e.preventDefault();
                    $("#search-panel-button").click().removeAttr("disabled");
                    callback();
                }
            });
        }
    }

    /*
     * Select parent issue dialog
     */
    RAVEN.Dialogs.createParentIssueSelectorDialog = function(type, issueTypeId, issueKey, labels, submit, cancel, appLinks, redirectOption, trigger) {
        // call parent constructor
        RAVEN.Dialogs.IssueSelectorBase.call(this, SIMPLE_MODE, type, issueKey, labels, appLinks, redirectOption);
        this.issueTypeId = issueTypeId;

        var self = this;
        this.remoteJiraSearchDialog = null;

        var issueSelectorDialog = new JIRA.FormDialog({
            id: 'raven-parent-issue-selector-dialog',
            width: 530,
            height: "250px",
            content: function(render) {
                var innerHtml = self.getParentIssueSelectorContent();
                initIssueSelector($(innerHtml).filter('#issue-selector-dialog'));
                render($(innerHtml));
                self.initIssuePicker(ISSUE_SELECTOR_REQUIRED);
            },
            submitHandler: function(e) {
                e.preventDefault();
                var selectedIssue = $('#raven-field-singleissuepicker').find(":selected").text();
                if (selectedIssue !== '') {
                    var parentId = null;
                    $.ajax({
                        url: contextPath + '/rest/api/2/issue/' + selectedIssue.slice(0, selectedIssue.indexOf(" ")),
                        type: 'GET',
                        contentType: "application/json",
                        async: false,
                        success: function(data) {
                            parentId = data.id
                        }
                    });

                    issueSelectorDialog.hide();
                    createSubTaskForm(parentId);
                } else {
                    issueSelectorDialog.hide();
                    issueSelectorDialog.show();
                    require('aui/flag')({
                        type: 'error',
                        body: AJS.I18n.getText('execute-test-action.choose-parent-issue.error'),
                        close: 'auto'
                    });
                }
            },
            autoClose: true
        });

        function createSubTaskForm(parentId) {

            var createForm = JIRA.Forms.createSubtaskForm({
                parentIssueId: parentId
            });

            var subtaskDialog = createForm.asDialog({
                id: 'create-subtask-dialog'
            });

            subtaskDialog.show();

        }

        function getCanRedirectToExecute() {
            return AJS.$("#can-redirect-execute-page").is(':checked');
        }

        // manual trigger
        $(document).off('click.raven.issue.selector').on('click.raven.issue.selector', '#raven-issue-selector-search-trigger', function(e) {
            e.stopPropagation();
            e.preventDefault();
            self.remoteJiraSearchDialog.show();
        });

        if (trigger !== null) {
            $(trigger).bind('click', function(e) {
                e.stopPropagation();
                e.preventDefault();
                issueSelectorDialog.show();
            });
        } else {
            issueSelectorDialog.show();
        }

        function initIssueSelector(dialog) {
            self.remoteJiraSearchDialog = new JIRA.FormDialog({
                id: "remote-jira-search-dialog", //"raven-issue-search-dialog",
                height: "565px",
                widthClass: "large",
                content: function(render) {
                    var content = RAVEN.Dialogs.IssueSearch.dialog();
                    var $dialogWrapper = $(content);
                    self.initSearchDialog($dialogWrapper);
                    render($dialogWrapper);
                    $("#remote-jira-search-dialog").addClass("raven-issue-search-dialog");
                },
                submitHandler: function(e, callback) {
                    e.preventDefault();
                    $("#search-panel-button").click().removeAttr("disabled");
                    callback();
                }
            });
        }
    }

    RAVEN.Dialogs.createIssueSelectorDialog.prototype = Object.create(RAVEN.Dialogs.IssueSelectorBase.prototype);

    RAVEN.Dialogs.createParentIssueSelectorDialog.prototype = Object.create(RAVEN.Dialogs.IssueSelectorBase.prototype);

    /*
     * Select and Search advanced dialog
     */
    RAVEN.Dialogs.createIssueSelectorAdvancedDialog = function(type, jql, issueTypeId, issueKey, labels, submit, cancel, assign, isCalledFromTestExecution, allowedFilters, projectKey) {
        // call parent constructor
        RAVEN.Dialogs.IssueSelectorBase.call(this, ADVANCED_MODE, type, issueKey, labels, null, projectKey);

        var self = this;
        this.issueTypeId = issueTypeId;
        this.jql = jql;
        this.assign = assign;
        this.allowedFilters = allowedFilters;
        this.projectKey = projectKey;
        this.selectedProjectKey = null;
        this.projectPicker = null;
        this.versionPicker = null;
        this.testSetVersionPicker = null;
        this.testSetPicker = null;
        this.filterPicker = null;
        this.reqVersionPicker = null;
        this.requirementPicker = null;
        this.testTypeCFMeta = null;
        this.testEnvironmentsCFMeta = null;
        this.allSearchResultKeys = null;
        this.statusPicker = null;
        this.componentPicker = null;
        this.testRepositoryFolderPicker = null;

        // try to fill the window
        var innerWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
        var innerHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;

        var width = Math.max(Math.round(innerWidth * 0.85), 740); // 80%
        var height = Math.max(Math.round(innerHeight * 0.90), 500); // 90%

        // clear previous dialog
        $('#raven-issue-selector-dialog').remove();

        var dialog = new AJS.Dialog({
            width: width,
            height: height,
            id: "raven-issue-selector-dialog",
            keypressListener: function(e) {
                if (e.keyCode === 27 && self.dialog.popup.element.is(":visible")) {
                    self.dialog.remove();
                }
            }
        });
        this.dialog = dialog; // keed using the local variable

        // header
        if (labels && labels.title) {
            title = labels.title;
        } else {
            title = AJS.I18n.getText('dialogs.issueselector.default.title', issueKey);
        }
        dialog.addHeader(title);

        // add select panel
        dialog.addPanel(AJS.I18n.getText('dialogs.issueselector.select-panel.label'), getSelectPanelContent(), "panel-body", SELECT);

        // add search panel
        dialog.addPanel(AJS.I18n.getText('dialogs.issueselector.search-panel.label'), getSearchPanelContent(), "panel-body", SEARCH);

        dialog.addSubmit(AJS.I18n.getText('dialogs.issueselector.label.add-selected'), function(dialog) {
            if (assign === true) {
                var assignee = $('#raven-assignee-select')[0].value;
                if (assignee == "-1") {
                    assignee = null;
                }
                submit(self.getSelectedIssues(), assignee);
            } else {
                submit(self.getSelectedIssues());
            }
            dialog.remove();
        });

        dialog.addSubmit(AJS.I18n.getText('dialogs.issueselector.label.add-all'), function(dialog) {
            var table = $('#raven-search-result-table').xrayDataTable();
            if (table && table.fnSettings() && table.fnSettings().meta) {
                if (assign === true) {
                    var assignee = $('#raven-assignee-select')[0].value;
                    if (assignee == "-1") {
                        assignee = null;
                    }
                    submit(table.fnSettings().meta, assignee);
                } else {
                    submit(table.fnSettings().meta);
                }
            }
            dialog.remove();
        });
        var $addAllButton = $('#raven-issue-selector-dialog .dialog-button-panel .button-panel-button.button-panel-submit-button').last();
        $addAllButton.hide();

        dialog.addLink(AJS.I18n.getText('common.forms.cancel'), function(dialog) {
            if (AJS.$("#filter-additions-button").attr("aria-expanded") === "true") {
                AJS.$("#filter-additions-button").trigger("aui-button-invoke");
            }

            dialog.remove();
            if (cancel) {
                cancel();
            }
        });

        // add assign support
        if (this.assign === true) {
            renderAssigneePicker();
        }

        initSelectPanel();
        initSearchPanel();

        dialog.gotoPanel(0);

        dialog.show();

        // Create checkbox Control to filter Tests that are in a Non-executable status
        createTestStatusesCheckBoxControl(type);

        $('#raven-issue-selector-dialog .dialog-page-menu .page-menu-item .item-button').click(function() {
            var panel = self.dialog.getCurrentPanel();

            var $addAllButton = $('#raven-issue-selector-dialog .dialog-button-panel .button-panel-button.button-panel-submit-button').last();
            if (panel.id === SEARCH_PANEL_ID) {
                $addAllButton.show();
            } else {
                $addAllButton.hide();
            }
        });

        function createTestStatusesCheckBoxControl(type) {
            if (isCalledFromTestExecution) {
                var div = document.createElement('div');
                div.className = "xray-cb-container";

                var label = document.createElement('label')
                label.htmlFor = "id";

                $('.dialog-button-panel').append(div);

                $('<input>', {
                    type: "checkbox",
                    id: "cbHideForbiddenTests",
                    checked: "checked",
                }).appendTo('.xray-cb-container');

                $('.xray-cb-container').append(label);

                if (type === "TEST") {
                    label.appendChild(document.createTextNode(AJS.I18n.getText('raven-admin-config.general.disallowed-execution-test.hide-forbidden.label')));
                    $('#cbHideForbiddenTests').change(
                        function() {
                            var $select = $("#raven-field-issuepicker");
                            var hideUnexecutableTests = $(this).is(":checked");
                            updateIssuePickerIncludeForbiddenTestStatuses($select, hideUnexecutableTests);

                        });
                } else if (type === "TESTSET") {
                    label.appendChild(document.createTextNode(AJS.I18n.getText('raven-admin-config.general.disallowed-execution-test.do-not-add-forbidden.label')));
                }
            } else {
                //If we are not in Test Execution search page then we don't want to hide any test based in status
                var $select = $("#raven-field-issuepicker");
                updateIssuePickerIncludeForbiddenTestStatuses($select, false);

            }
        }


        function getSelectPanelContent() {
            var innerHtml = self.getIssueSelectorContent();
            return $(innerHtml);
        }

        function initSelectPanel() {
            self.initIssuePicker();

            $("#issue-selector-form").submit(function() {
                return false;
            });
        }

        function renderAssigneePicker() {
            if (!isFilterAllowed(RAVEN.Dialogs.FILTER_ASSIGNEE)) {
                return;
            }
            var assigneeSelectHtml = RAVEN.Dialogs.IssueSearch.renderAssigneePicker({ 'issueKey': issueKey });
            $('#raven-issue-selector-dialog .dialog-components').append(assigneeSelectHtml);

            this.assigneePicker = new JIRA.AssigneePicker({
                element: $('.xray-assignee-picker')
            });
        }

        function getSearchPanelContent() {
            var content = RAVEN.Dialogs.IssueSearch.searchForm({ issueType: type, contextPath: contextPath });
            return content;
        }

        function initSearchPanel() {
            if (type === 'TEST') {
                initTestSearchPanel();
            } else if (type === 'TESTSET') {
                initTestSetSearchPanel();
            } else if (type === 'PRECONDITION') {
                initPreConditionSearchPanel();
            } else if (type === 'TESTPLAN') {
                initTestPlanSearchPanel();
            } else if (type === 'TESTEXECUTION') {
                initTestExecSearchPanel();
            }
        }

        function initTestSearchPanel() {
            fetchTestTypeCFMeta(function(data) {
                self.testTypeCFMeta = data;
                self.initSearchDialog($('#raven-issue-search'));
                createTestTypePicker($('#test-type-filter'));
                createProjectPicker($('#project-filter'));
                //createProjectFolderPicker($('#folder-filter')); TODO: remove this comment in the future. this was postponed to V3.1
                fetchAdditionalFilters(initAdditionalFilters);
            });
        }

        function initTestSetSearchPanel() {
            self.initSearchDialog($('#raven-issue-search'));
            createProjectPicker($('#project-filter'));
            fetchAdditionalFilters(initAdditionalFilters);
        }

        function initTestPlanSearchPanel() {
            self.initSearchDialog($('#raven-issue-search'));
            createProjectPicker($('#project-filter'));
            fetchAdditionalFilters(initAdditionalFilters);
        }

        function initPreConditionSearchPanel() {
            fetchPreConditionTypeCFMeta(function(data) {
                self.testTypeCFMeta = data;
                self.initSearchDialog($('#raven-issue-search'));
                createTestTypePicker($('#precondition-type-filter'));
                createProjectPicker($('#project-filter'));
                fetchAdditionalFilters(initAdditionalFilters);
            });
        }

        function initTestExecSearchPanel() {
            fetchTestTypeCFMeta(function(data) {
                self.testTypeCFMeta = data;
            });
            fetchCFMeta(function(data) {
                self.testEnvironmentsCFMeta = data;
            }, "TEST_ENVIRONMENTS");

            self.initSearchDialog($('#raven-issue-search'));
            createProjectPicker($('#project-filter'));
            fetchAdditionalFilters(initAdditionalFilters);
        }

        function initAdditionalFilters(filters) {
            // if we arrive here, the view was initialized successfuly
            $('#raven-issue-search').show();

            if (filters && filters.length > 0) {
                $(filters).each(function() {
                    if (!isFilterAllowed(this.name)) {
                        return;
                    }
                    $('#raven-issue-search-filter-additions ul').append('<li><a id="' + this.name + '-trigger" class="aui-dropdown2-checkbox interactive">' + this.label + '</a></li>');
                    var self = this;
                    $('#' + this.name + '-trigger').on({
                        'aui-dropdown2-item-check': function() {
                            addFilter(self.name);
                        },
                        'aui-dropdown2-item-uncheck': function() {
                            removeFilter(self.name);
                        }
                    });
                });
            } else {
                $('#filter-additions-button').attr("disabled", true).attr("title", "No additional filters available");
            }
        }

        function addFilter(name) {
            if (name === 'label-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderLabelFilter();
                $('#filter-additions').append(content);
                createLabelPicker($('#label-filter'));
            } else if (name === 'version-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderVersionFilter();
                $('#filter-additions').append(content);
                self.versionPicker = createIssueVersionPicker($('#version-filter'));
            } else if (name === 'testset-version-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderTestSetVersionFilter();
                $('#filter-additions').append(content);
                self.testSetVersionPicker = createTestSetVersionPicker($('#testset-version-filter'));
            } else if (name === 'requirement-version-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderRequirementVersionFilter();
                $('#filter-additions').append(content);
                self.reqVersionPicker = createRequirementVersionPicker($('#requirement-version-filter'));
            } else if (name === 'testset-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderTestSetFilter({ versions: '', contextPath: contextPath });
                $('#filter-additions').append(content);
                self.testSetPicker = createTestSetPicker($('#testset-filter'));
            } else if (name === 'requirement-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderRequirementFilter({ versions: '', contextPath: contextPath });
                $('#filter-additions').append(content);
                self.requirementPicker = createRequirementPicker($('#requirement-filter'));
            } else if (name === 'filter-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderFilterFilter();
                $('#filter-additions').append(content);
                self.filterPicker = createIssueFilterPicker($('#filter-filter'));
            } else if (name === 'workflow-status-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderStatusFilter();
                $('#filter-additions').append(content);
                self.statusPicker = createIssueStatusPicker($('#workflow-status-filter'));
            } else if (name === 'component-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderComponentFilter();
                $('#filter-additions').append(content);
                self.componentPicker = createIssueComponentPicker($('#component-filter'))
            } else if (name === 'environment-filter') {
                var content = RAVEN.Dialogs.IssueSearch.renderEnvironmentFilter();
                $('#filter-additions').append(content);
                self.environmentPicker = createIssueEnvironmentPicker($('#environment-filter'));
            }
        }

        function removeFilter(name) {
            if (name === 'label-filter') {
                $('#label-filter').closest('.group').remove();
            } else if (name === 'version-filter') {
                $('#version-filter').closest('.group').remove();
                self.versionPicker = null;
            } else if (name === 'testset-version-filter') {
                $('#testset-version-filter').closest('.group').remove();
                resetIssuePicker(self.testSetPicker, null);
                self.testSetVersionPicker = null;
            } else if (name === 'requirement-version-filter') {
                $('#requirement-version-filter').closest('.group').remove();
                resetIssuePicker(self.requirementPicker, null);
                self.reqVersionPicker = null;
            } else if (name === 'testset-filter') {
                $('#testset-filter').closest('.group').remove();
                self.testSetPicker = null;
            } else if (name === 'requirement-filter') {
                $('#requirement-filter').closest('.group').remove();
                self.requirementPicker = null;
            } else if (name === 'filter-filter') {
                $('#filter-filter').closest('.group').remove();
                self.filterPicker = null;
            } else if (name === 'workflow-status-filter') {
                $('#workflow-status-filter').closest('.group').remove();
                self.statusPicker = null;
            } else if (name === 'component-filter') {
                $('#component-filter').closest('.group').remove();
                self.componentPicker = null;
            } else if (name === 'environment-filter') {
                $('#environment-filter').closest('.group').remove();
                self.environmentPicker = null;
            }
        }

        function fetchTestTypeCFMeta(fnSuccess) {
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/customField/TEST_TYPE',
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    //var message = AJS.I18n.getText('dialogs.issueselector.search.init.error');
                    showInitializationError();
                },
                success: function(data, textStatus, jqXHR) {
                    fnSuccess(data);
                }
            });
        }

        function fetchCFMeta(fnSuccess, fieldName) {
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/customField/' + fieldName,
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    showInitializationError();
                },
                success: function(data, textStatus, jqXHR) {
                    fnSuccess(data);
                }
            });
        }

        function fetchPreConditionTypeCFMeta(fnSuccess) {
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/customField/PRECONDITION_TYPE',
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    //var message = AJS.I18n.getText('dialogs.issueselector.search.init.error');
                    showInitializationError();
                },
                success: function(data, textStatus, jqXHR) {
                    fnSuccess(data);
                }
            });
        }

        function fetchAdditionalFilters(fnSuccess) {
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/issue/search/filter?issueType=' + type,
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    //var message = AJS.I18n.getText('dialogs.issueselector.search.init.filters.error', errorThrown);
                    showInitializationError();
                },
                success: function(data, textStatus, jqXHR) {
                    fnSuccess(data);
                }
            });
        }

        function createProjectFolderPicker($selectField) {
            if (!isFilterAllowed(RAVEN.Dialogs.FILTER_FOLDER)) {
                $selectField.closest('.group').hide();
                return;
            }

            this.testRepositoryFolderPicker = new RAVEN.TestRepository.Fields.FolderPicker({
                $picker: $selectField,
                repositoryPath: "folderStructure",
                folderJQLName: 'testRepositoryFolderTests',
                entityKey: getCurrentProjectKey(),
                siblingDropdownElements: getSiblingDropdownElements()
            });
        }

        function getSiblingDropdownElements() {
            return [
                'project-filter-field',
                'test-type-filter-textarea',
                'label-filter-textarea',
                'version-filter-textarea',
                'testset-filter-textarea',
                'testset-version-filter-textarea',
                'requirement-filter-textarea',
                'requirement-version-filter-textarea',
                'filter-filter-field',
                'workflow-status-filter-textarea',
                'component-filter-textarea'
            ];
        };

        function createProjectPicker($selectField) {
            if (!isFilterAllowed(RAVEN.Dialogs.FILTER_PROJECT)) {
                $selectField.closest('.group').hide();
                return;
            }

            $selectField.on('initialized', function(e, field) {
                initDefaultValues(getCurrentProjectKey());

                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });

            this.projectPicker = new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/project',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-projects-opt-group",
                            replace: true,
                            label: "All Projects"
                        });

                        $(data).each(function(i, proj) {
                            var item = new AJS.ItemDescriptor({
                                value: proj.key,
                                label: proj.name,
                                icon: proj.avatarUrls['16x16'],
                                allowDuplicate: false,
                                selected: proj.key === getCurrentProjectKey() ? 'selected' : null
                            });
                            optgroup.addItem(item);
                        });
                        return [optgroup];
                    }
                }
            });

            $selectField.change(function() {
                var newSelectedValue = $selectField.find('option:selected').attr('value');
                if (this.selectedProjectKey !== newSelectedValue) {
                    this.selectedProjectKey = newSelectedValue;
                    // update version picker
                    resetVersionPicker(self.versionPicker, newSelectedValue);
                    resetVersionPicker(self.testSetVersionPicker, newSelectedValue);
                    resetVersionPicker(self.reqVersionPicker, newSelectedValue);
                    resetIssuePicker(self.testSetPicker, null);
                    resetIssuePicker(self.requirementPicker, null);

                    resetComponentPicker(self.componentPicker);

                    resetFolderPicker(newSelectedValue);

                }
            });
        }

        function resetVersionPicker(picker, projectKey) {
            if (picker) {
                picker.clearLozenges();
                var $el = picker.model.$element;
                $el.empty();
                picker.setProjectKey(projectKey);
            }
        }

        function resetIssuePicker(picker, versions, clear) {
            if (picker) {
                if (!clear || clear === true) {
                    picker.clearLozenges();
                    var $el = picker.model.$element;
                    $el.empty();
                }
                picker.setFixVersions(versions);
            }
        }

        function resetComponentPicker(picker) {
            if (picker) {
                $('#component-filter').closest('.group').remove();
                var content = RAVEN.Dialogs.IssueSearch.renderComponentFilter();
                $('#filter-additions').append(content);
                self.componentPicker = createIssueComponentPicker($('#component-filter'))
            }
        }

        function resetFolderPicker(projectKey) {
            if(this.testRepositoryFolderPicker) {
                this.testRepositoryFolderPicker.setEntityKey(projectKey);
                this.testRepositoryFolderPicker.resetFolderPicker();
            }
        }

        function initDefaultValues(projectKey) {
            // init context project
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/api/2/project/' + projectKey,
                processData: true,
                async: true,
                contentType: "application/json",
                success: function(proj, textStatus, jqXHR) {
                    var defaultProj = new AJS.ItemDescriptor({
                        value: proj.key,
                        label: proj.name,
                        icon: proj.avatarUrls['16x16'],
                        allowDuplicate: false,
                        selected: 'selected'
                    });
                    projectPicker.setSelection(defaultProj);
                }
            });
        }

        function getCurrentProjectKey() {
            //TODO in the future, we must never depend on the issuekey to extract the project key and instead always receive the project key in the constructor instead
            if (issueKey) {
                return issueKey.split('-')[0];
            }
            return projectKey;
        }

        function versionChanged($selectField, targetIssuePicker, clear) {
            // update dependent issue picker
            var selectedValues = $selectField.find('option:selected').map(function() {
                return $(this).attr('value');
            }).toArray().join(';');

            resetIssuePicker(targetIssuePicker, selectedValues, clear);
        }

        function createVersionPicker($selectField) {
            return new RAVEN.VersionPicker({
                element: $selectField,
                loadOnInit: true,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.versions.error"),
                maxInlineResultsDisplayed: 15,
                projectKey: getCurrentProjectKey()
            });
        }

        function createFilterPicker($selectField) {
            this.filterPicker = new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/filter/search',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-filter-opt-group",
                            replace: true,
                            label: this.data.query != "" ? AJS.I18n.getText("view-issue-search-filter.label.results") : AJS.I18n.getText("view-issue-search-filter.label.favourite-filters")
                        });

                        $(data).each(function(i, proj) {
                            this.label = this.name;
                            var item = new AJS.ItemDescriptor({
                                value: this.id + "",
                                label: this.name,
                                title: this.name,

                                highlighted: true,
                            });
                            optgroup.addItem(item);
                        });

                        return [optgroup];
                    }
                },
            });
        }

        function createIssueVersionPicker($selectField) {
            var versionPicker = createVersionPicker($selectField);

            // init version picker
            initVersionPicker(versionPicker, null);

            return versionPicker;
        }

        function createTestSetVersionPicker($selectField) {
            // register change event for the dependent issue picker
            $selectField.off('selected').on('selected', function(descriptor, picker) {
                versionChanged($(this), self.testSetPicker);
            });
            $selectField.off('unselect').on('unselect', function(descriptor, picker) {
                versionChanged($(this), self.testSetPicker, false);
            });

            var versionPicker = createVersionPicker($selectField);

            // init version picker
            initVersionPicker(versionPicker, self.testSetPicker);

            return versionPicker;
        }

        function createIssueFilterPicker($selectField) {
            var filterPicker = createFilterPicker($selectField);
            return filterPicker;
        }

        function createRequirementVersionPicker($selectField) {
            // register change event for the dependent issue picker
            $selectField.off('selected').on('selected', function(descriptor, picker) {
                versionChanged($(this), self.requirementPicker);
            });
            $selectField.off('unselect').on('unselect', function(descriptor, picker) {
                versionChanged($(this), self.requirementPicker, false);
            });

            var versionPicker = createVersionPicker($selectField);

            // init version picker
            initVersionPicker(versionPicker, self.requirementPicker);

            return versionPicker;
        }

        function initVersionPicker(versionPicker, targetIssuePicker) {
            // only init if the current selected project is the same as the context issue project
            var selectedProjKey = $('#project-filter option:selected').attr('value');
            if (getCurrentProjectKey() === selectedProjKey) {
                $.ajax({
                    type: "GET",
                    url: contextPath + '/rest/api/2/issue/' + issueKey + '?fields=fixVersions', //TODO no can do sir
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    success: function(data, textStatus, jqXHR) {
                        var fixVersions = data.fields.fixVersions;
                        if (fixVersions && fixVersions.length > 0) {
                            // clear target issue picker
                            if (targetIssuePicker) {
                                var versionNames = $(fixVersions).map(function() {
                                    return this.id;
                                }).toArray().join(';');

                                resetIssuePicker(targetIssuePicker, versionNames);
                            }
                            $(fixVersions).each(function(i, version) {
                                versionPicker.addItem(new AJS.ItemDescriptor({
                                    value: version.id,
                                    label: version.name,
                                    title: version.description,
                                    highlighted: true,
                                    selected: 'selected'
                                }), true);
                            });
                        }
                    }
                });
            }
        }

        function isFilterAllowed(filter) {
            if (!allowedFilters || allowedFilters.get(filter) === undefined) {
                return true;
            }
            return allowedFilters.get(filter) === true;
        }

        function createTestTypePicker($selectField) {
            if (!isFilterAllowed(RAVEN.Dialogs.FILTER_TESTTYPE)) {
                $selectField.hide();
                return;
            }

            if (self.testTypeCFMeta && self.testTypeCFMeta.options) {
                createTestTypeOptions($selectField, self.testTypeCFMeta.options);
            }

            return new AJS.MultiSelect({
                element: $selectField,
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: false,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.versions.error"),
                maxInlineResultsDisplayed: 15
            });
        }

        function createRequirementPicker($selectField) {
            var enter = '';
            if (this.labels && this.labels.enter) {
                enter = this.labels.enter;
            } else {
                enter = AJS.I18n.getText('dialogs.issueselector.default.enter');
            }

            var picker = new RAVEN.IssuePicker({
                element: $selectField,
                userEnteredOptionsMsg: enter,
                uppercaseUserEnteredOnSelect: true
            });

            picker.$container.next('.issue-picker-popup').remove();
            picker.$container.removeClass('hasIcon');

            return picker;
        }

        function createTestTypeOptions($selectField, data) {
            $(data).each(function(i, type) {
                $selectField.append('<option data-option-id="' + type.id + '" value="' + type.name + '">' + type.name + '</option>');
            });
        }

        function createLabelPicker($selectField) {
            return new JIRA.LabelPicker({
                element: $selectField,
                ajaxOptions: {
                    url: contextPath + '/rest/api/1.0/labels/suggest',
                    data: {}
                }
            });
        }

        function createIssueStatusPicker($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/status/',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-filter-opt-group",
                            replace: true
                        });

                        $(data).each(function(i, status) {
                            this.label = this.name;
                            var item = new AJS.ItemDescriptor({
                                value: this.name,
                                label: this.name,
                                title: this.description,

                                highlighted: true,
                            });
                            optgroup.addItem(item);
                        });

                        return [optgroup];
                    }
                }
            });
        }

        function createIssueComponentPicker($selectField) {
            var projKey = $('#project-filter option:selected').attr('value');
            if (!projKey) {
                projKey = getCurrentProjectKey();
            }

            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/project/' + projKey + '/components',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-filter-opt-group",
                            replace: true
                        });

                        $(data).each(function(i, status) {
                            this.label = this.name;
                            var item = new AJS.ItemDescriptor({
                                value: this.name,
                                label: this.name,
                                title: this.description,

                                highlighted: true,
                            });
                            optgroup.addItem(item);
                        });

                        return [optgroup];
                    }
                }
            });
        }

        function createIssueEnvironmentPicker($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/testenvironment/list',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-environments-opt-group",
                            replace: true
                        });

                        for (var i = 0; i < data.length; i++) {
                            var item = new AJS.ItemDescriptor({
                                value: data[i],
                                label: data[i],
                                title: data[i],
                                highlighted: true
                            });
                            optgroup.addItem(item);
                        }
                        return [optgroup];
                    }
                }
            });
        }

        function createTestSetPicker($selectField) {
            var enter = '';
            if (this.labels && this.labels.enter) {
                enter = this.labels.enter;
            } else {
                enter = AJS.I18n.getText('dialogs.issueselector.default.enter');
            }

            var picker = new RAVEN.IssuePicker({
                element: $selectField,
                userEnteredOptionsMsg: enter,
                uppercaseUserEnteredOnSelect: true
            });

            picker.$container.next('.issue-picker-popup').remove();
            picker.$container.removeClass('hasIcon');

            return picker;
        }

        function showInitializationError() {
            AJS.messages.error("#raven-initialization-error", {
                body: AJS.I18n.getText('dialogs.issueselector.search.init.error'),
                closeable: false
            });
        }

        return self;
    }

    RAVEN.Dialogs.createIssueSelectorAdvancedDialog.prototype = Object.create(RAVEN.Dialogs.IssueSelectorBase.prototype);

    RAVEN.Dialogs.createIssueSelectorAdvancedDialog.prototype.getIssuesFromSearchPanel = function() {
        if ($('#raven-search-result-table').length > 0) {
            return selectedValues;
        }
    }

    RAVEN.Dialogs.createIssueSelectorAdvancedDialog.prototype.getIssuesFromSelectPanel = function() {
        var value = [];
        $(document.body).find('#issue-selector-dialog ul.items li.item-row').each(function() {
            var val = $(this).find("button.value-item span.value-text");
            value.push(val.text());
        });
        return value;
    }

    RAVEN.Dialogs.createIssueSelectorAdvancedDialog.prototype.getSelectedIssues = function() {
        var panel = this.dialog.getCurrentPanel();

        if (panel.id === SELECT_PANEL_ID) {
            return this.getIssuesFromSelectPanel();
        } else {
            return this.getIssuesFromSearchPanel();
        }
    }

    //update ulr endoint to search for issues according to the selected jira server
    var updateIssuePicker = function($select, appId) {
        if ($select.length) {
            // Update the appId param
            $select.attr("data-ajax-options.data.app-id", appId);

            // If we have a remote JIRA server, then we must change the url of the AJAX request in the issuepicker.
            // If not, we don't do anything because, the url parameter is already filled.
            if (appId && appId !== "") {
                // Set the url for remote JIRA queries
                $select.attr("data-ajax-options.url", contextPath + "/rest/remoteJiraIssueLink/1/remoteJira/picker");
            }

            $select.trigger("updateOptions");

            // Now that we have changed server, our current issue selection is no longer relevant
            $select.trigger("clearSelection");
        }
    };

    //update whatever we can show TEst in forbidden statuses
    var updateIssuePickerIncludeForbiddenTestStatuses = function($select, hideUnexecutableTests) {
        if ($select.length) {
            // Update the appId param
            $select.attr("data-ajax-options.data.hide-unexecutable-tests", hideUnexecutableTests);
            $select.trigger("updateOptions");
        }
    };

    JIRA.bind(JIRA.Events.NEW_PAGE_ADDED, function(e, context) {

        var $select = $("#raven-field-issuepicker", context);
        if ($select.length) {
            var appId = $("#jira-app-link", context).val();
            updateIssuePicker($select, appId);
        }

        $("#jira-app-link", context).change(function() {
            var appId = $(this).val();
            updateIssuePicker($select, appId);
        });
    });
})(AJS.$);