(function ($) {
    RAVEN.namespace("RAVEN.Gadget.fields");
    var projectId;
    var componentId;
    var componentUserPref;
    var projectUserPref;
    var $componentContainer;
    var allComponentId = "all-component-id";

    const ALL_VALUE = "-1";
    const SELECT_PROJECT_LABEL = AJS.I18n.getText("common.label.select-project");
    const ALL_LABEL = AJS.I18n.getText("common.label.all");

    function message(title, message, type) {
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

    function hideComponent()
    {
        $("#" + componentId).css("display", "none");
        $("#" + allComponentId).css("display", "block");
    }

    function showComponent()
    {
        $("#" + componentId).css("display", "block");
        $("#" + allComponentId).css("display", "none");
    }

    function populateProjectComponents(gadget, selectedProject) {
        var componentComponent = $("#" + componentId);
        componentComponent.empty(); // clear options
        // append All component

        componentComponent.append('<option value=' + ALL_VALUE + ' selected>' + ALL_LABEL + '</option>');
        if (!selectedProject) {
            return;
        }

        if (selectedProject === '__ALL__' || selectedProject === ALL_VALUE) {
            hideComponent();
            return;
        }

        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + "/rest/api/2/project/" + selectedProject + "/components",
            contentType: "application/json",
            success: function (data) {
                if (data && data.length > 0) {
                    showComponent();
                    $.each(data, function (i, c) {
                        var selected = "";
                        if (gadget && gadget.getPref(componentUserPref) === c.id) {
                            $("#" + componentId + " option:selected").prop("selected", false);
                            selected = " selected";
                        }
                        componentComponent.append('<option value="' + c.id + '"' + selected + '>' + c.name + '</option>');
                    });
                } else {
                    hideComponent();
                }
            },
            error: function (qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                if (gadget) {
                    message(gadget.getMsg("gadget.common.error.components"), errorThrown);
                } else {
                    message(null, errorThrown);
                }
            }
        });
    }

    function populateProjects(gadget) {
        $("#" + projectId).empty(); // clear options
        // append All component
        $("#" + projectId).append('<option value=' + ALL_VALUE + '>' + SELECT_PROJECT_LABEL + '</option>');

        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + "/rest/api/2/project/",
            contentType: "application/json",
            success: function (data) {
                if (data) {
                    $.each(data, function (i, project) {
                        var selected = "";
                        if (gadget && gadget.getPref(projectUserPref) === project.key) {
                            selected = " selected";
                            populateProjectComponents(gadget, project.key);
                        }
                        $("#" + projectId).append('<option value="' + project.key + '"' + selected + '>' + project.name + '</option>');
                    });
                }
            },
            error: function (qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                if (gadget) {
                    message(gadget.getMsg("gadget.common.error.project"), errorThrown);
                } else {
                    message(null, errorThrown);
                }
            }
        });

    }

    RAVEN.Gadget.fields.ProjectComponentSelect = function (gadget, projectPref, componentPref, identifier) {
        projectId = "project" + identifier;
        componentId = "component" + identifier;
        componentUserPref = componentPref;
        projectUserPref = projectPref;
        return {
            id: "component-callback-field",
            userpref: componentUserPref,
            label: gadget.getMsg("gadget.common.field.component.label"),
            description: gadget.getMsg("gadget.common.field.component.desc"),
            type: "callbackBuilder",
            callback: function (parentDiv) {
                var $proj = $("<select></select>").attr({
                    "id": projectId,
                    "class": "select",
                    "name": projectUserPref,
                    "style": "display : block"
                });

                parentDiv.append($proj);


                $componentContainer = $("<div></div>");

                var $comp = $("<select></select>").attr({
                    "id": componentId,
                    "class": "select",
                    "name": componentUserPref,
                    "style": "margin-top : 2px; display : block"
                });

                var $allComponent = $("<select></select>").attr({
                    "id": allComponentId,
                    "class": "select",
                    "style": "margin-top : 2px; display : none"
                });
                var optionAll = '<option value=' + ALL_VALUE + ' selected="selected">' + ALL_LABEL + '</option>';
                $allComponent.append(optionAll);
                $comp.append(optionAll);

                $allComponent.prop('disabled', true);
                $componentContainer.append($comp);
                $componentContainer.append($allComponent);
                parentDiv.append($componentContainer);
                hideComponent();

                var desc = gadget.getMsg("gadget.common.field.component.desc");
                parentDiv.append('<div class="description">' + desc + '</div>');

                $(document).off('change.raven').on('change.raven', '#' + projectId, function () {
                    var selectedProject = $('#' + projectId + " option:selected").val();
                    populateProjectComponents(gadget, selectedProject);
                });

                populateProjects(gadget, projectUserPref);
            }
        };
    }
})(AJS.$);