(function($) {
	RAVEN.namespace("RAVEN.Gadget.fields");

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

	function fetchProjectVersions(gadget, project, defaultEnabled) {
		if (project) {
			$("#version").empty(); // clear options
            // append All version
            var allLabel = AJS.I18n.getText("common.label.all");
            if(defaultEnabled) {
                $("#version").append('<option value="-1">' + allLabel + '</option>');
            } else {
                $("#version").append('<option value="-1" selected disabled>' + "-" + '</option>');
            }

            if (project !== '__ALL__') {
                var isDisabled = $("#version").prop('disabled');
                $("#version").prop('disabled', true);

    			$.ajax({
    				type: "GET",
    	    		url: gadget.getBaseUrl() + "/rest/api/2/project/" + project + "/versions",
    	    		contentType: "application/json",
    	    		success: function(data) {
    	    			if (data && data.length > 0) {
    	    			    if(!defaultEnabled) {
                                $("#version").empty();
                            }
    	    				$.each(data, function(i, v) {
    	    					var selected = "";
    	    					if (gadget && gadget.getPref("version") === v.id) {
    	    						selected = " selected";
    	    					}
    	    					$("#version").append('<option value="' + v.id + '"' + selected + '>' + v.name + '</option>');
    	    				});
    	    			}
    	    		},
    				error: function(qXHR, textStatus, errorThrown) {
    					if (errorThrown && errorThrown.trim() != '') {
    						errorThrown = errorThrown + ': ' + qXHR.responseText;
    					} else {
    						errorThrown = qXHR.responseText;
    					}
    					if (gadget) {
    						message(gadget.getMsg("gadget.common.field.error.versions"), errorThrown);
    					} else {
    						message(null, errorThrown);
    					}
    				},
    				complete: function() {
                        if (isDisabled === false) {
    					   $("#version").prop('disabled', false);
                        }
    				}
    			});
            }
		}
	}

    RAVEN.Gadget.fields.VersionSelect = function(gadget, versionUserPref, versionDisabled, defaultEnabled, selectedProject) {
        return {
            id: "version-callback-field",
            userpref: versionUserPref,
            label: "Analysis Version",
            description: gadget.getMsg("gadget.common.version.desc"),
            type: "callbackBuilder",
            callback: function(parentDiv) {
                var $el = $("<select></select>").attr({
                    "id": "version",
                    "class": "select",
                    "name": "version"
                });

                if (versionDisabled === true) {
                    $el.prop('disabled', true);
                }
                parentDiv.append($el);
                var desc = gadget.getMsg("gadget.common.version.desc");
                parentDiv.append('<div class="description">' + desc + '</div>');

                var project = null;
                if (typeof(selectedProject) == "function") {
                    project = selectedProject();
                } else if (typeof selectedProject == 'string' || selectedProject instanceof String) {
                    project = selectedProject;
                }
                if (project) {
                    fetchProjectVersions(gadget, project, defaultEnabled);
                }
            },
            initProjectVersions: function(gadget, project, defaultEnabled) {
                fetchProjectVersions(gadget, project, defaultEnabled);
            }
        };
    }
})(AJS.$);