(function($) {

    RAVEN.namespace("RAVEN.Gadget");

	function message(title, message, type) {
		clearMessage();
		if (!type) {
			type = 'error';
		}
		AJS.messages[type]($('#gadget-message'), {
			title: title,
			body: message
		});
	}

    function serverError(gadget, message) {
	    clearMessage();
        var obj = null;
        try {
            obj = JSON.parse(message);
        } catch(e) {}

        var msg = null;
        if (obj && obj.message) {
            msg = obj.message;
        } else {
            msg = gadget.getMsg('gadget.test-execution.common.error.data');
        }

        $('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
        AJS.messages['error']($('#gadget-message'), {
            body: msg
        });
    }

	function clearMessage() {
        $('#ag-sys-msg').empty();
		$('#gadget-message').empty();
	}

	function createTestExecutionTable(gadget, data) {
        var content = $('<div id="gadget-content"><div id="gadget-message"></div></div>');
        gadget.getView().html(content);

		if (!data || !data.datasets || data.datasets.length <= 0) {
            message(null, gadget.getMsg("gadget.test-execution.common.empty"), 'info');
            return;
		}

		clearMessage();
		// Render table skeleton
        gadget.getView().html(RAVEN.Gadget.renderTestExecutionSummary());
        var tableContent = RAVEN.Gadget.renderTestExecutionTable({'data': data});
        var textExecutionTable = $('#test-execution-table');
        textExecutionTable.html(tableContent);
        textExecutionTable.xrayDataTable({
            "sPaginationType": "full_numbers",
            "bSort": false,
            "aaSorting": [],
            "bLengthChange": false,
            "bFilter": false,
            "iDisplayLength": parseInt(gadget.getPref("numberResults"))
        });
        textExecutionTable.on('draw', function () {
             gadgets.window.adjustHeight();
		});
    }

	function setTitle(gadget) {
		var customTitle = gadget.getPref("customTitle");
        var useCustomTitle = gadget.getPref("useCustomTitle");
		var execScope = gadget.getPref("execScope");
        var filterId = gadget.getPref("filterId");
        var testPlan = gadget.getPref("testPlanKey");
        var version = gadget.getPref("version");
        var environment = gadget.getPref("environment");
        var component = gadget.getPref("component");
        var testRunPriority = gadget.getPref("testRunPriority");

        var params = '?execScope=' + execScope;

        if(useCustomTitle) {
            params = params + '&useCustomTitle=' + useCustomTitle;
        }

        if (customTitle && customTitle != '') {
            params = params + '&customTitle=' + customTitle;
        }

        if (version && version != -1) {
            params = params + '&version=' + version;
        }

        if (environment && environment != '__ALL__') {
            params = params + '&environment=' + environment;
        }

        if (component) {
            params = params + '&component=' + component;
        }

        if (testPlan && testPlan != -1) {
            params = params + '&testPlan=' + testPlan;
        }

        if (filterId) {
            // remove prefix "filter-"
            params = params + "&savedFilter=" + filterId.substring(7);
        }

        if (testRunPriority && testRunPriority != '__ALL__') {
            params = params + '&testRunPriority=' + testRunPriority;
        }

		$.ajax({
			type: "GET",
			url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/testexecution/title' + params,
			contentType: "application/json",
			success: function(data) {
				if (data && data.titleParams) {
					var title = AJS.format(gadget.getMsg("gadget.test-execution.title"), data.titleParams);
					gadgets.window.setTitle(title);
				} else {
					var title = gadget.getMsg("gadget.test-execution.title.default");
					gadgets.window.setTitle(title);
				}
			}
		});
	}

	RAVEN.Gadget.testExecution = function(contextPath) {
		return AJS.Gadget({
			baseUrl: contextPath, /* Used to make base url available to gadget */
			useOauth: "/rest/gadget/1.0/currentUser",
			config: {
    			descriptor: function(args) {
					var gadget = this;

					// register event handler for refreshing project versions
					$(document).off('change.raven').on('change.raven', '#project', function() {
                        var versionField = gadget.versionField;
                        var componentField = gadget.componentField;
						var selectedProject = $("#project option:selected").val();

						versionField.initProjectVersions(gadget, selectedProject);
						componentField.initProjectComponents(gadget, selectedProject);
					});

					// register event handler for toggling execution scope fields
					$(document).on('change.raven.execscope', '#execScope', function() {
						var selectedScope = $("#execScope option:selected").val();
						if (selectedScope === "version") {
							$("#testPlanKey").prop('disabled', true);
							$("#version").prop('disabled', false);
						} else {
							$("#testPlanKey").prop('disabled', false);
							$("#version").prop('disabled', true);
						}
					});

                    $(document).on('change.raven.title', '#use-custom-title', function() {
                        var useCustomTitle = $("#use-custom-title").attr('checked');
                        if (useCustomTitle) {
                            $("#customTitle").prop('disabled', false);
                        } else {
                            $("#customTitle").prop('disabled', true);
                        }
                    });

                    var customTitleDisabled = true;
                    var useCustomTitle = gadget.getPref("useCustomTitle");
                    if (useCustomTitle === "true") {
                        customTitleDisabled = false;
                    }

					var versionDisabled = false;
					var testPlanKeyDisabled = false;
					var selectedScope = gadget.getPref("execScope");
					if (selectedScope === "version") {
						testPlanKeyDisabled = true;
					} else {
						versionDisabled = true;
					}

					// get project keys because the versions rest service only
					// accepts keys for project in JIRA prior to v6.1
					var projects = $.map(args.projects, function(p) {
						return {
							label: p.name,
							value: p.key
						};
					});

                    projects = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(projects);

                    var priorities = $.map(args.priorities, function(p) {
                        return {
                            label: p.name,
                            value: p.id
                        };
                    });

                    priorities = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(priorities);

                    var environments = $.map(args.filters.environments, function (p) {
                        return {
                            label: p,
                            value: p
                        };
                    });

                    environments = [
                        {label: gadget.getMsg("common.label.all-environments"), value: "-1"},
                        {label: gadget.getMsg("common.label.empty"), value: "__NULL__"}
                        ].concat(environments);

					gadget.versionField = RAVEN.Gadget.fields.VersionSelect(gadget, "version", versionDisabled, false, function() {
						return $("#project option:selected").val(); //gadget.getPref("project");
					});

                    gadget.componentField = RAVEN.Gadget.fields.ComponentSelect(gadget,  gadget.getMsg("gadget.common.field.component.label"), "component", false, function() {
						return $("#project option:selected").val();
					});

                    var filters = null;
                    $.each(args.savedfilters.options, function() {
                        if(this.group.label === "Filters") {
                            filters = this.group;
                        }
                    });

                    if (filters) {
                        var filterId = gadget.getPref("filterId");
                        if (filterId) {
                            var filter = _.find(filters.options, function(f) {
                                return f.value == filterId;
                            });
                            if (filter) {
                                gadget.projectOrFilterName = filter.label;
                            }
                        } else {
                            gadget.projectOrFilterName = null;
                        }
                    } else {
                        gadget.projectOrFilterName = null;
                    }

                    var searchParam;
                    if (/^jql-/.test(this.getPref("filterId")) || this.getPref("isPopup") === "true") {
                        searchParam =
                            {
                                userpref: "filterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("filterId"))
                            };
                    } else {
                        searchParam = RAVEN.gadget.fields.filterPicker(gadget, "filterId");
                    }

					return {
					    action: '/rest/raven/1.0/chart/testexecution/validate',
					    theme : function() {
					        if (gadgets.window.getViewportDimensions().width < 450){
					            return "gdt top-label";
					        } else {
					            return "gdt";
					        }
					    }(),
					    fields: [
                            {
                                id: "customTitle",
                                userpref: "customTitle",
                                label: gadget.getMsg("gadget.test-execution.common.field.custom-title.label") + ":",
                                type: "custom",
                                value: gadget.getPref("customTitle"),
                                template: function() {
                                    var disabled = "";
                                    if (customTitleDisabled === true) {
                                        disabled = " disabled"
                                    }
                                    return '<input type="text" class="text" value="' + gadget.getPref("customTitle") + '" id="customTitle" name="customTitle"' + disabled + '>';
                                }
                            },
                            {
                                userpref: "useCustomTitle",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "use-custom-title",
                                        label: gadget.getMsg("gadget.test-execution.common.field.custom-title.desc"),
                                        value: "true",
                                        selected : gadget.getPref("useCustomTitle") == 'true'
                                    }
                                ]
                            },
                            searchParam,
                            {
								id: "execScope",
        						userpref: "execScope",
        						label: gadget.getMsg("gadget.common.field.executionscope.label"),
        						description: gadget.getMsg("gadget.test-execution.common.field.exec-scope.desc"), //todo
        						type: "select",
        						selected: gadget.getPref("execScope"),
        						options:[
						            {
						                label: "Version",
						                value: "version"
						            },
						            {
						                label: "Test Plan",
						                value: "testPlan"
						            }
						        ]
							},
					    	{
								userpref: "project",
								label: gadget.getMsg("common.label.project"),
								description: gadget.getMsg("gadget.test-execution.common.field.project.desc"),
								type: "select",
								selected: gadget.getPref("project"),
								options: projects
							},
							{
                                id: "testPlanKey",
                                userpref: "testPlanKey",
                                label: gadget.getMsg("gadget.common.field.testplankey.label"),
                                description: gadget.getMsg("gadget.test-execution.common.field.test-plan-key.desc"),
                                type: "custom",
                                value: gadget.getPref("testPlanKey"),
                                template: function() {
                                	var disabled = "";
                                	if (testPlanKeyDisabled === true) {
                                		disabled = " disabled"
                                	}
						            return '<input type="text" class="text" value="' + gadget.getPref("testPlanKey") + '" id="testPlanKey" name="testPlanKey"' + disabled + '>';
						        }
                            },
					        gadget.versionField,
                            {
                                id: "environmentKey",
                                userpref: "environment",
                                label: gadget.getMsg("common.label.test-environment"),
                                description: gadget.getMsg("gadget.common.field.environment.label"),
                                type: "select",
                                selected: gadget.getPref("environment"),
                                options: environments
                            },
					        gadget.componentField,
							{
                                userpref: "testRunPriority",
                                label: gadget.getMsg("gadget.test-execution.common.field.priority.label") + ":",
                                description: gadget.getMsg("gadget.test-execution.common.field.priority.desc"),
                                type: "select",
                                selected: gadget.getPref("testRunPriority"),
                                options: priorities
                            },
                            {
                                userpref: "numberResults",
                                label: gadget.getMsg("gadget.test-execution.common.field.number-of-results.label") + ":",
                                description: gadget.getMsg("gadget.test-execution.common.field.number-of-results.desc"),
                                type: "text",
                                value: gadget.getPref("numberResults")
                            },
    						AJS.gadget.fields.nowConfigured()
					    ]
					};
				},
    			args: [
    				{
        				key: "projects",
						ajaxOptions: {
							url: "/rest/api/2/project",
							contentType: "application/json"
						}
    				},
                    {
                        key: "savedfilters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "priorities",
                        ajaxOptions: {
                            url: "/rest/api/2/priority",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/raven/1.0/chart/testexecution/filters",
                            contentType: "application/json"
                        }
                    }
				]
			},
		   	view: {
				enableReload: true,
				onResizeReload: true,
				onResizeAdjustHeight: false,
				template: function (args) {
					var gadget = this;
					// set gadget title
					setTitle(this);
					// server errors and timeout errors are not catched by the error handler of the REST date request !!!
					if (!args.data) {
						message(null, gadget.getMsg("gadget.test-execution.common.error.data"));
					} else {
						createTestExecutionTable(gadget, args.data);
					}
				},
				args: [{
					key: "data",
					ajaxOptions: function() {
                        var gadget = this;

                        var execScope = this.getPref("execScope");
                        var version = this.getPref("version");
                        var component = this.getPref("component");
                        var testPlan = this.getPref("testPlanKey");
                        var testRunPriority = this.getPref("testRunPriority");
                        var filterId = gadgets.util.unescapeString(this.getPref("filterId"));
						var numberResults = this.getPref("numberResults");
                        var environment = this.getPref("environment");

                        var params = '';

                        params = params + '?execScope=' + execScope;

                        if (version && version != -1)
                            params = params + '&version=' + version;

                        if (component)
                            params = params + '&component=' + component;

                        if (testPlan && testPlan != -1)
                            params = params + '&testPlan=' + testPlan;

                        // remove prefix "filter-"
                        if (filterId)
                            params = params + "&savedFilter=" + filterId.substring(7);

                        if (testRunPriority && testRunPriority != '__ALL__')
                            params = params + '&testRunPriority=' + testRunPriority;

                        if (numberResults)
                        	params +='&numberResults=' + numberResults;

                        if (environment && environment != '__ALL__')
                            params +='&environment=' + environment;

						return {
		    				type: "GET",
		    				url: contextPath
		    						+ '/rest/raven/1.0/chart/testexecution/execution'
                                    + params,
		    				processData: false,
		    				async: true,
							timeout: 120000,
		    				contentType: "application/json",
							error: function(qXHR, textStatus, errorThrown) {
								if (errorThrown && errorThrown.trim() != '') {
									errorThrown = errorThrown + ': ' + qXHR.responseText;
								} else {
									errorThrown = qXHR.responseText;
								}
								serverError(gadget, errorThrown);
								gadgets.window.adjustHeight();
							}
			    		}
			    	}
    			}]
		   	}
		});
	}
})(AJS.$);
