(function ($) {

    RAVEN.namespace("RAVEN.Gadget");

    MAX_DATASET_LENGTH = 1000;

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

    function serverError(gadget, message) {
        var obj = null;
        try {
            obj = JSON.parse(message);
        } catch (e) {
            appendMessage('Error');
            return;
        }

        var msg = null;
        if (obj && obj.message) {
            msg = obj.message;
        } else {
            msg = gadget.getMsg('gadget.common.error.data');
        }
        appendMessage(msg);
    }

    function appendMessage(msg) {
        $('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
        AJS.messages['error']($('#gadget-message'), {
            body: msg
        });
    }

    function clearMessage() {
        $('#gadget-message').empty();
    }

    function createTestListTable(gadget, data) {
        var content = $('<div id="gadget-content"><div id="gadget-message"></div></div>');
        gadget.getView().html(content);

        if (!data || !data.datasets || data.datasets.length <= 0) {
            message(null, gadget.getMsg("gadget.common.test-runs.empty"), 'info');
            return;
        }

        if (data.datasets.length > MAX_DATASET_LENGTH) {
            var datasetLength = data.datasets.length;
            data.datasets = data.datasets.slice(0, MAX_DATASET_LENGTH);
            createTable(gadget, data);
            message(null, AJS.format(gadget.getMsg("gadget.common.too-many-results"), datasetLength, MAX_DATASET_LENGTH), 'info');
        } else {
            createTable(gadget, data);
        }
    }

    var createTable = function (gadget, data) {
        // Render table skeleton
        gadget.getView().html(RAVEN.Gadget.renderTestList());
        var tableContent = RAVEN.Gadget.renderTestListTable({'data': data});
        var testListTable = $('#test-list-table');
        testListTable.html(tableContent);
        testListTable.xrayDataTable({
            "sPaginationType": "full_numbers",
            "bSort": false,
            "aaSorting": [],
            "bLengthChange": false,
            "bFilter": false,
            "iDisplayLength": parseInt(gadget.getPref("numberResults"))
        });
        testListTable.on('draw', function () {
            gadgets.window.adjustHeight();
        });
    };

    function setTitle(gadget) {
        var useCustomTitle = gadget.getPref("useCustomTitle");
        var customTitle = gadget.getPref("customTitle");
        var filterId = gadget.getPref("filterId");
        var startDate = gadget.getPref("startDate");
        var endDate = gadget.getPref("endDate");
        var testRunAssignee = gadget.getPref("testRunAssignee");
        var executedBy = gadget.getPref("executedBy");
        var component = gadget.getPref("component");
        var priority = gadget.getPref("priority");
        var status = gadget.getPref("status");

        var params = '?useCustomTitle=' + useCustomTitle;

        if (filterId) {
            // remove prefix "filter-"
            params = params + "&savedFilter=" + filterId.substring(7);
        }

        params += addNotEmptyParameter(customTitle, "customTitle");
        params += addNotEmptyParameter(startDate, "startDate");
        params += addNotEmptyParameter(endDate, "endDate");
        params += addNotEmptyParameter(testRunAssignee, "testRunAssignee");
        params += addNotEmptyParameter(executedBy, "executedBy");
        params += addParameter(component, "component");
        params += addParameter(priority, "priority");
        params += addParameter(status, "status");

        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/testrun/list/title' + params,
            contentType: "application/json",
            success: function (data) {
                if (data && data.titleParams) {
                    var title = AJS.format(gadget.getMsg("gadget.test-runs-list.title"), data.titleParams);
                    gadgets.window.setTitle(title);
                } else {
                    var title = gadget.getMsg("gadget.test-runs-list.title.default");
                    gadgets.window.setTitle(title);
                }
            }
        });
    }

    function addParameter(value, paramName) {
        if (value) {
            return "&" + paramName + "=" + value;
        }
        return "";
    }

    function addNotEmptyParameter(value, paramName) {
        if (value && value != '') {
            return "&" + paramName + "=" + value;
        }
        return "";
    }

    function appendParameter(params, name, value) {
        if (!value)
            return params;

        var prefix = params.length == 0 ? '?' : '&';
        return params + prefix + name + '=' + value;
    }

    RAVEN.Gadget.testRunsList = function (contextPath) {
        return AJS.Gadget({
            baseUrl: contextPath, /* Used to make base url available to gadget */
            useOauth: "/rest/gadget/1.0/currentUser",
            config: {
                descriptor: function (args) {
                    var gadget = this;

                    $(document).on('click.raven.title', '#use-custom-title', function () {
                        $("#customTitle").prop('disabled', !$("#use-custom-title").attr('checked'));
                    });

                    var priorities = $.map(args.priorities, function (p) {
                        return {
                            label: p.name,
                            value: p.id
                        };
                    });

                    priorities = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(priorities);

                    gadget.componentField = RAVEN.Gadget.fields.ProjectComponentSelect(gadget, "project", "component", "1", function () {
                        return $("#project option:selected").val();
                    });
                    var startDate = RAVEN.gadget.fields.simpleDatePicker(gadget, 'start', 'startDate', gadget.getMsg("gadget.common.date.start"), '');
                    var endDate = RAVEN.gadget.fields.simpleDatePicker(gadget, 'end', 'endDate', gadget.getMsg("gadget.common.date.end"), '');

                    var filters = null;
                    $.each(args.savedfilters.options, function () {
                        if (this.group.label === "Filters") {
                            filters = this.group;
                        }
                    });

                    if (filters) {
                        var filterId = gadget.getPref("filterId");
                        if (filterId) {
                            var filter = _.find(filters.options, function (f) {
                                return f.value == filterId;
                            });
                            if (filter) {
                                gadget.projectOrFilterName = filter.label;
                            }
                        } else {
                            gadget.projectOrFilterName = null;
                        }
                    } else {
                        gadget.projectOrFilterName = null;
                    }

                    var statuses = $.map(args.testStatuses, function (status) {
                        return {
                            label: status,
                            value: status
                        };
                    });

                    statuses = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(statuses);

                    var searchParam;
                    if (/^jql-/.test(this.getPref("filterId")) || this.getPref("isPopup") === "true") {
                        searchParam =
                            {
                                userpref: "filterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("filterId"))
                            };
                    } else {
                        searchParam = RAVEN.gadget.fields.filterPicker(gadget, "filterId");
                    }

                    gadget.assigneeField = RAVEN.gadget.fields.assigneePicker(
                        gadget,
                        "testRunAssignee",
                        gadget.getMsg("gadget.common.field.testrun-assignee.label") + ":",
                        gadget.getMsg("gadget.common.field.testrun-assignee.desc")
                    );


                    gadget.executedByField = RAVEN.gadget.fields.assigneePicker(
                        gadget,
                        "executedBy",
                        gadget.getMsg("gadget.common.field.testrun-executedby.label") + ":",
                        gadget.getMsg("gadget.common.field.testrun-executedby.desc")
                    );
                    return {
                        action: '/rest/raven/1.0/chart/testrun/list/validate',
                        theme: function () {
                            if (gadgets.window.getViewportDimensions().width < 450) {
                                return "gdt top-label";
                            } else {
                                return "gdt";
                            }
                        }(),
                        fields: [
                            {
                                id: "customTitle",
                                userpref: "customTitle",
                                label: gadget.getMsg("gadget.common.custom-title.label") + ":",
                                type: "custom",
                                value: gadget.getPref("customTitle"),
                                template: function () {
                                    var disabled = "";
                                    if (gadget.getPref("useCustomTitle") !== "true") {
                                        disabled = " disabled"
                                    }
                                    return '<input type="text" class="text" value="' + gadget.getPref("customTitle") + '" id="customTitle" name="customTitle"' + disabled + '>';
                                }
                            },
                            {
                                userpref: "useCustomTitle",
                                type: "checkbox",
                                options: [
                                    {
                                        id: "use-custom-title",
                                        label: gadget.getMsg("gadget.common.custom-title.desc"),
                                        value: "true",
                                        selected: gadget.getPref("useCustomTitle") == 'true'
                                    }
                                ]
                            },
                            searchParam,
                            startDate,
                            endDate,
                            gadget.assigneeField,
                            gadget.executedByField,
                            gadget.componentField,
                            {
                                userpref: "priority",
                                label: gadget.getMsg("gadget.common.field.priority.label") + ":",
                                description: gadget.getMsg("gadget.common.field.priority.desc"),
                                type: "select",
                                selected: gadget.getPref("priority"),
                                options: priorities
                            },
                            {
                                userpref: "testRunStatus",
                                label: gadget.getMsg("gadget.common.field.teststatus.label") + ":",
                                description: gadget.getMsg("gadget.common.field.teststatus.desc"),
                                type: "select",
                                selected: gadget.getPref("testRunStatus"),
                                options: statuses
                            },
                            {
                                userpref: "numberResults",
                                label: gadget.getMsg("gadget.common.field.number-of-results.label") + ":",
                                description: gadget.getMsg("gadget.common.field.number-of-results.desc"),
                                type: "text",
                                value: gadget.getPref("numberResults")
                            },
                            AJS.gadget.fields.nowConfigured()
                        ]
                    };
                },
                args: [
                    {
                        key: "savedfilters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "priorities",
                        ajaxOptions: {
                            url: "/rest/api/2/priority",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/raven/1.0/chart/testexecution/filters",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "testStatuses",
                        ajaxOptions: {
                            url: "/rest/raven/1.0/chart/testrun/statuses",
                            contentType: "application/json"
                        }
                    }
                ]
            },
            view: {
                enableReload: true,
                onResizeReload: true,
                onResizeAdjustHeight: false,
                template: function (args) {
                    var gadget = this;
                    // set gadget title
                    setTitle(this);
                    // server errors and timeout errors are not catched by the error handler of the REST date request !!!
                    if (!args.data) {
                        message(null, gadget.getMsg("gadget.common.error.data"));
                    } else {
                        createTestListTable(gadget, args.data);
                    }
                },
                args: [{
                    key: "data",
                    ajaxOptions: function () {
                        var gadget = this;
                        var filterId = gadgets.util.unescapeString(this.getPref("filterId"));
                        var startDate = this.getPref("startDate");
                        var endDate = this.getPref("endDate");
                        var testRunAssignee = this.getPref("testRunAssignee");
                        var executedBy = this.getPref("executedBy");
                        var component = this.getPref("component");
                        var priority = this.getPref("priority");
                        var testRunStatus = this.getPref("testRunStatus");
                        var numberResults = this.getPref("numberResults");
                        //project is not an actual data filter, it only filters components, add this in the future if we decide to use it
                        //var project = this.getPref("project");

                        var params = '';
                        params = appendParameter(params, 'filterId', filterId.substring(7));
                        params = appendParameter(params, 'startDate', startDate);
                        params = appendParameter(params, 'endDate', endDate);
                        params = appendParameter(params, 'testRunAssignee', testRunAssignee);
                        params = appendParameter(params, 'executedBy', executedBy);
                        params = appendParameter(params, 'component', component);
                        params = appendParameter(params, 'priority', priority);
                        params = appendParameter(params, 'testRunStatus', testRunStatus);

                        return {
                            type: "GET",
                            url: contextPath
                            + '/rest/raven/1.0/chart/testrun/list/data'
                            + params,
                            processData: false,
                            async: true,
                            timeout: 120000,
                            contentType: "application/json",
                            error: function (qXHR, textStatus, errorThrown) {
                                if (errorThrown && errorThrown.trim() != '') {
                                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                                } else {
                                    errorThrown = qXHR.responseText;
                                }
                                serverError(gadget, errorThrown);
                                gadgets.window.adjustHeight();
                            }
                        }
                    }
                }]
            }
        });
    }
})(AJS.$);
