(function($) {

    RAVEN.namespace("RAVEN.Gadget");

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]($('#gadget-message'), {
            title: title,
            body: message
        });
    }

    function serverError(gadget, message) {
        var obj = null;
        try {
            obj = JSON.parse(message);
        } catch(e) {}

        var obj = JSON.parse(message);

        var msg = null;
        if (obj && obj.message) {
            msg = obj.message;
        } else {
            msg = gadget.getMsg('gadget.test-runs-summary.common.error.data');
        }

        $('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
        AJS.messages['error']($('#gadget-message'), {
            body: msg
        });
    }

    function clearMessage() {
        $('#gadget-message').empty();
    }

    function createOverallCoverageChart(gadget, data) {
        var content = $('<div id="gadget-content"><div id="gadget-message"></div></div>');
        gadget.getView().html(content);

        if (data && data.labels.length > 0) {
            var content = RAVEN.Gadget.renderTestRunsSummary({'view': gadget.getPref("view")});

            gadget.getView().html(content);
            var chart = $("#test-runs-summary-chart")[0].getContext("2d");
            window.chart = new Chart(chart, {
                type: 'bar',
                data: data,
                options: {
                    scales: {
                        xAxes: [{
                            gridLines: {
                                display:false
                            }
                        }],
                        yAxes: [{
                            stacked: true,
                            ticks: {
                                beginAtZero: true,
                                callback: function(value) {if (value % 1 === 0) {return value;}}
                            }
                        }]
                    },
                    hover :{
                        mode: "single",
                        animationDuration: 0
                    }
                }
            });

            // Render table skeleton
            var tableContent = RAVEN.Gadget.renderTestRunsSummaryTable({'data': data});

            $('#test-runs-summary-table').html(tableContent);
            $('#test-runs-summary-table').xrayDataTable({
                "sPaginationType": "full_numbers",
                "bSort": false,
                "aaSorting": [],
                "bLengthChange": false,
                "bFilter": false,
                "iDisplayLength": parseInt(gadget.getPref("numberResults"))
            });
            $('#test-runs-summary-table').on('draw', function () {
                gadgets.window.adjustHeight();
            });

            if(gadget.getPref("view") === "chart") {
                $('#test-runs-summary-table_wrapper').hide();
            } else {
                $('#test-runs-summary-chart').hide();
            }
        } else {
            message(null, gadget.getMsg("gadget.test-runs-summary.common.empty"), 'info');
        }
    }

    function setTitle(gadget) {
        var customTitle = gadget.getPref("customTitle");
        var useCustomTitle = gadget.getPref("useCustomTitle");
        var filterId = gadget.getPref("filterId");
        var testRunAssignee = gadget.getPref("testRunAssignee");
        var testRunStatus = gadget.getPref("testRunStatus");
        var testRunPriority = gadget.getPref("testRunPriority");
        var grouping = gadget.getPref("grouping");

        var params = "";

        if(useCustomTitle) {
            params = appendParam(params, "useCustomTitle",useCustomTitle);
        }

        if (customTitle && customTitle != '') {
            params = appendParam(params, "customTitle",customTitle);
        }

        if (testRunAssignee && testRunAssignee != '__NONE__') {
            params = appendParam(params, "testRunAssignee", testRunAssignee);
        }

        if (testRunStatus && testRunStatus != '__NONE__') {
            params = appendParam(params, "testRunStatus", testRunStatus);
        }

        if (testRunPriority && testRunPriority != '__NONE__') {
            params = appendParam(params, "testRunPriority", testRunPriority);
        }

        if (grouping) {
            params = appendParam(params, "grouping", grouping);
        }

        if (filterId) {
            // remove prefix "filter-"
            params = appendParam(params, "savedFilter", filterId.substring(7));
        }

        params = encodeURI(params);

        $.ajax({
            type: "GET",
            url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/testrun/title?' + params,
            contentType: "application/json",
            success: function(data) {
                if (data && data.titleParams) {
                    var title = AJS.format(gadget.getMsg("gadget.test-runs-summary.title"), data.titleParams);
                    gadgets.window.setTitle(title);
                } else {
                    var title = gadget.getMsg("gadget.test-runs-summary.title.default");
                    gadgets.window.setTitle(title);
                }
            }
        });
    }

    function appendParam(parameters, paramTitle, paramValue){
        if(parameters === ""){
            return paramTitle + "=" + paramValue;
        }
        return parameters + "&" + paramTitle + "=" + paramValue;
    }

    RAVEN.Gadget.testRunsSummary = function(contextPath) {
        return AJS.Gadget({
            baseUrl: contextPath, /* Used to make base url available to gadget */
            useOauth: "/rest/gadget/1.0/currentUser",
            config: {
                descriptor: function(args) {
                    var gadget = this;

                    $(document).off('click.raven');
                    $(document).off('change.raven');

                    $(document).on('click.raven.view', '#button-view-trigger', function() {
                        if($('#test-runs-summary-table').is(":visible")) {
                            $('#test-runs-summary-table_wrapper').hide();
                            $('#test-runs-summary-chart').show();
                            $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-table"));
                            gadgets.window.adjustHeight();
                            gadget.savePref("view", "chart");
                        } else {
                            $('#test-runs-summary-chart').hide();
                            $('#test-runs-summary-table_wrapper').show();
                            $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-chart"));
                            gadgets.window.adjustHeight();
                            gadget.savePref("view", "table");
                        }
                    });

                    $(document).on('change.raven.title', '#use-custom-title', function() {
                        var useCustomTitle = $("#use-custom-title").attr('checked');
                        if (useCustomTitle) {
                            $("#customTitle").prop('disabled', false);
                        } else {
                            $("#customTitle").prop('disabled', true);
                        }
                    });

                    var customTitleDisabled = true;
                    var useCustomTitle = gadget.getPref("useCustomTitle");
                    if (useCustomTitle === "true") {
                        customTitleDisabled = false;
                    }

                    var priorities = $.map(args.priorities, function(p) {
                        return {
                            label: p.name,
                            value: p.id
                        };
                    });

                    priorities = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(priorities);

                    var statuses = $.map(args.statuses, function(p) {
                        return {
                            label: p,
                            value: p
                        };
                    });

                    statuses = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(statuses);

                    gadget.groupingField = RAVEN.Gadget.fields.TestRunGroupingSelect(gadget, "grouping", function() {
                        return $("#project option:selected").val(); //gadget.getPref("project");
                    });

                    var filters = null;
                    $.each(args.filters.options, function() {
                        if(this.group.label === "Filters") {
                            filters = this.group;
                        }
                    });

                    if (filters) {
                        var filterId = gadget.getPref("filterId");
                        if (filterId) {
                            var filter = _.find(filters.options, function(f) {
                                return f.value == filterId;
                            });
                            if (filter) {
                                gadget.projectOrFilterName = filter.label;
                            }
                        } else {
                            gadget.projectOrFilterName = null;
                        }
                    } else {
                        gadget.projectOrFilterName = null;
                    }

                    var searchParam;
                    if (/^jql-/.test(this.getPref("filterId")) || this.getPref("isPopup") === "true") {
                        searchParam =
                            {
                                userpref: "filterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("filterId"))
                            };
                    }
                    else {
                        searchParam = RAVEN.gadget.fields.filterPicker(gadget, "filterId");
                    }

                    gadget.assigneeField = RAVEN.gadget.fields.assigneePicker(gadget, "testRunAssignee", gadget.getMsg("gadget.common.field.assignee.label") + ":", gadget.getMsg("gadget.common.field.assignee.desc"));

                    return {
                        action: '/rest/raven/1.0/chart/testrun/validate',
                        theme : function(){
                            if (gadgets.window.getViewportDimensions().width < 450){
                                return "gdt top-label";
                            } else{
                                return "gdt";
                            }
                        }(),
                        fields: [
                            {
                                id: "customTitle",
                                userpref: "customTitle",
                                label: gadget.getMsg("gadget.common.field.custom-title.label") + ":",
                                type: "custom",
                                value: gadget.getPref("customTitle"),
                                template: function() {
                                    var disabled = "";
                                    if (customTitleDisabled === true) {
                                        disabled = " disabled"
                                    }
                                    return '<input type="text" class="text" value="' + gadget.getPref("customTitle") + '" id="customTitle" name="customTitle"' + disabled + '>';
                                }
                            },{
                                userpref: "useCustomTitle",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "use-custom-title",
                                        label: gadget.getMsg("gadget.common.field.custom-title.desc"),
                                        value: "true",
                                        selected : gadget.getPref("useCustomTitle") == 'true'
                                    }
                                ]
                            },
                            searchParam,
                            {
                                userpref: "numberResults",
                                label: gadget.getMsg("gadget.common.field.number-of-results.label") + ":",
                                description: gadget.getMsg("gadget.common.field.number-of-results.desc"),
                                type: "text",
                                value: gadget.getPref("numberResults")
                            },
                            gadget.assigneeField,
                            {
                                userpref: "testRunStatus",
                                label: gadget.getMsg("gadget.common.field.status.label") + ":",
                                description: gadget.getMsg("gadget.common.field.status.desc"),
                                type: "select",
                                selected: gadget.getPref("testRunStatus"),
                                options: statuses
                            },
                            {
                                userpref: "testRunPriority",
                                label: gadget.getMsg("gadget.test-runs-summary.common.field.priority.label") + ":",
                                description: gadget.getMsg("gadget.test-runs-summary.common.field.priority.desc"),
                                type: "select",
                                selected: gadget.getPref("testRunPriority"),
                                options: priorities
                            },
                            gadget.groupingField,
                            AJS.gadget.fields.nowConfigured()
                        ]
                    };
                },
                args: [
                    {
                        key: "statuses",
                        ajaxOptions: {
                            url: "/rest/raven/1.0/chart/testrun/statuses",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "priorities",
                        ajaxOptions: {
                            url: "/rest/api/2/priority",
                            contentType: "application/json"
                        }
                    },
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    }
                ]
            },
            view: {
                enableReload: true,
                onResizeReload: true,
                onResizeAdjustHeight: false,
                template: function (args) {
                    var gadget = this;

                    // set gadget title
                    setTitle(this);

                    // server errors and timeout errors are not catched by the error handler of the REST date request !!!
                    if (!args.data) {
                        message(null, gadget.getMsg("gadget.test-runs-summary.common.error.data"));
                    } else {
                        createOverallCoverageChart(gadget, args.data);

                        $(document).off('click.raven');

                        $(document).on('click.raven.view', '#button-view-trigger', function() {
                            if($('#test-runs-summary-table').is(":visible")) {
                                $('#test-runs-summary-table_wrapper').hide();
                                $('#test-runs-summary-chart').show();
                                $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-table"));
                                gadget.savePref("view", "chart");
                            } else {
                                $('#test-runs-summary-chart').hide();
                                $('#test-runs-summary-table_wrapper').show();
                                $('#button-view-trigger').text(gadget.getMsg("gadget.common.show-as-chart"));
                                gadget.savePref("view", "table");
                            }
                            gadgets.window.adjustHeight();
                        });
                    }
                },
                args: [{
                    key: "data",
                    ajaxOptions: function() {
                        var gadget = this;

                        var testRunAssignee = gadgets.util.unescapeString(this.getPref("testRunAssignee"));
                        var testRunStatus = this.getPref("testRunStatus");
                        var testRunPriority = this.getPref("testRunPriority");
                        var filterId = gadgets.util.unescapeString(this.getPref("filterId"));

                        var params = '';

                        if (testRunAssignee && testRunAssignee !== '__NONE__') {
                            params = params + '&testRunAssignee=' + testRunAssignee;
                        }

                        if (testRunStatus && testRunStatus !== '__ALL__') {
                            params = params + '&testRunStatus=' + testRunStatus;
                        }

                        if (testRunPriority && testRunPriority !== '__ALL__') {
                            params = params + '&testRunPriority=' + testRunPriority;
                        }

                        if (filterId) {
                            // remove prefix "filter-"
                            params = params + "&savedFilter=" + filterId.substring(7);
                        }

                        return {
                            type: "GET",
                            url: contextPath + '/rest/raven/1.0/chart/testrun/summary?groupingField=' + this.getPref("grouping") + '&numberResults=' + this.getPref("numberResults") + params,
                            processData: false,
                            async: true,
                            timeout: 120000,
                            contentType: "application/json",
                            error: function(qXHR, textStatus, errorThrown) {
                                if (errorThrown && errorThrown.trim() != '') {
                                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                                } else {
                                    errorThrown = qXHR.responseText;
                                }
                                serverError(gadget, errorThrown);
                                gadgets.window.adjustHeight();
                            }
                        }
                    }
                }]
            }
        });
    }
})(AJS.$);
