(function($) {

    window.RAVEN = window.RAVEN || {};

    var view = null;
    var statusView = null;

    var strategyType = "version";
    var strategyValue = "no-version";
    var environment = "-1";

    function getIssueKey() {
        return AJS.$("#issue-key").val();
    }

    function getIssueId() {
        return AJS.$("#issue-id").val();
    }

    function isRequirement() {
        return AJS.$("#is-requirement").val() == "true";
    }

    function getTestIssueId() {
        return AJS.$("#test-issue-id").val();
    }

    function getSubTaskTestExecIssueId() {
        return AJS.$("#subtask-testexec-issue-id").val();
    }

    function hasSubTaskTestExecutionIssueType() {
        return AJS.$("#has-subtask-testexec-issue-type").val() == "true";
    }

    function subTaskTestExecTestsCustomFieldId() {
        return AJS.$("#raven-subtask-testexec-tests-customFieldId").val();
    }

    function getVersionName() {
        return AJS.$("#version-name").val();
    }

    function getVersionId() {
        return AJS.$("#version-id").val();
    }

    function getPopulateObject() {
        return JSON.parse(AJS.$("#issue-dialog-populate-object").val());
    }

    function getEnvironments() {
        return JSON.parse(AJS.$("#xray-environments").val());
    }

    function getVersions() {
        var versions;
        $.ajax({
            type: "GET",
            url: contextPath + '/rest/raven/1.0/requirement/' + getIssueKey() + "/strategy/version",
            async: false,
            contentType: "application/json",
            success: function(data, textStatus, jqXHR) {
                versions = data;
            },
        });
        return versions;
    }

    var testCoverageModel = Backbone.Model.extend({
        priority: null,
        key: null,
        summary: null,
        status: null,
        resolution: null
    });

    var testCoverageCollection = Backbone.Collection.extend({
        model: testCoverageModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/requirement/' + getIssueKey() + '/tests';
        }
    });

    var testCoverageCollection = Backbone.Collection.extend({
        model: testCoverageModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/requirement/' + getIssueKey() + '/tests?' + strategyType + "=" + strategyValue;
        }
    });

    var testCoverageView = Backbone.View.extend({
        model: new testCoverageCollection(),
        el: $('#test-coverage-table'),

        initialize: function() {
            var self = this;
            params = $.param({ environment: environment }) + "&" + strategyType + "=" + strategyValue;
            this.model.fetch({
                data: params,
                success: function() {
                    self.render();
                },
            });
        },

        render: function() {
            $('#test-coverage-table').empty();
            // empty table
            if (this.model.length == 0) {
                $('#test-coverage-no-tests').show();
                $('#test-coverage-filters').hide();
            } else {
                $('#test-coverage-no-tests').hide();
                $('#test-coverage-filters').show();
                // render table skeleton
                var tableHtml = RAVEN.RequirementTestCoverage.renderTestCoverageTable();
                $('#test-coverage-table').html(tableHtml);

                var $tbody = $('#test-coverage-table').find('tbody');

                var rows = this.model.toArray();
                if (rows && rows.length > 0) {
                    _.each(rows, function(test, i) {
                        var status = test.get("workflowStatus");

                        $tbody.append(RAVEN.RequirementTestCoverage.renderTestCoverageRow({
                            rowInfo: test.attributes,
                            status: status,
                            contextPath: AJS.contextPath()
                        }));

                        $("#test-coverage-summary-cell").tooltip();
                        $("#test-coverage-key-cell").tooltip();
                        $("#test-coverage-status-cell").tooltip();
                    });
                }

                $("#test-coverage-filters").show();
            }

            return this;
        }
    });

    var requirementStatusModel = Backbone.Model.extend({
        status: null,
        url: function() {
            return contextPath + '/rest/raven/1.0/requirement/' + getIssueKey() + '/status?';
        }
    });

    var requirementStatusView = Backbone.View.extend({
        model: new requirementStatusModel(),
        el: $('#test-coverage-table'),

        initialize: function() {
            var self = this;
            params = $.param({ environment: environment }) + "&" + strategyType + "=" + strategyValue;
            this.model.fetch({
                data: params,
                success: function() {
                    self.render();
                },
            });
        },

        render: function() {
            $('#requirement-status-field').empty();
            // empty table
            var html = RAVEN.RequirementTestCoverage.renderStatusCell({
                status: this.model.attributes
            });
            $('#requirement-status-field').html(html);

            return this;
        }
    });

    function renderStrategyPicker() {
        if ($("#strategy-picker-field").html() === null) {
            var $element = $('#test-coverage').find("#strategy-picker");
            $element.on("change", function(e, field) {
                strategyType = $('#test-coverage').find("#strategy-picker").val()[0];
                if (strategyType === "version") {
                    strategyValue = "no-version";
                    $("#test-coverage-table").block();
                    refresh();
                    $("#test-coverage-table").unblock();
                }

                renderStrategyValuePicker();
            });

            new RAVEN.SubtleSelect({
                element: $element,
                showDropdownButton: false,
                revertOnInvalid: true,
            });
        }
    }

    function renderStrategyValuePicker() {
        switch (strategyType) {
            case "version":
                $('#test-coverage').find("#strategy-value-field").html(
                    RAVEN.RequirementTestCoverage.versionStrategyValuePicker({
                        versions: getVersions(),
                        selected: strategyValue
                    }));
                new AJS.SingleSelect({
                    element: $('#test-coverage').find("#strategy-value-picker"),
                    showDropdownButton: true,
                    revertOnInvalid: true,
                });
                break;
            case "sprint":
                $('#test-coverage').find("#strategy-value-field").html(
                    RAVEN.RequirementTestCoverage.sprintStrategyValuePicker());
                new AJS.SingleSelect({
                    element: $('#test-coverage').find("#strategy-value-picker"),
                    showDropdownButton: true,
                    revertOnInvalid: true,
                });
                break;
            case "testPlan":
                $('#test-coverage').find("#strategy-value-field").html(
                    RAVEN.RequirementTestCoverage.testPlanStrategyValuePicker({
                        contextPath: "",
                        issueKey: "",
                        label: "",
                        issueType: ""
                    }));

                this.strategyPicker = new AJS.SingleSelect({
                    element: $('#strategy-value-picker'),
                    showDropdownButton: true,
                    submitInputVal: true,
                    noQueryNoRequest: false,
                    revertOnInvalid: true,
                    loadOnInit: true,
                    ajaxOptions: {
                        query: true,
                        url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                        formatResponse: function(data) {
                            var section = data.sections[0];
                            var optgroup = new AJS.GroupDescriptor({
                                weight: 0,
                                id: section.id + "-opt-group",
                                replace: true,
                                label: section.label + "(" + section.sub + ")"
                            });

                            $(section.issues).each(function(i, issue) {
                                var item = new AJS.ItemDescriptor({
                                    value: issue.key,
                                    label: issue.key + ' - ' + issue.summaryText,
                                    title: issue.summary,
                                    icon: AJS.contextPath() + issue.img,
                                    allowDuplicate: false,
                                    selected: issue.key === strategyValue[0] ? 'selected' : null
                                });
                                optgroup.addItem(item);
                            });
                            return [optgroup];
                        }
                    }
                });
                $("#strategy-value-picker-field").attr("placeholder", AJS.I18n.getText("view-issue-requirement-test-coverage-section.placeholer.strategy.test-plan"));
                break;
        }

        $('#test-coverage').find("#strategy-value-field").append(
            RAVEN.RequirementTestCoverage.environmentStrategyValuePicker({
                environments: getEnvironments(),
                selected: environment
            }));

        $('#test-coverage').find("#strategy-value-picker").on("change", function(e, field) {
            strategyValue = $('#test-coverage').find("#strategy-value-picker").val();
            $("#test-coverage-table").block();
            refresh();
            $("#test-coverage-table").unblock();
        });

        this.environmentPicker = createSubtleSingleSelect($('#raven-report-analysis-environment-select'));
        $(this.environmentPicker).on('selectedItemChanged', function(e) {
            environment = e.currentTarget.selectedItem.id;
            $("#test-coverage-table").block();
            refresh();
            $("#test-coverage-table").unblock();
            /*self.environment = e.currentTarget.selectedItem.id;
            self.render();*/
        });
    }

    function createSubtleSingleSelect($selectField) {
        var items = _.map($selectField.find('option'), function(item) {
            return {
                'id': item.value,
                'name': item.text,
            };
        });

        var selectedItem = _.first(_.map($selectField.find('option:selected'), function(item) {
            return {
                'id': item.value,
                'name': item.text,
            };
        }));

        var subtlePicker = new RAVEN.Field.SubtleSingleSelect(items, selectedItem, $selectField.closest('#raven-subtle-container'));
        subtlePicker.render();

        // remove select element. This is only for retrieving the initial items.
        $selectField.remove();

        return subtlePicker;
    }

    function renderViewOptions() {
        $("#view-issue-requirement-test-coverage-section_heading .ops").html(RAVEN.RequirementTestCoverage.renderViewOptions({
            issueId: getIssueId(),
            hasSubTaskTestExecutionIssueType: hasSubTaskTestExecutionIssueType(),
            contextPath: AJS.contextPath()
        }));
    };

    function initialize() {
        renderViewOptions();
        AJS.messages.setup();
        renderStrategyPicker();
        renderStrategyValuePicker();
        view = new testCoverageView();
        statusView = new requirementStatusView();

        $(document).bind("dialogContentReady", processDialogContentReadyNewTest);
    };

    function refresh() {
        view = new testCoverageView();
        statusView = new requirementStatusView();

        $(document).bind("dialogContentReady", processDialogContentReadyNewTest);
        $("#test-coverage-table").unblock();
    };

    function processDialogContentReadyNewTest(e, dialog) {
        // just to make sure its the right dialog
        var popup = dialog.$popup[0];
        if (popup.id == 'create-issue-dialog' && popup.classList.contains('raven-create-test-dialog')) {
            var populateObj = getPopulateObject();

            // links
            $(popup).find("#issuelinks-linktype").val("tests");
            $(popup).find("#issuelinks-issues").trigger('selectOption', [{ label: getIssueKey(), value: getIssueKey(), title: getIssueKey() }]);

            $(popup).find("#summary").val("Test " + populateObj.summary);

            // Description
            if (populateObj.description && populateObj.description !== "") {
                $(popup).find("#description").val("Tests " + populateObj.description);
                $(popup).find("div[field-id=description] rich-editor").html(populateObj.descriptionRendered);
            }

            // Environment
            if (populateObj.environment && populateObj.environment !== "") {
                $(popup).find("#environment").val(populateObj.environment);
            }

            $(populateObj.components).each(function(index, obj) {
                var jsonObj = JSON.parse(obj);
                $(popup).find("#components").trigger('selectOption', [{ label: jsonObj.name, value: jsonObj.id, title: jsonObj.name }]);
            });

            $(populateObj.labels).each(function(index, obj) {
                $(popup).find("#labels").trigger('selectOption', [{ label: obj, value: obj, title: obj }]);
            });

            $(populateObj.fixedVersions).each(function(index, obj) {
                var jsonObj = JSON.parse(obj);
                AJS.$('select[id="fixVersions"] option').each(function() {
                    //If the name of version is the same then we choose it.
                    //this is only using names because identical versions in differents projects have not the same ID
                    if (jQuery.trim($(this).text()) == jsonObj.name) {
                        $(popup).find("#fixVersions").trigger('selectOption', [{ value: $(this).val(), title: jsonObj.name, label: jsonObj.name }]);
                    }
                });

            });
        }
    }

    function processDialogContentReadyNewSubTaskTestExecution(e, dialog) {
        $(document).unbind("dialogContentReady", processDialogContentReadyNewSubTaskTestExecution);
        // just to make sure its the right dialog
        var popup = dialog.$popup[0];
        if (popup.id == 'create-subtask-dialog' && popup.classList.contains('raven-create-subtask-testexec-dialog')) {
            // change the selected issuetype to Sub-Task Test Execution
            $(popup).find("#summary").val(AJS.I18n.getText("view-issue-requirement-test-coverage-section.create-subtask-testexec.summary", getIssueKey()));

            var populateObj = getPopulateObject();
            $(populateObj.fixedVersions).each(function(index, obj) {
                var jsonObj = JSON.parse(obj);
                AJS.$('select[id="fixVersions"] option').each(function() {
                    //If the name of version is the same then we choose it.
                    //this is only using names because identical versions in differents projects have not the same ID
                    if (jQuery.trim($(this).text()) == jsonObj.name) {
                        $(popup).find("#fixVersions").trigger('selectOption', [{ value: $(this).val(), title: jsonObj.name, label: jsonObj.name }]);
                    }
                });

            });

            AJS.$('input[id="issuetype"]').trigger('set-selection-value', getSubTaskTestExecIssueId());

            setTimeout(populateTests, 1000);
        }

    }

    function populateTests() {
        if ($("#create-subtask-dialog").find('select[id=' + subTaskTestExecTestsCustomFieldId() + ']').length == 0) {
            setTimeout(populateTests, 1000);
        } else {
            var model = view.model.models;
            model.forEach(function(element) {
                var key = element.attributes.key;
                $("#create-subtask-dialog").find('select[id=' + subTaskTestExecTestsCustomFieldId() + ']').trigger('selectOption', [{ label: key, value: key, title: key }]);
            }, this);
        }
    }

    function processDialogContentReadyLinkTest(e, dialog) {
        $(document).unbind("dialogContentReady", processDialogContentReadyLinkTest);
        // just to make sure its the right dialog
        var popup = dialog.$popup[0];
        if (popup.id == "link-issue-dialog") {
            $(popup).find("#link-type").val("tested by");
        }
    }

    function processNewTest() {
        var createForm = JIRA.Forms.createCreateIssueForm({
            issueType: getTestIssueId(),
        });

        var dialog = createForm.asDialog({
            id: 'create-issue-dialog'
        });

        dialog.show();

        $('#create-issue-dialog').addClass("raven-create-test-dialog");

        // Refresh on issue create
        JIRA.bind("QuickCreateIssue.sessionComplete", processTestCreated);
    };

    function processTestCreated() {
        JIRA.unbind("QuickCreateIssue.sessionComplete", processTestCreated);
        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
        initialize();
    }

    function processNewSubTaskTestExec() {
        var createForm = JIRA.Forms.createSubtaskForm({
            parentIssueId: getIssueId()
        });

        var dialog = createForm.asDialog({
            id: 'create-subtask-dialog'
        });

        $(document).bind("dialogContentReady", processDialogContentReadyNewSubTaskTestExecution);

        dialog.show();

        $('#create-subtask-dialog').addClass("raven-create-subtask-testexec-dialog");
    };

    function processLinkTest() {
        $(document).bind("dialogContentReady", processDialogContentReadyLinkTest);
    };

    $(document).on('click', '#test-coverage-raven-create-test', function() {
        processNewTest();
    });

    $(document).on('click', '#test-coverage-raven-create-subtask-testexec', function() {
        processNewSubTaskTestExec();
    });

    $(document).on('click', '#raven-create-subtask-testexec', function() {
        processNewSubTaskTestExec();
    });

    $(document).on('click', '#raven-link-test', function() {
        processLinkTest();
    });

    // register for NEW_CONTENT_ADDED
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
        if (isRequirement()) {
            var $section = $("#test-coverage");
            // reset values needed for issue search details view
            if (reason == JIRA.CONTENT_ADDED_REASON.pageLoad) {
                if ($section.length > 0 && !$section.attr('data-initialized')) {
                    strategyType = "version";
                    strategyValue = "no-version";
                    initialize();
                    $section.attr("data-initialized", "true");
                }
            } else if (reason == JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
                if ($section.length > 0 && !$section.attr('data-initialized')) {
                    strategyType = "version";
                    strategyValue = "no-version";
                    initialize();
                    $section.attr("data-initialized", "true");
                }
            }
        }
    });

})(AJS.$);