(function($) {

    var fancyboxDefaultOpts = {
            type: "image",
            centerOnScroll: true,
            // disable JIRA's keyboard shortcuts when displaying a lightbox
            onStart : AJS.disableKeyboardShortcuts,
            onClosed : AJS.enableKeyboardShortcuts
    };

    var imageRegex = /([^\s]+(\.(jp(e|g|eg)|gif|png|bmp|jfif|svg|ico))$)/;

	var canEditTest = false;

	var selectedTestType = null;
	var manualStepsTable;
	var inlineEdit = null;
	var testTypeField = null;
	var cucumberTypeField = null;
	var genericDefField = null;
	var cucumberStepsField = null;

	function getTestIssueKey() {
		return $('#raven-test-details-issue-key').val();
	}

    function isManualTestTypeOption(value) {
        var manualOptions = $("#manual-options").val();
        if (manualOptions) {
            var ids = JSON.parse(manualOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isCucumberTestTypeOption(value) {
        var cucumberOptions = $("#cucumber-options").val();
        if (cucumberOptions) {
            var ids = JSON.parse(cucumberOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isOtherTestTypeOption(value) {
        return isCucumberTestTypeOption(value) == false && isManualTestTypeOption(value) == false;
    }

    function compareTypeOptions(value1, value2) {
        return (isManualTestTypeOption(value1) === true && isManualTestTypeOption(value2) === true)
            || (isCucumberTestTypeOption(value1) === true && isCucumberTestTypeOption(value2) === true)
            || (isOtherTestTypeOption(value1) === true && isOtherTestTypeOption(value2) === true);
    }

	function hasPrecondition() {
		var preConditionIssueKey = JSON.parse($('#test-details-precondition-issue-key').val());
		if (preConditionIssueKey !== null && preConditionIssueKey.length > 0) {
			return true;
		}
		return false;
	}

	function deleteTestInformation() {
		return $('#delete-test-definition').val() === "true";
	}

	function renderPathAsText() {
		return $('#render-path-as-text').val() === "true";
	}

	function renderRichText() {
		return $("#use-rich-text-editor").val() === "true";
	}

	function clearMessage() {
        $('#raven-error-message').remove();
	}

	function message(title, message, type) {
		clearMessage();
		AJS.messages.error('#raven-test-details', {
			title: title,
			body: message
		});
	}

	var IndexCustomReadView = RAVEN.RestfulTable.CustomReadView.extend({
		render: function (self) {
			return $("<div class=\"index\">" + self.value + "</div>");
		}
	});

	var IndexCustomEditView = RAVEN.RestfulTable.CustomEditView.extend({
		render: function (self) {
			if (self.value) {
				return $("<div class=\"index\">" + self.value + "</div>");
			} else {
				return '';
			}
		}
	});

	var fnAttachmentsView = function (self, model) {
		var attachs = [];
		if (self.value) {
			attachs = self.value;
		}
		var content = RAVEN.Attachments.createStepAttachments({
			entityType: 'TEST_STEP',
			stepId: self.model.id,
			issueKey: getTestIssueKey(),
			attachments: attachs,
			contextPath: AJS.contextPath(),
			readOnly: !canEditTest
		});
		return $(content);
	};

	var attachmentsReadView = RAVEN.RestfulTable.CustomReadView.extend({
		render: fnAttachmentsView
	});

	var attachmentsEditView = RAVEN.RestfulTable.CustomEditView.extend({
		render: fnAttachmentsView
	});

	var attachmentsCreateView = RAVEN.RestfulTable.CustomCreateView.extend({
		render: function(self, model) {
			return '';
		}
	});

	var stepEditView = new RAVEN.RestfulTable.CustomEditView.extend({
		render: function (self) {
			var value = (self.value == undefined) ? "" : self.value;
			return $('<textarea style="height:50px;width:100%;" name="step">'+value+'</textarea>'); //.autoresize();
		}
	});

	var dataEditView = RAVEN.RestfulTable.CustomEditView.extend({
		render: function (self) {
			var value = (self.value == undefined) ? "" : self.value;
			return $('<textarea style="height:50px;width:100%;" name="data">'+value+'</textarea>'); //.autoresize();
		}
	});

	var resultEditView = RAVEN.RestfulTable.CustomEditView.extend({
		render: function (self) {
			var value = (self.value == undefined) ? "" : self.value;
			return $('<textarea style="height:50px; width:100%;" name="result">'+value+'</textarea>'); //.autoresize();
		}
	});

	function deleteStepConfirmation(model) {
		var title = AJS.I18n.getText('view-issue-test-section.steps.delete.confirm.title');
		var severity = "info";
		var message = AJS.I18n.getText('view-issue-test-section.steps.delete.confirm.message', getTestIssueKey());
		return RAVEN.Dialog.deleteConfirmationDialog({title:title, message:message, severity:severity});
	}

	function cloneStepConfirmation(model) {
		var title = AJS.I18n.getText('view-issue-test-section.steps.clone.confirm.title');
		var severity = "info";
		var message = AJS.I18n.getText('view-issue-test-section.steps.clone.confirm.message', getTestIssueKey());
		return RAVEN.Dialog.cloneConfirmationDialog({title:title, message:message, severity:severity});
	}

	function createTestStepsRESTfulTable() {
		var tableActions = canEditTest;


		var url = AJS.contextPath() + "/rest/raven/1.0/test/" + getTestIssueKey() + "/steps";
		manualStepsTable = new RAVEN.RestfulTable({
			autoFocus : true,
			el : $("#manual-steps-table"),
			remainOrder: true,
			createPosition : "bottom",
			deleteConfirmation: deleteStepConfirmation,
			cloneConfirmation: cloneStepConfirmation,
			resources : {
				all : url,
				self : url
			},
			allowReorder : tableActions,
			allowEdit : tableActions,
			allowClone : tableActions,
			allowDelete : tableActions,
			allowCreate : tableActions,
			columns : [
			    {
					id : "index",
					header : "",
					allowEdit: false,
					readView: IndexCustomReadView,
					editView: IndexCustomEditView
				},
				{
					id : "step",
					header : AJS.I18n.getText('common.label.step'),
					allowEllipsize: true,
					inlineDialog: {
						name: AJS.I18n.getText('common.label.step')
					},
					readView : RAVEN.RestfulTable.HtmlWikiReadView,
					editView : RAVEN.RestfulTable.InlineDialogEditView,
					allowCopy : {
						id:'step',
						getCopyValue: function(value) {
							if (value && value.raw) {
            					return value.raw;
        					}
        					return null;
						}
					}
				},
				{
					id : "data",
					header : AJS.I18n.getText('view-issue-testdetails.header.data'),
					allowEllipsize: true,
					inlineDialog: {
						name: AJS.I18n.getText('view-issue-testdetails.header.data')
					},
					readView : RAVEN.RestfulTable.HtmlWikiReadView,
					editView : RAVEN.RestfulTable.InlineDialogEditView,
					allowCopy : {
						id:'data',
						getCopyValue: function(value) {
							if (value && value.raw) {
            					return value.raw;
        					}
        					return null;
						}
					}
				},
				{
					id : "result",
					header : AJS.I18n.getText('view-issue-testdetails.header.expected-result'),
					allowEllipsize: true,
					inlineDialog: {
						name: AJS.I18n.getText('view-issue-testdetails.header.expected-result')
					},
					readView : RAVEN.RestfulTable.HtmlWikiReadView,
					editView : RAVEN.RestfulTable.InlineDialogEditView,
					allowCopy : {
						id:'result',
						getCopyValue: function(value) {
							if (value && value.raw) {
            					return value.raw;
        					}
        					return null;
						}
					}
				},
				{
					id : "attachments",
					header: AJS.I18n.getText('execute-test-action.stepstable.attachments'),
					readView: attachmentsReadView,
					editView: attachmentsEditView,
					createView: attachmentsCreateView,
					allowEdit: false
				}
			],
			noEntriesMsg : AJS.I18n.getText('view-issue-test-section.manualsteps.nosteps')
		});

		var limitedTestCellHeight = $("#limited-test-cell-height").val();

        if( limitedTestCellHeight == "false" ) {
            manualStepsTable.classNames.CONTENT = "raven-restfultable-content-unlimited-height";
        }

        $(manualStepsTable)
            .on(RAVEN.RestfulTable.Events.REORDER_SUCCESS, function () {
                updateHistory();
                updateDates();
                adjustAttachmentsColumn();
            })
            .on(RAVEN.RestfulTable.Events.ROW_ADDED, function () {
                updateHistory();
                updateDates();
                adjustAttachmentsColumn();
            })
            .on(RAVEN.RestfulTable.Events.ROW_REMOVED, function () {
				updateHistory();
				updateDates();
			})
			.on(RAVEN.RestfulTable.Events.ROW_UPDATED, function () {
				updateHistory();
				updateDates();
                adjustAttachmentsColumn();
			})
            .on(RAVEN.RestfulTable.Events.INITIALIZED, function () {
                adjustAttachmentsColumn();
            });

        $(document).on(RAVEN.RestfulTable.Events.EDIT_FOCUS, function () {
            adjustAttachmentsColumn();
        });
	}

	function afterInitRestfultable () {
		$("#manual-steps-table").bind(RAVEN.RestfulTable.Events.INITIALIZED, function () {
			applyFancybox();
		});
	}

	// Applies Fancybox to all supported formats
	function applyFancybox() {
		$(".item-attachments.step-attachments a").each(
			function(key, value){
				if(imageRegex.test(value.href.toLowerCase())){
					$(value).fancybox(fancyboxDefaultOpts);
				}
		});
	}

	function resetTestStepsRESTfulTable() {
		$("#manual-steps-table").empty();
		deleteAllSteps(createTestStepsRESTfulTable);
	}

	function updateHistory() {
		if (AJS.$('#changehistory-tabpanel').attr('class') == 'active') {
			var alt_token = AJS.$('input[name="atl_token"]').val();
			$.ajax({
				type : "GET",
				url : contextPath + '/browse/' + getTestIssueKey() +"?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="+alt_token,
				contentType : "text/plain",
				success : function (data, status) {
					var tab = AJS.$(data).find('#activitymodule div.mod-content #issue_actions_container');
					var jiraTab = AJS.$('#activitymodule div.mod-content').find('#issue_actions_container');
					$(jiraTab).html(tab);
				}
			});
		}
	}

	function updateDates() {

		var updatedDate = null;

		if(AJS.$('#datesmodule #updated-date').length === 1) {
			updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
		} else if(AJS.$('#datesmodule #updated-val').length === 1) {
			updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
		} else {
			return null;
		}

		$.ajax({
			type : "GET",
			url : AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
			contentType : "application/json",
			success : function (data) {
				var title = data.title;
				var dateTimeIso8601 = data.dateTimeIso8601;
				updatedDate.attr("datetime", dateTimeIso8601);
				$.each(AJS.$('#datesmodule dd.date'), function() {
                    if($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                        $(this).attr("title", title);
                    }
				});
				updatedDate.livestamp();
			}
		});
	}

	// we only get here if a change really happens
	function changeViewMode(saveCallback, cancelCallback) {
		// Check if it is necessary to show a confirmation message
		var isFieldEmpty = false;
		if (isManualTestTypeOption(selectedTestType) === true) {
			if($('#manual-steps-table').find('.raven-restfultable-no-entires').length > 0) {
				isFieldEmpty = true;
			}
		} else if (isCucumberTestTypeOption(selectedTestType) === true) {
			if (!$.trim(cucumberStepsField.getValue())) { // based on falsy
				isFieldEmpty = true;
			}
		} else {
			if (!$.trim(genericDefField.getValue())) { // based on falsy
				isFieldEmpty = true;
			}
		}

        var $typeField = $('#test-type-select');
        var typeValue = $typeField.val();
        var previousValue = selectedTestType;

        var showMessage = (!isFieldEmpty && !compareTypeOptions(typeValue, previousValue)) || (hasPrecondition() && !compareTypeOptions(typeValue, previousValue));

		if (showMessage && (hasPrecondition() || deleteTestInformation())) {
			var currentType = $('#test-type-select').find('option:selected').attr('label');
			var title, msg;
			if(deleteTestInformation()) {
				title = AJS.I18n.getText('view-issue-test-section.dialogs.changetype.title', currentType);
				msg = AJS.I18n.getText('view-issue-test-section.dialogs.changetype.generic.message');
			} else if (hasPrecondition()) {
				title = AJS.I18n.getText('view-issue-test-section.dialogs.preconditions-mismatch.title', currentType);
				msg = AJS.I18n.getText('view-issue-test-section.dialogs.preconditions-mismatch.message');
			}

			confirmationChangeTestType(title, msg, saveCallback, cancelCallback);
		} else {
			saveCallback();
		}
	}

	function confirmationChangeTestType(title, message, saveCallback, cancelCallback) {
		var dialog = new JIRA.FormDialog({
			id: 'raven-change-test-type',
			content: function (callback) {
        		var innerHtml = RAVEN.Dialog.confirmationDialog({title:title, message:message});
				callback(innerHtml);
			},
			submitHandler: function (e) {
        		saveCallback();
        		dialog.hide();
				e.preventDefault();
			},
			autoClose : true
		});
		dialog.bind('Dialog.hide', function(e, popup, hideReason) {
			if (hideReason === JIRA.Dialog.HIDE_REASON.cancel || hideReason === JIRA.Dialog.HIDE_REASON.escape) {
				cancelCallback();
			}
		});

		dialog.show();
	}

	function deleteAllSteps(complete) {
		$.ajax({
			type : "DELETE",
			url : contextPath + '/rest/raven/1.0/test/' + getTestIssueKey() + "/steps",
			processData : false,
			async : true,
			contentType : "application/json",
			error : function(qXHR, textStatus, errorThrown) {
				$('#raven-error-message').remove(); // clear previous message
				if (errorThrown && errorThrown.trim() != '') {
					errorThrown = errorThrown + ': ' + qXHR.responseText;
				} else {
					errorThrown = qXHR.responseText;
				}
				AJS.messages.error({
					title : AJS.I18n.getText('view-issue-test-section.manualsteps.reset.error', getTestIssueKey()),
					body : errorThrown,
					id : "raven-error-message"
				});
			},
			complete : complete
		});
		updateHistory();
		updateDates();
	}

	function deleteAttachmentFromStep(options) {
		$('#raven-test-details').block();

		clearMessage();

		var id = options.$el.val();
		var path = contextPath + '/rest/raven/1.0/test/' + getTestIssueKey() + '/steps/' + options.stepId + '/attachment/' + id;

		$.ajax({
    	    type: "DELETE",
    	    url: path,
    	    processData: false,
    	    async: true,
    	    contentType: "application/json",
			error: function(qXHR, textStatus, errorThrown) {
				if (errorThrown && errorThrown.trim() != '') {
					errorThrown = errorThrown + ': ' + qXHR.respons0eText;
				} else {
					errorThrown = qXHR.responseText;
				}

				message(AJS.I18n.getText('view-issue-test-section.manualsteps.attachments.delete.error'), errorThrown)
			},
			success: function(data, textStatus, jqXHR) {
				var rowElem = $(options.$el).closest('tr');
				var row = manualStepsTable.getRowFromElement(rowElem);
				// update backbone model
				var attachments = row.model.get("attachments");
				var newAttachments = $.grep(attachments, function(e){
					return e.id !== id;
				});
				row.model.set("attachments", newAttachments);

				$(options.$el).remove();
			},
			complete: function() {
				$('#raven-test-details').unblock();
			}
    	});
	}

	function refreshStepAttachmentsFromServer(row) {
		$('#raven-test-details').block();

		clearMessage();

		var stepId = row.model.id;
		var path = contextPath + '/rest/raven/1.0/test/' + getTestIssueKey() + '/steps/' + stepId + '/attachment';

		$.ajax({
    	    type: "GET",
    	    url: path,
    	    processData: true,
    	    async: true,
    	    contentType: "application/json",
			error: function(qXHR, textStatus, errorThrown) {
				if (errorThrown && errorThrown.trim() != '') {
					errorThrown = errorThrown + ': ' + qXHR.responseText;
				} else {
					errorThrown = qXHR.responseText;
				}

				message(AJS.I18n.getText('view-issue-test-section.manualsteps.attachments.refresh.error'), errorThrown)
			},
			success: function(data, textStatus, jqXHR) {
				// update backbone model
				row.model.set("attachments", data);

				// attachments' cell is ALWAYS the 5th DIRECT td child of any tr.raven-restfultable-row
				var cell = $(manualStepsTable.$table).find('tr[data-id="' + stepId + '"] > td').get(5);
				$(cell).empty();
				var innerHtml = RAVEN.Attachments.createStepAttachments({
					entityType: 'TEST_STEP',
					stepId: stepId,
					issueKey: getTestIssueKey(),
					attachments: data,
					contextPath: AJS.contextPath() });
				$(cell).html(innerHtml);
			},
			complete: function() {
				applyFancybox();
				$('#raven-test-details').unblock();
                adjustAttachmentsColumn();
			}
    	});
	}

	function initializeStepAttachmentsDialog() {
		var attachDialog = new JIRA.FormDialog({
            id: "raven-attach-file-dialog",
            trigger: ".add-step-attachments",
            ajaxOptions: JIRA.Dialogs.getDefaultAjaxOptions,
            onSuccessfulSubmit : function() {
				var row = manualStepsTable.getRowFromElement($(this.$activeTrigger).closest('tr'));
				refreshStepAttachmentsFromServer(row);
			},
            onContentRefresh: function () {
                jQuery(".overflow-ellipsis").textOverflow();
            },
			onDialogFinished : function () {
			},
			autoClose : true
        });
	}

	function deleteAttachmentConfirmationDialog(attachmentId, attachmentName, fnDeleteAttachment, options) {
		var dialog = new JIRA.FormDialog({
			id: 'delete-attachment-dialog-' + attachmentId,
			content: function (callback) {
        		var innerHtml = RAVEN.Attachments.deleteAttachmentConfirmationDialog({attachmentName:attachmentName});
				callback(innerHtml);
			},
			submitHandler: function (e) {
        		fnDeleteAttachment(options);
        		dialog.hide();
				e.preventDefault();
			},
			autoClose : true
		});

		dialog.show();
	}

	function initView() {
		canEditTest = $('#can-edit-test').val() === 'true';

		selectedTestType = $('#test-type-field-value').val();

		// if test is not editable, display message informing the user that this test is not editable
		if (canEditTest === false) {
			AJS.messages.info({
				body : AJS.I18n.getText('view-issue-test-section.readonly.message'),
				id : "raven-info-message",
				closeable: false
			});
		} else {
			$('#raven-info-message').remove(); // clear previous
		}

		// set view mode acording to the selected test type
		toogleViewMode(false);

		initInlineEditFields();

		createTestStepsRESTfulTable();
		afterInitRestfultable();
	}

	function initInlineEditFields() {
		// create Inline Edit
		testTypeField = new RAVEN.InlineEdit.SelectPickerField(
			$('#test-type-field'),
			{
				validate: function(saveCallback, cancelCallback) {
					changeViewMode(saveCallback, cancelCallback);
				},
				success: function() {
					var newSelectedValue = $('#test-type-select').val();
					selectedTestType = newSelectedValue;
					toogleViewMode(true);
					updateHistory();
					JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
					clearMessage();
				},
				error: fieldSaveError
			}
		);
		cucumberTypeField = new RAVEN.InlineEdit.SelectPickerField(
			$('#automated-test-type-field'),
			{
				success: function() {
					updateHistory();
					updateDates();
					clearMessage();
				},
				error: fieldSaveError
			}
		);
		if(!renderPathAsText()) {
            if(renderRichText()) {
                genericDefField = new RAVEN.InlineEdit.JiraRichTextField(
                    $('#test-generic-field'),
                    $('.path-input'),
                    {
                        success: function () {
                            updateHistory();
                            updateDates();
                            clearMessage();
                        },
                        error: fieldSaveGenericDefError
                    }
                );
            }else {
                genericDefField = new RAVEN.InlineEdit.JiraWikiTextField(
                    $('#test-generic-field'),
                    $('.path-input'),
                    {
                        success: function () {
                            updateHistory();
                            updateDates();
                            clearMessage();
                        },
                        error: fieldSaveGenericDefError
                    }
                );
			}
		} else {
			genericDefField = new RAVEN.InlineEdit.TextField(
					$('#test-generic-field'),
					{
						success: function() {
							updateHistory();
							updateDates();
							clearMessage();
						},
						error: fieldSaveGenericDefError
					}
				);
		}
		cucumberStepsField = new RAVEN.InlineEdit.AceEditorField(
			$('#cucumber-steps-field'),
			{
				success: function() {
					updateHistory();
					updateDates();
					clearMessage();
				},
				error: fieldSaveCucumberStepsError
			}
		);


		inlineEdit = new RAVEN.InlineEdit();
	}

	function fieldSaveError(field, error) {
		clearMessage();
		var	errorMessages = error;
		var	fieldName = $(field.el).data('field-name');

		AJS.messages.error('#raven-test-details', {
			title : AJS.I18n.getText('view-issue-test-section.save.error', fieldName),
			body : errorMessages ,
			id : "raven-error-message"
		});
	}

	function customFieldSaveError(inlineEditElem, field, error, fieldName){
		clearMessage();
		var message = "";
        var errorMessages;
		try{
			// JIRA >= 7.3
			errorMessages = JSON.parse(error);
			if(errorMessages.errorMessages.length !== 0){
				message = errorMessages.errorMessages;
			} else {
                for (var errorElem in errorMessages.errors) {
                    if (errorMessages.errors.hasOwnProperty(errorElem)) {
                        message += errorMessages.errors[errorElem] + "\n";
                    }
                }
			}
		}catch(e1){
			// JIRA < 7.3
			try {
                errorMessages = JSON.parse(error.substring(error.indexOf(':')+1));
                if(errorMessages.errors.length !== 0){
                    for (var errorElem in errorMessages.errors) {
                        if (errorMessages.errors.hasOwnProperty(errorElem)) {
                            message += errorMessages.errors[errorElem] + "\n";
                        }
                    }
                } else {
                    message = error;
                }
            }catch(e2){
				//should not happen
            }
		}

        var keepData = "";
        var data = JSON.parse(field.data);
        for (var field in data.fields) {
            if (data.fields.hasOwnProperty(field)) {
                keepData +=data.fields[field];
            }
        }
        inlineEditElem.setValue(keepData);
        inlineEditElem.edit();

		AJS.messages.error('#raven-test-details', {
			title : AJS.I18n.getText('view-issue-test-section.save.error', fieldName),
			body : message ,
			id : "raven-error-message"
		});
	}

	function fieldSaveGenericDefError(field, error) {
		customFieldSaveError(this, field, error, AJS.I18n.getText("view-issue-test-section.field-path-label"));
	}

	function fieldSaveCucumberStepsError(field, error){
		customFieldSaveError(this, field, error, AJS.I18n.getText('common.label.steps'));
	}

	function toogleViewMode(reset) {
		if (isManualTestTypeOption(selectedTestType) === true) {
			if (reset && deleteTestInformation()) {
				cucumberStepsField.reset();
				genericDefField.reset();
			}
			$('.manual-steps').show();
			$('.cucumber-type').hide();
			$('.cucumber-steps').hide();
			$('.generic-def').hide();
		} else if (isCucumberTestTypeOption(selectedTestType) === true) {
			if (reset && deleteTestInformation()) {
				genericDefField.reset();
				resetTestStepsRESTfulTable();
			}
			$('.manual-steps').hide();
			$('.cucumber-type').show();
			$('.cucumber-steps').show();
			$('.generic-def').hide();
		} else {
			if (reset && deleteTestInformation()) {
				cucumberStepsField.reset();
				resetTestStepsRESTfulTable();
			}
			$('.manual-steps').hide();
			$('.cucumber-type').hide();
			$('.cucumber-steps').hide();
			$('.generic-def').show();
		}
	}

	function initialize() {
		AJS.messages.setup();

        clearMessage();

		canEditTest = $('#can-edit-test').val() === 'true';

		/**
		 * Next section is deprecated. This fucntionality is handled by the restfultable
		 */
		/*
		  if (!hasPermissions()) {
		  $("#manual-steps-table").bind(RAVEN.RestfulTable.Events.INITIALIZED, function () {
   		  $.each($('button.raven-restfultable-delete'), function(){
   		  $(this).remove();
   		  });
   		  $.each($('.raven-restfultable-draghandle'), function(){
   		  $(this).remove();
   		  });

		  });
		  }
		*/

		/**
		 * Remove Priority property in the view screen
		 */
		/*
		  $('li.item').find('span#priority-val').parent().parent().next().next().removeClass('full-width');
		  $('li.item').find('span#priority-val').parent().parent().remove();
		*/

		/**
		 * Remove Description menu
		 */
		/*
		  $('#descriptionmodule').css('display', 'none');
		*/

		$(document).on('preconditionAssocDeleted', function(e, preconditionKey) {
			var testDetailsPreconditionKeys = JSON.parse($('#test-details-precondition-issue-key').val());

			testDetailsPreconditionKeysResult = testDetailsPreconditionKeys.filter( function( testDetailsPreconditionKey ) {
				return preconditionKey.indexOf( testDetailsPreconditionKey ) < 0;
			} );

			$('#test-details-precondition-issue-key').val(JSON.stringify(testDetailsPreconditionKeysResult));
		});

		$(document).on('preconditionAssocAdded', function(e, preconditionKey) {
			var testDetailsPreconditionKeys = JSON.parse($('#test-details-precondition-issue-key').val());
			var testDetailsPreconditionKeysResult = $.merge(testDetailsPreconditionKeys, preconditionKey);

			$('#test-details-precondition-issue-key').val(JSON.stringify(testDetailsPreconditionKeysResult));
		});

		// delete step attachments icon click handler
		$(document).on('click', '.step-attachments li .attachment-remove-command', function(e) {
			var rowElem = $(e.target).closest('tr');
			var row = manualStepsTable.getRowFromElement(rowElem);

			var el = $(e.target).closest('li');
			var id = el.val();
			var fileName = el.data('name');

			var options = {};
			options.$el = el;
			options.stepId = row.model.id;

			deleteAttachmentConfirmationDialog(id, fileName, deleteAttachmentFromStep, options)
		});

		//adjust the maximum width of attachments column according to table size
        $(window).resize(function() {
            adjustAttachmentsColumn()
        });

        initView();
		initializeStepAttachmentsDialog();
	}

	function adjustAttachmentsColumn() {
        var maxWidth = $('#raven-test-details').width() / 5;
        maxWidth = maxWidth <= 100 ? 100 : maxWidth;
        $('.attachment-tablecell').css('max-width', maxWidth);
	}

	// register for NEW_CONTENT_ADDED
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
		// FIXME: Both load page and refresh are called when loading a page and the table is loaded 2 times
		// test if our view is already loaded because pageLoaded event is called 2 times in the issue search page
		if (reason == JIRA.CONTENT_ADDED_REASON.pageLoad || reason == JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
            var $section = $('#raven-test-details');
            if ($section.length > 0 && !$section.attr('data-initialized') && RAVEN.Utils.isCurrentIssueValid(getTestIssueKey())) {
                initialize();
				$('#raven-test-details').attr("data-initialized", "true");
            }
        }
	});

})(AJS.$);
