(function($) {

    RAVEN.namespace("RAVEN.Test");

    var gherkinTheme = require("ace/theme/textmate");

    var testType = null;

    var PRECONDITION_ASSOC_DELETED_EVENT = "preconditionAssocDeleted";
    var PRECONDITION_ASSOC_ADDED_EVENT = "preconditionAssocAdded";

    /**
     * Move positions constants
     */
    var AFTER = "After";
    var BEFORE = "Before";

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Test.PreConditionsTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/precondition/' + this._getTestIssueKey() + '/columns';
        },

        defaults: function() {
            return [];
        },

        _getTestIssueKey: function() {
            return $('#raven-test-precondition-issue-key').val();
        }
    });

    /**
     * This is the main view for the Pre-Condition table on a Test issue
     */
    RAVEN.Test.PreConditionsView = Backbone.View.extend({
        el: '#raven-test-preconditions',

        /**
         * DataTable Instance
         */
        dataTable: null,

        /**
         * The Test issue key
         */
        testKey: null,

        /**
         * Permissions
         */
        canEdit: false,

        /**
         * The Bulk view
         */
        bulkEditView: null,

        columnLayoutView: null,

        currentDropdownTrigger: null,

        events: {
            'click #raven-test-preconditions #raven-add-precondition': '_onAddPreConditionsClick',
            'click #raven-test-preconditions #raven-create-precondition': 'processCreatePrecondition',
            'click .test-preconditions-conditions-column-value .content-expand': '_triggerTooltip'
        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.precondition a.raven-remove-command': '_onPreConditionDeleteClick',
            'click.raven.precondition .bulk-remove-command': '_onBulkPreConditionDeleteClick',
            'click.raven.precondition a.raven-rank-change-command': '_onChangePreConditionRankClick',
            'click.raven.precondition .raven-dropdown-trigger': '_onDropDownTriggerClick',
            'aui-dropdown2-hide.raven.precondition .aui-dropdown2': '_onDownDownHide'
        },

        initialize: function() {
            var self = this;

            AJS.messages.setup();

            this._clearMessage();

            this.model = new RAVEN.Test.PreConditionsTableMetaModel();
            this._delegateUnscopedEvents();

            this.testKey = $('#raven-test-precondition-issue-key').val();

            this.canEdit = $('#can-edit-test-precondition').val() === 'true';

            this._setEditable(this.canEdit);

            this.model.fetch({
                success: function() {
                    $('#raven-test-preconditions').unblock();
                    if (self.$el.closest(document.documentElement).length > 0) {
                        self.render();
                    }
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-test-precondition-section.column-layout.error'), null, 'error');
                    $('#raven-test-preconditions').unblock();
                }
            });

            this.initializePrecontidion();
        },

        render: function() {
            this._clearMessage();

            // Render table skeleton
            var $table = RAVEN.Test.renderTestPreConditionTable({ 'columns': _.values(this.model.attributes) });
            var $tableWrapper = this.$el.find('#precondition-entries-table-wrapper');
            $tableWrapper.empty();
            $tableWrapper.html($table);


            // create the dataTable
            this._createPreConditionsDataTable();

            // render bulk mode button
            if (this.canEdit === true && !this.bulkEditView) {
                this._renderBulkModeButton();
            }

            // render test runs toolbar view
            this._renderPreConditionsToolbarView();

            $tableWrapper.show();

            this.$el.show();

            return this;
        },

        /**
         * Redraws the datatable
         */
        refresh: function() {
            this._cleanView();
            this.initialize();
        },

        destroy: function() {
            this._cleanView();

            if (this.columnLayoutView) {
                this.columnLayoutView.destroy();
                this.columnLayoutView = null;
            }

            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();

            this.$el.removeData().unbind();

            this.unbind();

            this.remove();
        },

        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.bulkEditView = null;

            this.model = null;

            var $tableWrapper = this.$el.find('#precondition-entries-table-wrapper');

            $tableWrapper.empty();
        },

        message: function(title, message, type) {
            type = (type === undefined || type == "") ? "error" : type;

            if (!message && title) {
                message = title;
                title = null;
            }

            if (type === "error") {
                this._clearMessage();
                AJS.messages.error('#raven-preconditions-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "success") {
                AJS.messages.success('#raven-preconditions-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "info") {
                AJS.messages.info('#raven-preconditions-message-bar', {
                    title: title,
                    body: message
                });
            }
        },

        _clearMessage: function() {
            $('#raven-preconditions-message-bar').empty();
        },

        _setEditable: function(editable) {
            if (editable == false) {
                $('#raven-add-precondition').hide();
                $('#raven-create-precondition').hide();
            } else {
                $('#raven-add-precondition').show();
                $('#raven-create-precondition').show();
            }
        },

        _createPreConditionsDataTable: function() {

            var self = this;

            var path = contextPath + '/rest/raven/1.0/test/' + self.testKey + '/precondition';

            if (this.dataTable) {
                this.dataTable.fnDestroy();
            }

            this.dataTable = $('#raven-test-preconditions .preconditions-table').xrayDataTable({
                "bProcessing": false,
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sEmptyTable": AJS.I18n.getText('view-issue-test-precondition-section.emptytable'),
                    "sInfoEmpty": AJS.I18n.getText('view-issue-test-precondition-section.infoempty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bSort": false,
                "bFilter": false,
                "aaSorting": [],
                "sAjaxSource": path,
                "sPaginationType": "full_numbers",
                "bServerSide": true,
                "bRetrieve": true,
                "bAutoWidth": false,
                "sAjaxDataProp": "entries",
                "aoColumnDefs": this._createDataTableColumnDefs(),
                "fnServerData": function(sSource, aoData, fnCallback, oSettings) {
                    self.$el.block();
                    oSettings.jqXHR = $.ajax({
                        "dataType": 'json',
                        "type": "GET",
                        "url": sSource,
                        "data": aoData,
                        "error": function(qXHR, textStatus, errorThrown) {
                            if (errorThrown && errorThrown.trim() != '') {
                                errorThrown = errorThrown + ': ' + qXHR.responseText;
                            } else {
                                errorThrown = qXHR.responseText;
                            }
                            self.message(AJS.I18n.getText('view-issue-testset-tests-section.processingerror'), errorThrown)
                        },
                        "complete": function() {
                            self.$el.unblock();
                        },
                        "success": fnCallback
                    });

                    if (self.bulkEditView) {
                        self.bulkEditView.setCheckBoxRowsSelectionValue(false);
                    }
                },
                "fnRowCallback": function(nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    nRow.id = aData["key"];
                    return nRow;
                },
                "fnDrawCallback": function(oSettings) {
                    // set initial overflow to allow overflow columns to be visible
                    self.$el.css("overflow", "initial");

                    self._tooglePreConditionsTable();
                    if (self.bulkEditView) {
                        self.bulkEditView.showHideBulkActions();
                    }
                }
            });

            if (this.canEdit === true) {
                this._initSortableTable(this.$el.find('.preconditions-table'));
            }

        },

        /**
         * Creates the DataTable column def descriptors
         */
        _createDataTableColumnDefs: function() {
            var self = this;

            var bulkToogleColumnTitle = null;
            var bulkActionColumnTitle = null;

            if (self.canEdit === true) {
                bulkToogleColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkSelectAllInput();

                var bulkActions = '<a href="javascript:void(0);" data-key="bulkKey" class="bulk-remove-command active aui-dropdown2-active" title="' + AJS.I18n.getText('common.action.remove') + '">' +
                    '<span class="icon icon-delete menu-icon"></span>' + AJS.I18n.getText('common.action.remove') + '</a>';
                bulkActionColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkActionTrigger({ id: "precond-bulk-action", actions: bulkActions });
            }

            var userColumnDefs = this._createDataTableUserColumn();

            var firstColumnDefs = [{
                    "mData": null,
                    "sTitle": bulkToogleColumnTitle,
                    "mDataProp": null,
                    "sWidth": "20px",
                    "sDefaultContent": "<input type='checkbox' ></input>",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function(data, type, row) {
                        if (self.canEdit === true) {
                            return RAVEN.DataTable.BulkEdit.renderBulkSelectInputAndDragHandle({ id: row["key"] });
                        } else {
                            return '';
                        }
                    },
                    "aTargets": [0]
                },
                {
                    "mData": "rank",
                    "bSortable": false,
                    "sWidth": "20px",
                    "bSearchable": false,
                    "mRender": function(data, type, row) {
                        return "<span class='rank' style='padding-left: 2px'>" + data + "</span>";
                    },
                    "aTargets": [1]
                }
            ];

            var lastColumnDefs = [{
                "mData": "condition",
                "bSortable": false,
                "sWidth": "48px",
                "sTitle": bulkActionColumnTitle,
                "mRender": function(data, type, row) {
                    var menu = '<div style="width:48px">';
                    if (data.raw) {
                        testType = row.type;

                        var sharedHtml = '<div class="test-preconditions-conditions-column-value" style="height:0;"><div class="overflow"></div><span class="content-expand icon icon-inline-dialog-sml"></span><div class="test-preconditions-conditions-more" style="display:none;">';
                        if (testType !== "Cucumber") {
                            menu += sharedHtml + '<span class="conditionData">' + data.rendered + '</span></div>';
                        } else {
                            menu += sharedHtml + '<div class=""><p id="precondition-rendered-condition" style="display: none;">' + data.rendered + '</p><input type="hidden" class="cucumber-precondition-value"/> <div class="wrap raven-test-section-cucumber-condition"><span id="cucumber-precondition-condition-val"><div id="cucumber-precondition-ace-editor" class="ace-mode-viewer"></div></span></div></div></div>';
                        }
                    }

                    if (self.canEdit === true) {
                        menu += RAVEN.Test.preconditionsMenu({
                            key: row["key"],
                            canEdit: self.canEdit,
                            rank: row["rank"]
                        });
                    }

                    menu += '</div>';
                    return menu;
                },
                "aTargets": [_.values(this.model.attributes).length + 2]
            }];

            return firstColumnDefs.concat(userColumnDefs).concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumn: function() {
            return _.map(_.values(this.model.attributes), function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id,
                    "bSortable": column.sortable,
                    "bSearchable": false,
                    "aTargets": [2 + index]
                };
            });
        },

        /**
         * Adds support for moving datatable items
         */
        _initSortableTable: function(tableSelector) {
            var self = this;

            $(tableSelector).sortable({
                items: 'tbody tr',
                handle: '.draghandle',
                placeholder: "drag-placeholder",
                helper: function(e, tr) {
                    var $originals = tr.children();
                    var $helper = tr.clone();
                    $helper.children().each(function(index) {
                        // Set helper cell sizes to match the original sizes
                        $(this).width($originals.eq(index).width())
                    });
                    return $helper;
                },
                start: function(event, ui) {
                    ui.placeholder.height(ui.item.height() + 15);

                    var $this = ui.placeholder.find("td");
                    // Make sure that when we start dragging widths do not change
                    ui.item.children().each(function(i) {
                        $(this).width($this.eq(i).width());
                    });
                },
                update: function(event, ui) {
                    var position = null;
                    var ancorNode = null;

                    var ancorNode = ui.item.prev();
                    if (ancorNode.length > 0) {
                        position = AFTER;
                    } else {
                        var ancorNode = ui.item.next();
                        if (ancorNode.length > 0) {
                            position = BEFORE;
                        }
                    }

                    if (position != null) {
                        var movedPreConditionData = self.dataTable.fnGetData(ui.item[0]);
                        var ancorPreConditionData = self.dataTable.fnGetData(ancorNode[0]);

                        if (movedPreConditionData && movedPreConditionData.key && ancorPreConditionData && ancorPreConditionData.key) {
                            self._movePreCondition(self.dataTable, movedPreConditionData, ancorPreConditionData, position);
                        }
                    }
                }
            });
        },

        _tooglePreConditionsTable: function() {
            if (this.dataTable && this.dataTable.fnRecordsTotal() > 0) {
                this.$el.find('.nopreconditions').hide();
                this.$el.find('.xrayDataTables_wrapper').show();
            } else {
                this.$el.find('.xrayDataTables_wrapper').hide();
                this.$el.find('.nopreconditions').show();
            }
        },

        _renderBulkModeButton: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.bulkEditView = new RAVEN.DataTable.BulkEditView({
                el: $wrapper
            });
        },

        _updateRankRow: function(newRankValue, originalRankSpanValue, row) {
            //Validate values
            if (newRankValue > 0 && newRankValue <= this.dataTable.fnSettings().fnRecordsTotal() && newRankValue != originalRankSpanValue) {
                var movedTestData = this.dataTable.fnGetData(row);

                if (movedTestData && movedTestData.key) {
                    this._movePreCondition(this.dataTable, movedTestData, null, newRankValue);
                }
            }
        },

        _movePreCondition: function(table, movedPreConditionData, ancorPreConditionData, position) {
            var self = this;

            this.$el.block();

            var path = contextPath + '/rest/raven/1.0/test/' + this.testKey + '/precondition/' + movedPreConditionData.key + '/move'

            var value = {};
            if (position == AFTER) {
                value.after = '/' + ancorPreConditionData.key;
            } else if (position == BEFORE) {
                value.before = '/' + ancorPreConditionData.key;
            } else {
                value.index = position;
            }
            var data = JSON.stringify(value);

            $.ajax({
                type: "POST",
                url: path,
                processData: true,
                data: data,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-test-precondition-section.move.error', movedPreConditionData.key), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                    self.dataTable.fnDraw(false);
                },
                complete: function() {
                    self.$el.unblock();
                }
            });
        },

        _addPreConditions: function(preconditionKeys) {
            var self = this;

            this.$el.block();

            var value = JSON.stringify(preconditionKeys);
            $.ajax({
                type: "POST",
                url: contextPath + '/rest/raven/1.0/test/' + self.testKey + '/precondition',
                processData: true,
                async: true,
                data: value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('view-issue-test-precondition-section.add.error', self.testKey), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                    if (data.errors && data.errors.length > 0) {
                        var errorMsg = '';
                        $.each($(data.errors), function(i, msg) {
                            errorMsg = errorMsg + msg;
                            if (i < data.errors.length) {
                                errorMsg = errorMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-test-precondition-section.add.controlled.error', self.testKey), errorMsg);
                    }

                    if (data.precondition) {
                        self.dataTable.fnAddData(data.precondition);
                    }
                },
                complete: function() {
                    self.$el.unblock();
                    self._updateHistory();
                    self._updateDates();
                }
            });
            $(document).trigger(PRECONDITION_ASSOC_ADDED_EVENT, [preconditionKeys]);
        },

        _confirmationDeletePreCondition: function(preConditionKeys, yesFunction, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                id: 'delete-precondition-dialog',
                content: function(callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('view-issue-test-precondition-section.bulk.delete.confirm.title');
                        message = AJS.I18n.getText('view-issue-test-precondition-section.bulk.delete.confirm.message', self.testKey);
                    } else {
                        title = AJS.I18n.getText('view-issue-test-precondition-section.delete.confirm.title', preConditionKeys[0]);
                        message = AJS.I18n.getText('view-issue-test-precondition-section.delete.confirm.message', self.testKey);
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({ title: title, message: message, severity: severity });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(preConditionKeys);
                    dialog.hide();
                },
                autoClose: true
            });
            dialog.show();
        },

        _deletePreCondition: function(preConditionKeys) {
            var self = this;

            this.$el.block();

            //if we have records to delete
            if (preConditionKeys && preConditionKeys.length > 0) {
                //Build query param
                var preConditionKeysParam = self._buildQueryParamsFromSelectedRows(preConditionKeys);

                $.ajax({
                    type: "DELETE",
                    url: contextPath + '/rest/raven/1.0/test/' + self.testKey + '/precondition?' + preConditionKeysParam + "",
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        if (preConditionKeys.length == 1) {
                            self.message(AJS.I18n.getText('view-issue-test-precondition-section.delete.error', preConditionKeys[0]), errorThrown);
                        } else {
                            self.message(AJS.I18n.getText('view-issue-test-precondition-section.bulk.delete.error'), errorThrown);
                        }

                        self.$el.unblock();
                    },
                    success: function(data, textStatus, jqXHR) {
                        self._deletePreConditionsCallback(preConditionKeys, data);
                    },
                    complete: function() {
                        self.$el.unblock();
                        self._updateHistory();
                        self._updateDates();
                    }
                });
                $(document).trigger(PRECONDITION_ASSOC_DELETED_EVENT, [preConditionKeys]);
            }
        },

        /**
         * Build query param
         */
        _buildQueryParamsFromSelectedRows: function(selectedIds) {
            var preConditionKeys = "";
            var i;
            for (i = 0; i < selectedIds.length; ++i) {
                if (preConditionKeys.length > 0) {
                    preConditionKeys += "&";
                }
                preConditionKeys += "preconditionKey=" + selectedIds[i];
            }

            return preConditionKeys;
        },

        _deletePreConditionsCallback: function(preConditionKeys, data) {
            var self = this;
            if (preConditionKeys && preConditionKeys.length > 0) {
                var allData = this.dataTable.fnGetData();
                if (allData.length >= 1) {
                    _.each(_.initial(preConditionKeys), function(preConditionKey) {
                        self.dataTable.fnDeleteRow(preConditionKey, null, false);
                    });
                    this.dataTable.fnDeleteRow(_.last(preConditionKeys));
                } else {
                    // don't refresh the table if the last element is removed because it will request more data from server
                    this.dataTable.fnDeleteRow(_.last(preConditionKeys), null, false);
                }
            }
        },

        _confirmationChangeRank: function(title, message, row, originalRankSpanValue, yesFunction) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                content: function(callback) {
                    var innerHtml = RAVEN.Dialog.editValueDialog({ title: title, fieldName: message, actionText: AJS.I18n.getText('common.action.ok') });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    e.preventDefault();
                    var newRankValue = $('#edit-value-dialog-value')[0].value;
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(newRankValue, originalRankSpanValue, row);
                    dialog.hide();
                },
                autoClose: true
            });
            dialog.show();
        },

        _updateHistory: function() {
            if ($('#changehistory-tabpanel').attr('class') == 'active') {
                var alt_token = $('input[name="atl_token"]').val();
                $.ajax({
                    type: "GET",
                    url: contextPath + '/browse/' + this.testKey + "?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_=" + alt_token,
                    contentType: "text/plain",
                    success: function(data, status) {
                        var tab = $(data).find('#activitymodule div.mod-content #issue_actions_container');
                        var jiraTab = $('#activitymodule div.mod-content').find('#issue_actions_container');
                        $(jiraTab).html(tab);
                    }
                });
            }
        },

        _updateDates: function() {
            var updatedDate = null;

            if (AJS.$('#datesmodule #updated-date').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
            } else if (AJS.$('#datesmodule #updated-val').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
            } else {
                return null;
            }

            $.ajax({
                type: "GET",
                url: AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
                contentType: "application/json",
                success: function(data) {
                    var title = data.title;
                    var dateTimeIso8601 = data.dateTimeIso8601;
                    updatedDate.attr("datetime", dateTimeIso8601);
                    $.each(AJS.$('#datesmodule dd.date'), function() {
                        if ($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                            $(this).attr("title", title);
                        }
                    });
                    updatedDate.livestamp();
                }
            });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.precondition');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.precondition');
        },

        /**
         * Delete precondition click handler
         */
        _onPreConditionDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var preconditionKey = $(e.target).closest('a').data('key');
            this._confirmationDeletePreCondition([preconditionKey], this._deletePreCondition);
        },

        /**
         * Delete Bulk test click handler
         */
        _onBulkPreConditionDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var rowIds = this.bulkEditView.getSelectedRowIds();

            if (rowIds && rowIds.length > 0) {
                this._confirmationDeletePreCondition(rowIds, this._deletePreCondition, true);
            }
        },

        /**
         * Add Precondition button click handler
         */
        _onAddPreConditionsClick: function() {
            var self = this;

            this._clearMessage();

            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.precondition.label');
            labels.title = AJS.I18n.getText('dialogs.issueselector.precondition.title', this.testKey);
            labels.description = AJS.I18n.getText('dialogs.issueselector.precondition.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.precondition.enter');

            var issueTypeId = $('#precond-issue-type-id').val();
            var preCondTypeCF = $('#precond-type-cf-id').val();
            var selectedTestType = $('#test-type-select option:selected').text();

            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog('PRECONDITION', 'key not in testPreConditions(' + this.testKey + ') and cf[' + preCondTypeCF + '] = ' + selectedTestType, issueTypeId, this.testKey, labels, function(issues) {
                if (issues && issues.length > 0) {
                    self._addPreConditions(issues);
                }
            }, null);
        },

        /**
         * Change Pre-Condition rank click handler.
         */
        _onChangePreConditionRankClick: function(e) {
            e.stopPropagation();
            e.preventDefault();
            this._clearMessage();

            var originalRankSpanValue = $(e.target).attr('data-key');

            var $row = $(e.target).closest('tr');

            var rowKey = this.dataTable.fnGetData($row[0]).key;

            var label = AJS.I18n.getText('dialogs.edit-value.rank.label');
            var title = rowKey + ' - ' + AJS.I18n.getText('dialogs.edit-value.rank.title');
            this._confirmationChangeRank(title, label, $row[0], originalRankSpanValue, this._updateRankRow);
        },

        /**
         * Save current dropdown trigger
         */
        _onDropDownTriggerClick: function(e) {
            this.currentDropdownTrigger = $(e.target);
            this.currentDropdownTrigger.closest('tr').addClass('active');
        },

        /**
         * Clear current dropdown trigger
         */
        _onDownDownHide: function() {
            if (this.currentDropdownTrigger) {
                this.currentDropdownTrigger.closest('tr').removeClass('active');
            }
            this.currentDropdownTrigger = null;
        },

        _renderPreConditionsToolbarView: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#toolbar-wrapper').length > 0) {
                return;
            }

            var toolbarHtml = RAVEN.Test.renderPreConditionsToolBar();
            $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.Test.PreConditionColumnLayoutView({
                mainView: this
            });
        },

        _triggerTooltip: function(e) {
            e.preventDefault();

            var windowHeight = window.innerHeight || document.documentElement.clientHeight || document.getElementsByTagName('body')[0].clientHeight;
            var mousePositionHeight = e.clientY;
            var gravity = (windowHeight / 2) > mousePositionHeight ? 'ne' : 'se';
            var el = $(e.target).tooltip({
                gravity: gravity,
                trigger: 'manual',
                className: 'precondition-tipsy',
                html: true,
                title: function() {
                    var $data = $(e.target).closest('.test-preconditions-conditions-column-value').find('.test-preconditions-conditions-more');
                    if ($data.length > 0) {
                        var elem = $data.clone();
                        elem.find("#cucumber-precondition-ace-editor").attr("id", "cucumber-precondition-ace-editor-tipsy");
                        return elem.css('display', 'block')[0].innerHTML;
                    } else {
                        return null;
                    }
                }
            });

            if (!el.hasClass('expanded')) {
                $($(e.target)).data('tipsy').options.gravity = gravity;

                // collapse all other expanded test sets custom fields in page
                var expandedCfs = $('.test-preconditions-conditions-column-value .content-expand.expanded');
                expandedCfs.removeClass('expanded');
                expandedCfs.tooltip('hide');

                el.tooltip('show');
                el.addClass('expanded');
            } else {
                el.removeClass('expanded');
                el.tooltip('hide');
            }

            if (testType === "Cucumber") {
                var condition = $(e.target).closest('tr').find($('#precondition-rendered-condition')).text()
                $(e.target).closest('tr').find(($('.cucumber-precondition-value'))[0]).val(condition);

                require(['ace/ace'], function(ace) {
                    var aceValue = $(e.target).closest('tr').find($('.cucumber-precondition-value')).val();
                    cucumberPrecondition = ace.edit("cucumber-precondition-ace-editor-tipsy");
                    cucumberPrecondition.setTheme(gherkinTheme);
                    cucumberPrecondition.setFontSize(14);
                    cucumberPrecondition.getSession().setMode("ace/mode/gherkin");
                    cucumberPrecondition.getSession().setUseWrapMode(true)
                    cucumberPrecondition.renderer.setShowGutter(true);
                    cucumberPrecondition.renderer.setShowPrintMargin(false);
                    cucumberPrecondition.getSession().setValue(aceValue);
                    cucumberPrecondition.setReadOnly(true);
                });
            }

            $(document).mouseup(function(e) {

                var el = $('.tipsy');

                if (!el.is(e.target) && el.has(e.target).length === 0) {
                    var expandedCfs = $('.test-preconditions-conditions-column-value .content-expand.expanded');
                    expandedCfs.removeClass('expanded');
                    expandedCfs.text('(' + "show more" + '...)');
                    expandedCfs.tooltip('hide');

                    // remove loose tipsies
                    $('.tipsy:has(.test-preconditions-conditions-tooltip-close)').remove();
                }
            });
        },

        processCreatePrecondition: function() {
            var createForm = JIRA.Forms.createCreateIssueForm({
                issueType: $('#precond-issue-type-id').val(),
            });

            var dialog = createForm.asDialog({
                id: 'create-issue-dialog'
            });

            dialog.show();

            // Create Precondition dialog identification
            $("#create-issue-dialog").addClass('raven-create-precondition-dialog');

            // Refresh on issue create
            JIRA.bind("QuickCreateIssue.sessionComplete", processPreconditionCreated);

            function processPreconditionCreated() {
                JIRA.unbind("QuickCreateIssue.sessionComplete", processPreconditionCreated);
                JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
            }
        },

        /*
         * Initialize Create Pre-condition dependencies
         */
        initializePrecontidion: function() {

            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                var $section = $("#raven-test-preconditions");

                if ($section.length > 0 && !$section.attr('data-preconditions-initialized')) {
                    processEvent();
                }

                function processEvent() {
                    AJS.messages.setup();
                    $(document).bind("dialogContentReady", processDialogContentReadyNewPrecondiditon);
                    $section.attr("data-preconditions-initialized", "true");
                }
            });

            function processDialogContentReadyNewPrecondiditon(e, dialog) {
                var popup = dialog.$popup[0];
                var isOnCreatePreconditionDialog = (popup.id == 'create-issue-dialog' && popup.classList.contains('raven-create-precondition-dialog'));

                if (isOnCreatePreconditionDialog) {
                    var issueTest = getIssueTestData();
                    var customFieldPreconditionTestId = getCustomFieldPreconditionTestId();

                    selectPreconditionType();

                    // Add linked Test
                    $(popup).find("#" + customFieldPreconditionTestId).trigger('selectOption', [{ label: issueTest.issueKey, value: issueTest.issueKey, title: issueTest.issueKey }]);

                    // Fixed Versions
                    $(issueTest.fixedVersions).each(function(index, obj) {
                        var jsonObj = JSON.parse(obj);
                        $('select[id="fixVersions"] option').each(function() {
                            if (jQuery.trim($(this).text()) == jsonObj.name) {
                                $(popup).find("#fixVersions").trigger('selectOption', [{ value: $(this).val(), title: jsonObj.name, label: jsonObj.name }]);
                            }
                        });
                    });
                }
            };

            /**
             * Select a PreconditionType by TestType text.
             * Uses the onChange event from precondition-custom-fields.js
             */
            function selectPreconditionType() {
                var customFieldPreconditionTypeId = getCustomFieldPreconditionTypeId();
                var testTypeText = getTestTypeText();

                $('select[id="' + customFieldPreconditionTypeId + '"] option').each(function() {
                    var option = $(this);

                    // remove all selected attribute to avoid duplications.
                    option.removeAttr("selected");

                    if (jQuery.trim(option.text()) == testTypeText) {
                        // Select by TestType text
                        $('#' + customFieldPreconditionTypeId + ' option[value="' + option.val() + '"]').attr("selected", "selected");

                        // Add the TypeID to the current precondition selection. Will be used in onChange event.
                        $('#raven-precondition-field-type-value').val(option.val());

                        // Trigger the onChange event to format the page style for the current PreconditionType
                        $("#" + customFieldPreconditionTypeId).trigger("change.raven-edit-precondition");
                    }
                });
            }

            function getIssueTestData() {
                return JSON.parse($("#issue-dialog-populate-object").val());
            };

            function getTestTypeText() {
                return $("#test-issue-type-value").val();
            };

            function getCustomFieldPreconditionTestId() {
                return $("#customfield-precondition-test-id").val();
            };

            function getCustomFieldPreconditionTypeId() {
                return $("#raven-precondition-field-type").val();
            };
        }
    });
})(AJS.$);