(function($) {

    RAVEN.namespace("RAVEN.Test");

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Test.TestPlansTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/test/' + this._getTestIssueKey() + '/testplan/columns';
        },

        defaults: function() {
            return [];
        },

        _getTestIssueKey: function () {
            return $('#raven-test-testplans-issue-key').val();
        }
    });

    /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Test.TestPlansView = Backbone.View.extend({
        el: '#raven-test-testplans',

        /**
         * DataTable Instance
         */
        dataTable: null,

        /**
         * The Test issue key
         */
        testKey: null,

        /**
         * The Project key
         */
        projectKey: null,

        /**
         * The assignee
         */
        assignee: null,

        /**
         * The TestExecution issue type
         */
        testExecutionIssueType: null,

        /**
         * Filter Instance
         */
        projectFilter: null,
        statusFilter: null,
        versionFilter: null,
        clearFilter: null,
        datePickerStart: null,
        datePickerEnd: null,
        milisecStart: null,
        milisecEnd: null,

        /**
         * Permissions
         */
        canEdit: false,

        /**
         * The Bulk view
         */
        bulkEditView: null,

        currentDropdownTrigger: null,

        columnLayoutView: null,

        events: {

        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.testtestplan .raven-dropdown-trigger': '_onDropDownTriggerClick',
            'aui-dropdown2-hide.raven.testtestplan .aui-dropdown2': '_onDownDownHide',
            'click.raven.testtestplan .tp-remove-command' : '_onTestDeleteClick',
            'click.raven.testtestplan .tp-bulk-remove-command' : '_onBulkTestDeleteClick',
            'click.raven.testtestplan #addTestPlan' : '_processAddTestPlans'
        },

        initialize: function() {
            var self = this;

            this._clearMessage();

            this.model = new RAVEN.Test.TestPlansTableMetaModel();

            this._delegateUnscopedEvents();

            AJS.messages.setup();

            this.testKey = $('#raven-test-testplans-issue-key').val();

            this.canEdit = $('#can-edit-test-testplans').val() === 'true';

            $('#raven-test-testplans').block();
            this.model.fetch({
                success: function() {
                    $('#raven-test-testplans').unblock();
                    if (self.$el.closest(document.documentElement).length > 0) {
                        self.render();
                    }
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-test-testplans-section.column-layout.error'), null, 'error');
                    $('#raven-test-testplans').unblock();
                }
            });
        },

        render: function() {
            this._clearMessage();

            // Render table skeleton
            var $table = RAVEN.Test.renderTestPlansTable({'columns': _.values(this.model.attributes)});

            var $tableWrapper = this.$el.find('#testplan-entries-table-wrapper');
            $tableWrapper.empty();
            $tableWrapper.html($table);

            // create the dataTable
            this._createTestPlansDataTable();

            // render bulk mode button
            if (this.canEdit === true && !this.bulkEditView) {
                this._renderBulkModeButton();
            }

            // render test runs toolbar view
            this._renderTestPlansToolbarView();

            $tableWrapper.show();

            this.$el.show();

            this.ready = true;

            return this;
        },

        /**
         * Handles the creation of the datatable.
         */
        _createTestPlansDataTable: function() {
            var self = this;

            var path = contextPath + '/rest/raven/1.0/test/' + this.testKey + "/testplans";

            if(this.dataTable) {
                this.dataTable.fnDestroy();
            }

            this.dataTable = $('#testplan-entries-table').xrayDataTable({
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                    "sEmptyTable": AJS.I18n.getText('common.table.label.no-resources-add-one'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch":  AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bServerSide" : true,
                "sAjaxSource" : path,
                "bDestroy" : true,
                "sAjaxDataProp" : "entries",
                "bProcessing" : false,
                "bAutoWidth" : false,
                "bFilter": false,
                "sPaginationType" : "full_numbers",
                "bSort" : true,
                "aaSorting" : [],
                "aoColumnDefs": this._createTestPlanTableColumnDefs(),
                "fnServerData" : function( sSource, aoData, fnCallback, oSettings){
                    self._fnServerData( sSource, aoData, fnCallback, oSettings );
                },
                "fnInitComplete" : function(oSettings, json) {
                    self._toogleTestPlansTable();
                },
                "fnRowCallback": function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                    nRow.id = aData["key"];
                    return nRow;
                },
                "fnDrawCallback": function( oSettings ) {
                    // set initial overflow to allow overflow columns to be visible
                    self.$el.css("overflow", "initial");

                    // hack for drag&drop to work
                    $('#testplan-entries-table').closest('div').addClass('datatable-drag-overflow');

                    self._toogleTestPlansTable();
                    if (self.bulkEditView) {
                        self.bulkEditView.showHideBulkActions();
                    }
                }
            });

            this._initSortableColumns();
        },

        _initSortableColumns: function() {
            var self = this;

            $('#testplan-entries-table thead tr').sortable({
                items: 'th:not(.fixed)',
                placeholder: "dragging-placeholder",
                update: function (event, ui) {
                    var ancorNode = ui.item.prev();
                    var columnId = $(ui.item).data('column-id');
                    if (columnId) {
                        var anchorColumnId = null;
                        if (!$(ancorNode).hasClass('fixed')) {
                            anchorColumnId = $(ancorNode).data('column-id');
                        }

                        self.columnLayoutView.moveColumn(columnId, anchorColumnId);
                    }
                }
            });
        },

        /**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createTestPlanTableColumnDefs: function() {
            var self = this;

            var bulkToogleColumnTitle = null;
            var bulkActionColumnTitle = null;
            if (self.canEdit === true) {
                bulkToogleColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkSelectAllInput();

                var bulkActions = '<a href="javascript:void(0);" data-key="bulkKey" class="bulk-plan-remove-command active aui-dropdown2-active tp-bulk-remove-command" title="' + AJS.I18n.getText('common.action.remove') + '">'+
                    '<span class="icon icon-delete menu-icon"></span>' + AJS.I18n.getText('common.action.remove') + '</a>';
                bulkActionColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkActionTrigger({id: "testplan-bulk-action", actions: bulkActions});
            }

            var firstColumnDefs = [
                {
                    "mData": null,
                    "sTitle": bulkToogleColumnTitle,
                    "mDataProp": null,
                    "sWidth": "20px",
                    "sDefaultContent": "<input type='checkbox'/>",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        if (self.canEdit === true) {
                            return RAVEN.DataTable.BulkEdit.renderBulkSelectInput({id: row["key"]});
                        } else {
                            return '';
                        }
                    },
                    "aTargets": [ 0 ]
                }
            ];

            var userColumnDefs = this._createDataTableUserColumn();

            var lastColumnDefs = [
                {
                    "mData": null,
                    "sTitle": bulkActionColumnTitle,
                    "bSortable": false,
                    "sWidth": "50px",
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        if (self.canEdit === true) {
                            return RAVEN.Test.testPlansMenu({
                                key: row["key"],
                                canEdit: self.canEdit
                            });
                        }
                        return null;
                    },
                    "aTargets": [ 1 + _.values(this.model.attributes).length ]
                }
            ];

            return firstColumnDefs.concat(userColumnDefs).concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumn: function() {
            return _.map(_.values(this.model.attributes), function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id,
                    "bSortable": column.sortable,
                    "bSearchable": false,
                    "aTargets": [ 1 + index ]
                };
            });
        },

        _renderTestPlansToolbarView: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#toolbar-wrapper').length > 0) {
                return;
            }

            var toolbarHtml = RAVEN.Test.renderTestPlansToolBar();
            $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.Test.TestPlansColumnLayoutView({
                mainView: this
            });
        },

        _toogleTestPlansTable : function() {
            var self = this;
            if (self.dataTable && self.dataTable.fnRecordsTotal() > 0) {
                $('#raven-test-testplans .notestplans').hide();
                $('#testplan-entries-table-wrapper').show();
            } else {
                $('#testplan-entries-table-wrapper').hide();
                $('#raven-test-testplans .notestplans').show();
            }
        },

        _renderBulkModeButton: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.bulkEditView = new RAVEN.DataTable.BulkEditView({
                el: $wrapper
            });
        },

        _fnServerData: function(sSource, aoData, fnCallback, oSettings) {
            var self = this;

            $('#raven-test-testplans').block();

            oSettings.jqXHR = $.ajax({
                dataType : 'json',
                type : "GET",
                url : sSource,
                data : aoData,
                error : function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-test-testplans-section.processingerror'),	errorThrown);
                },
                complete : function() {
                    $('#raven-test-testplans').unblock();
                },
                success : fnCallback
            });
        },

        _showInitializationError: function(message) {
            AJS.messages.error("#raven-initialization-error", {
                body: message,
                closeable: true
            });
        },

        /**
         * Redraws the datatable
         */
        refresh: function() {
            this._cleanView();
            this.initialize();
        },

        destroy: function() {
            this._cleanView();

            if (this.columnLayoutView) {
                this.columnLayoutView.destroy();
                this.columnLayoutView = null;
            }

            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();

            this.$el.removeData().unbind();

            this.unbind();

            this.remove();
        },

        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.bulkEditView = null;

            this.model = null;

            var $tableWrapper = this.$el.find('#testplan-entries-table-wrapper');

            $tableWrapper.empty();
        },

        _clearMessage: function() {
            $('#raven-test-testplans-message-bar').empty();
        },

        message: function(title, message, type) {
            type = (type === undefined)? "error" : type;
            this._clearMessage();

            if (type === "error") {
                AJS.messages.error('#raven-test-testplans-message-bar', {
                    title : title,
                    body : message
                });
            } else if (type === "info") {
                AJS.messages.info('#raven-test-testplans-message-bar', {
                    title : title,
                    body : message
                });
            } else if (type === "warning") {
                AJS.messages.warning('#raven-test-testplans-message-bar', {
                    title : title,
                    body : message
                });
            } else if (type === "success") {
                AJS.messages.success('#raven-test-testplans-message-bar', {
                    title : title,
                    body : message
                });
            }
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.testtestplan');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.testtestplan');
        },

        /**
         * Save current dropdown trigger
         */
        _onDropDownTriggerClick: function(e) {
            this.currentDropdownTrigger = $(e.target);
            this.currentDropdownTrigger.closest('tr').addClass('active');
        },

        /**
         * Clear current dropdown trigger
         */
        _onDownDownHide: function() {
            if (this.currentDropdownTrigger) {
                this.currentDropdownTrigger.closest('tr').removeClass('active');
            }
        },

        _onTestDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var testKey = $(e.target).closest('a').data('key');

            this._deleteTestPlanConfirmationDialog([ testKey ], this._deleteTest);
        },

        /**
         * Delete Bulk test click handler
         */
        _onBulkTestDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var rowIds = this.bulkEditView.getSelectedRowIds();

            if(rowIds && rowIds.length > 0){
                this._deleteTestPlanConfirmationDialog(rowIds, this._deleteTest, true);
            }
        },

        _deleteTestPlanConfirmationDialog: function(keys, fnDeleteTestPlan, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                id: 'delete-testplan-dialog',
                content: function (callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('view-issue-test-testplans-section.bulk.delete.confirm.title');
                        message = AJS.I18n.getText('view-issue-test-testplans-section.bulk.delete.confirm.message', self.testKey);
                    } else {
                        var title = AJS.I18n.getText('view-issue-test-testplans-section.delete.confirm.title', keys[0]);
                        message = AJS.I18n.getText('view-issue-test-testplans-section.delete.confirm.message', self.testKey);
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({title:title, message:message, severity:severity});
                    callback(innerHtml);
                },
                submitHandler: function (e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(fnDeleteTestPlan, self);
                    proxyFunc(keys);
                    dialog.hide();
                },
                autoClose : true
            });
            dialog.show();
        },

        _deleteTest: function(testPlanKeys) {
            var self = this;

            this.$el.block();

            //if we have records to delete
            if(testPlanKeys && testPlanKeys.length > 0) {
                //Build query param
                var testKeysParam = self._buildQueryParamsFromSelectedRows(testPlanKeys);

                $.ajax({
                    type: "DELETE",
                    url: contextPath + '/rest/raven/1.0/test/' + self.testKey + "/testplans?" + testKeysParam,
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        if (testPlanKeys.length == 1) {
                            self.message(AJS.I18n.getText('view-issue-test-testplans-section.delete.error', testPlanKeys[0]), errorThrown);
                        } else {
                            self.message(AJS.I18n.getText('view-issue-test-testplans-section.bulk.delete.error'), errorThrown);
                        }

                        self.$el.unblock();
                    },
                    success: function(data, textStatus, jqXHR) {
                        self._deleteTestPlansCallback(testPlanKeys, data);
                    },
                    complete: function() {
                        self.$el.unblock();
                        self._updateHistory();
                        self._updateDates();
                    }
                });
            }
        },

        _deleteTestPlansCallback: function(testPlanKeys, data) {
            var self = this;
            if (testPlanKeys && testPlanKeys.length > 0) {
                var allData = this.dataTable.fnGetData();
                if (allData.length >= 1) {
                    _.each(_.initial(testPlanKeys), function(testPlanKey) {
                        self.dataTable.fnDeleteRow(testPlanKey, null, false);
                    });
                    this.dataTable.fnDeleteRow(_.last(testPlanKeys));
                } else {
                    // don't refresh the table if the last element is removed because it will request more data from server
                    this.dataTable.fnDeleteRow(_.last(testPlanKeys), null, false);
                }
            }
        },

        _buildQueryParamsFromSelectedRows: function(selectedIds) {
            var testPlanKeys = "testplanKeys=";
            var i;
            for (i = 0; i < selectedIds.length; ++i) {
                if(i > 0){
                    testPlanKeys+=",";
                }
                testPlanKeys += selectedIds[i];
            }

            return testPlanKeys;
        },

        _processAddTestPlans: function() {
            var self = this;
            var testKey = this.testKey;

            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.testplan.label');
            labels.title = AJS.I18n.getText('dialogs.issueselector.testplan.title', testKey);
            labels.description = AJS.I18n.getText('dialogs.issueselector.testplan.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.testplan.enter');

            var issueTypeId = $('#testplan-issue-type-id').val();

            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog('TESTPLAN', "key not in testTestPlan(" + testKey +")",issueTypeId, testKey, labels, function(issues) {
                if (issues && issues.length > 0) {
                    self._addTestPlans(testKey, issues);
                }
            }, null);
        },


        _addTestPlans: function(testkey, testplankeys) {
            var self = this;

            this.$el.block();

            var value = JSON.stringify(testplankeys);
            $.ajax({
                type: "POST",
                url: contextPath + '/rest/raven/1.0/test/' + testkey + '/testplans',
                processData: true,
                async: true,
                data: value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('view-issue-test-testplans-section.add.error', testkey), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                    if (data.errors && data.errors.length > 0) {
                        var errorMsg = '';
                        $.each($(data.errors), function(i, msg) {
                            errorMsg = errorMsg + msg;
                            if (i < data.errors.length) {
                                errorMsg = errorMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-test-testplans-section.add.controlled.error', testkey), errorMsg);
                    }

                    if (data.testPlans && data.testPlans.length > 0) {
                        self.dataTable.fnAddData(data.testPlans);
                        // JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                    }
                },
                complete: function() {
                    self.$el.unblock();
                    self._updateHistory();
                    self._updateDates();
                }
            });
        },

        _updateHistory: function() {
            if (AJS.$('#changehistory-tabpanel').attr('class') == 'active') {
                var alt_token = AJS.$('input[name="atl_token"]').val();
                $.ajax({
                    type : "GET",
                    url : contextPath + '/browse/' + this.testKey +"?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="+alt_token,
                    contentType : "text/plain",
                    success : function (data, status) {
                        var tab = AJS.$(data).find('#activitymodule div.mod-content #issue_actions_container');
                        var jiraTab = AJS.$('#activitymodule div.mod-content').find('#issue_actions_container');
                        $(jiraTab).html(tab);
                    }
                });
            }
        },

        _updateDates: function() {

            var updatedDate = null;

            if(AJS.$('#datesmodule #updated-date').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
            } else if(AJS.$('#datesmodule #updated-val').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
            } else {
                return null;
            }

            $.ajax({
                type : "GET",
                url : AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
                contentType : "application/json",
                success : function (data) {
                    var title = data.title;
                    var dateTimeIso8601 = data.dateTimeIso8601;
                    updatedDate.attr("datetime", dateTimeIso8601);
                    $.each(AJS.$('#datesmodule dd.date'), function() {
                        if($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                            $(this).attr("title", title);
                        }
                    });
                    updatedDate.livestamp();
                }
            });
        }
    });

    RAVEN.Test.TestPlansView.REFRESH_VIEW = "refreshTestPlansView";

})(AJS.$);
