(function($) {

    RAVEN.namespace("RAVEN.Test");

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Test.TestRunsTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/test/' + this._getTestIssueKey() + '/columns';
        },

        defaults: function() {
            return [];
        },

        _getTestIssueKey: function() {
            return $('#raven-test-testruns-issue-key').val();
        }
    });

    /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Test.TestRunsView = Backbone.View.extend({
        el: '#raven-test-testruns',

        /**
         * DataTable Instance
         */
        dataTable: null,

        /**
         * The Test issue key
         */
        testKey: null,

        /**
         * The Project key
         */
        projectKey: null,

        /**
         * The assignee
         */
        assignee: null,

        /**
         * The TestExecution issue type
         */
        testExecutionIssueType: null,

        /**
         * Filter Instance
         */
        projectFilter: null,
        statusFilter: null,
        versionFilter: null,
        clearFilter: null,
        datePickerStart: null,
        datePickerEnd: null,
        milisecStart: null,
        milisecEnd: null,

        /**
         * Permissions
         */
        canExecuteInline: false,

        hasRequiredFields: null,
        currentDropdownTrigger: null,

        columnLayoutView: null,

        events: {
            'click.raven.test #testrun-div-filter': '_toogleTestSectionFilter',
            'change.raven.test #projects-filter': '_onChangeProjectsFilter',
            'change.raven.test #versions-filter': '_onChangeVersionsFilter',
            'change.raven.test #status-filter': '_onChangeStatusFilter',
            'click.raven.test #clean-filter': '_onClickCleanFilter',
            'click.raven.test .item-delete': '_onItemRemoved'
        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.test #raven-run-test-new-test-execution': '_onClickProcessNewTestExecution',
            'click.raven.test #raven-run-test-exiting-test-execution': '_onClickProcessExistingTestExecution',
            'click.raven.test .raven-test-execute-inline': '_onClickInlineExecuteTestRun',
            'click.raven.test .raven-dropdown-trigger': '_onDropDownTriggerClick',
            'aui-dropdown2-hide.raven.test .aui-dropdown2': '_onDownDownHide'
        },

        initialize: function() {
            var self = this;

            this.model = new RAVEN.Test.TestRunsTableMetaModel();

            this._delegateUnscopedEvents();

            AJS.messages.setup();

            this.testKey = $('#raven-test-testruns-issue-key').val();
            this.projectKey = $('#raven-test-testruns-project-key').val();
            this.assignee = $('#raven-test-testruns-assignee').val();
            this.projectId = $("#raven-test-testruns-project-id").val();
            this.hasRequiredFields = $('#has-required-fields').val() === "true";
            this.canExecuteInline = $('#can-execute-inline').val() === 'true';
            this.testExecutionIssueType = $('#testexecution-issue-type-id').val();
            this.testExecCustomField = $("#raven-testexec-test-customFieldId").val();
            this.customField = $("#raven-testexec-test-customFieldEnv").val();
            this.fixVersions = $("#raven-versions").val();

            if (this.fixVersions) {
                this.fixVersions = JSON.parse(this.fixVersions);
            }

            $('#raven-exec-tests').block();
            this.model.fetch({
                success: function() {
                    $('#raven-test-testruns').unblock();
                    if (self.$el.closest(document.documentElement).length > 0) {
                        self.render();
                        self.disableFixVersionInput();
                    }
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-test-testruns-section.column-layout.error'), null, 'error');
                    $('#raven-test-testruns').unblock();
                }
            });
        },

        render: function() {
            //this._clearMessage();

            // Render table skeleton
            var $table = RAVEN.Test.renderTestRunsTable({ 'columns': _.values(this.model.attributes) });

            var $tableWrapper = this.$el.find('#testrun-entries-table-wrapper');
            $tableWrapper.empty();
            $tableWrapper.html($table);

            this._initializeFilter();

            // create the dataTable
            this._createTestRunsDataTable();

            // render test runs toolbar view
            this._renderTestRunsToolbarView();

            $tableWrapper.show();

            this.$el.show();

            this.ready = true;

            return this;
        },

        /**
         * Handles the creation of the datatable.
         */
        _createTestRunsDataTable: function() {
            var self = this;

            var path = contextPath + '/rest/raven/1.0/test/' + this.testKey + "/testrun";

            if (this.dataTable) {
                this.dataTable.fnDestroy();
            }

            this.dataTable = $('#testrun-entries-table').xrayDataTable({
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                    "sEmptyTable": AJS.I18n.getText('common.table.label.no-resources-add-one'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bServerSide": true,
                "sAjaxSource": path,
                "bDestroy": true,
                "sAjaxDataProp": "entries",
                "bProcessing": false,
                "bAutoWidth": false,
                "bFilter": false,
                "sPaginationType": "full_numbers",
                "bSort": true,
                "aaSorting": [],
                "aoColumnDefs": this._createTestRunTableColumnDefs(),
                "aoColumns": this._createTestRunTableColumns(),
                "fnServerData": function(sSource, aoData, fnCallback, oSettings) {
                    self._fnServerData(sSource, aoData, fnCallback, oSettings);
                },
                "fnInitComplete": function(oSettings, json) {
                    self._toogleTestsSection();
                },
                "fnDrawCallback": function(oSettings) {
                    // set initial overflow to allow overflow columns to be visible
                    self.$el.css("overflow", "initial");

                    // hack for drag&drop to work
                    $('#exec-entries-table').closest('div').addClass('datatable-drag-overflow');

                    //tooltip message
                    $(".not-executable").tipsy();
                }
            });

            this._initSortableColumns();
        },

        _initSortableColumns: function() {
            var self = this;

            $('#testrun-entries-table thead tr').sortable({
                items: 'th:not(.fixed)',
                placeholder: "dragging-placeholder",
                update: function(event, ui) {
                    var ancorNode = ui.item.prev();
                    var columnId = $(ui.item).data('column-id');
                    if (columnId) {
                        var anchorColumnId = null;
                        if (!$(ancorNode).hasClass('fixed')) {
                            anchorColumnId = $(ancorNode).data('column-id');
                        }

                        self.columnLayoutView.moveColumn(columnId, anchorColumnId);
                    }
                }
            });
        },

        /**
         * Creates the DataTable column descriptors dynamically based on the user column layout
         */
        _createTestRunTableColumns: function() {
            var userColumnDefs = this._createTestRunUserColumns();

            var lastColumnDefs = [{
                    "mData": "status"
                },
                {
                    "mData": null
                }
            ];

            return userColumnDefs.concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createTestRunTableColumnDefs: function() {
            var self = this;
            var userColumnDefs = this._createDataTableUserColumn();

            var lastColumnDefs = [{
                    "mData": "status",
                    "bSortable": true,
                    "sWidth": "110px",
                    "mRender": function(data, type, row) {
                        if (data) {
                            return "<span data-status-id=" + data.id + " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:" + data.color + "\" title=\"" + data.description + "\">" + data.name + "</span>";
                        } else {
                            return "-";
                        }
                    },
                    "aTargets": [_.values(this.model.attributes).length]
                },
                {
                    "mData": null,
                    "bSortable": false,
                    "sWidth": "114px",
                    "sClass": "test-lastcol-actions",
                    "mRender": function(data, type, row) {
                        var hasStartedExecuting = false;
                        var canExecuteTestRun = row["canExecute"];
                        var htmlMenus = '';

                        if (row && row.status) {
                            hasStartedExecuting = !(row.status.name == "TODO");
                        }

                        if (canExecuteTestRun === true || hasStartedExecuting) {
                            htmlMenus += RAVEN.Test.testRunMenu({
                                key: row["key"],
                                canExecute: canExecuteTestRun,
                                hasStartedExecuting: hasStartedExecuting,
                                canExecuteInline: self.canExecuteInline && canExecuteTestRun,
                                executeHref: self._getExecuteTestHref(row["key"]),
                                transitions: row["transitions"],
                                isFinal: row["status"].isFinal
                            });
                        }

                        if (canExecuteTestRun === false) {
                            var str = row["disallowedExecuteMessage"];
                            var element = '<a class="not-executable" title="' + str + '"></a>';
                            htmlMenus += element;
                        }

                        return htmlMenus;
                    },
                    "aTargets": [_.values(this.model.attributes).length + 1]
                }
            ];

            return userColumnDefs.concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column for user columns
         */
        _createTestRunUserColumns: function() {
            return _.map(_.values(this.model.attributes), function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id
                };
            });
        },

        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumn: function() {
            return _.map(_.values(this.model.attributes), function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id,
                    "bSortable": column.sortable,
                    "bSearchable": false,
                    "aTargets": [index]
                };
            });
        },

        _renderTestRunsToolbarView: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#toolbar-wrapper').length > 0) {
                return;
            }

            var toolbarHtml = RAVEN.Test.renderTestRunsToolBar();
            $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.Test.TestsColumnLayoutView({
                mainView: this
            });
        },

        _toogleTestSectionFilter: function() {
            var self = this;

            if ($('#testruns-section-filter') != undefined &&
                $('#testruns-section-filter').hasClass('close')) {
                $('#testruns-section-filter').removeClass('close');
                $('#testruns-section-filter').removeClass('collapsed');
                $('#testruns-section-filter').addClass('open');
                $('#testruns-section-filter').addClass('expanded');
                $('#run-filter').show();
            } else {
                self._cleanFilter();
                $('#testruns-section-filter').removeClass('open');
                $('#testruns-section-filter').removeClass('expanded');
                $('#testruns-section-filter').addClass('close');
                $('#testruns-section-filter').addClass('collapsed');
                $('#run-filter').hide();
            }
        },

        _toogleTestsSection: function() {
            var self = this;
            if (self.dataTable && self.dataTable.fnRecordsTotal() > 0) {
                $('#raven-test-testruns .notests').hide();
                $('#test-testruns-section').show();
            } else {
                $('#test-testruns-section').hide();
                $('#raven-test-testruns .notests').show();
            }
        },

        _fnServerData: function(sSource, aoData, fnCallback, oSettings) {
            var self = this;

            if ($('#test-testruns-filter-module').hasClass('expanded') && self.clearFilter != true) {
                var versions = $('#versions-filter').val();
                if (versions) {
                    aoData.push({ "name": "filter_fixVersions", "value": versions });
                }

                var project = $('#projects-filter option:selected').val();
                if (project && project != 'none') {
                    aoData.push({ "name": "filter_project", "value": project });
                }
                var status = $('#status-filter').val();
                if (status) {
                    aoData.push({ "name": "filter_testrun_status", "value": status });
                }

                if (self.milisecStart) {
                    aoData.push({ "name": "filter_testrun_started", "value": self.milisecStart });
                }

                if (self.milisecEnd) {
                    aoData.push({ "name": "filter_testrun_finished", "value": self.milisecEnd });
                }

                self.clearFilter = false;
            }

            $('#raven-test-testruns').block();

            oSettings.jqXHR = $.ajax({
                dataType: 'json',
                type: "GET",
                url: sSource,
                data: aoData,
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-test-testruns-section.processingerror'), errorThrown);
                },
                complete: function() {
                    $('#raven-test-testruns').unblock();
                },
                success: fnCallback
            });
        },

        _showInitializationError: function(message) {
            AJS.messages.error("#raven-initialization-error", {
                body: message,
                closeable: true
            });
        },

        _createVersionOptions: function($selectField, data) {
            var group = $('<optgroup label="All Versions"></optgroup>');
            $(data).each(function(i, ver) {
                group.append('<option value="' + ver.id + '" title="' + ver.description + '">' + ver.name + '</option>');
            });
            $selectField.append(group);
        },

        _updateVersionPicker: function($selectField, projectKey) {
            var self = this;

            if (!projectKey || projectKey === "none")
                return false;

            $.ajax({
                type: "GET",
                url: contextPath + '/rest/api/2/project/' + projectKey + '/versions',
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    var message = AJS.I18n.getText('dialogs.issueselector.search.init.version.error', errorThrown);
                    self._showInitializationError(message);
                },
                success: function(data, textStatus, jqXHR) {
                    self._createVersionOptions($selectField, data);
                    versionPicker._createSelectModel();
                    versionPicker._setInitState();
                    // trigger an initialized so that MultiSelect gets the correct size on JIRA 6.1
                    versionPicker.model.$element.trigger("initialized", [versionPicker]);
                }
            });
        },

        _createProjectPicker: function($selectField) {
            var self = this;

            self.projectFilter = new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/project',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-projects-opt-group",
                            replace: true,
                            label: "All Projects"
                        });

                        $(data).each(function(i, proj) {
                            var item = new AJS.ItemDescriptor({
                                value: proj.key,
                                label: proj.name,
                                html: '<div style="overflow: hidden;text-overflow: ellipsis;" title="' + proj.key + ' - ' + proj.name + '" ><img class="icon" alt="" src="' + proj.avatarUrls['16x16'] + '" />' + proj.key + " - " + proj.name + '</div>',
                                icon: proj.avatarUrls['16x16'],
                                allowDuplicate: false,
                                selected: null
                            });
                            optgroup.addItem(item);
                        });
                        return [self._noneItem(AJS.I18n.getText("view-issue-testrun-section.filter.all-projects")), optgroup];
                    }
                }
            });
        },

        _createVersionPicker: function($selectField) {
            versionPicker = new AJS.MultiSelect({
                element: $selectField,
                itemAttrDisplayed: "label",
                removeOnUnSelect: false,
                submitInputVal: true,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.versions.error"),
                maxInlineResultsDisplayed: 15,
            });
        },

        _cleanFilter: function() {
            this.milisecStart = null;
            this.milisecEnd = null;

            if (this.projectFilter) {
                this.projectFilter.setSelection(this._noneItem(AJS.I18n.getText("view-issue-testrun-section.filter.all-projects")));

                $("#projects-filter-single-select").removeClass("aui-ss-has-entity-icon");
            }

            if (this.versionFilter) {
                versionPicker.clearLozenges();
                $('#versions-filter').empty();
            }

            if (this.statusFilter) {
                this.statusFilter.clearLozenges();
            }

            $('#date-start-filter').val(null);
            $('#date-end-filter').val(null);

            $('#testrun-entries-table').xrayDataTable().fnDraw(false);

        },

        _onItemRemoved: function() {
            var self = this;

            if ($('#status-filter').val() != "") {
                self.clearFilter = false;
                $('#testrun-entries-table').xrayDataTable().fnDraw(false);
            }
        },

        _initializeFilter: function() {
            var self = this;

            self.clearFilter = false;
            if (self.statusFilter == null) {
                $projectField = $('#projects-filter');

                self._createProjectPicker($projectField);

                self._createVersionPicker($('#versions-filter'));

                self.statusFilter = new AJS.MultiSelect({
                    element: $('#status-filter'),
                    revertOnInvalid: true
                });

                self._initRavenCalendar();

                $('#test-testruns-filter-module').show();
            }
        },

        _getExecuteTestHref: function(testExecKey) {
            var self = this;

            var testKey = self.testKey;
            return contextPath + '/secure/XrayExecuteTest!default.jspa?testExecIssueKey=' + testExecKey + '&testIssueKey=' + testKey;
        },

        /**
         * Redraws the datatable
         */
        refresh: function() {
            this._cleanView();
            this.initialize();
        },

        destroy: function() {
            this._cleanView();

            if (this.columnLayoutView) {
                this.columnLayoutView.destroy();
                this.columnLayoutView = null;
            }

            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();

            this.$el.removeData().unbind();

            this.unbind();

            this.remove();
        },

        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.model = null;

            var $tableWrapper = this.$el.find('#testrun-entries-table-wrapper');

            $tableWrapper.empty();
        },

        _processNewTestExecution: function() {
            var self = this;

            var testKey = self.testKey;
            var projectKey = self.projectKey;
            var assignee = self.assignee;
            var summary = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.create-new.summary', testKey);
            var labels = {};
            labels.title = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.create-new.title', testKey);
            labels.enter = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.create-new.enter');

            var fixVersions = self.fixVersions;
            var customField = self.customField;
            var revisionCF = $("#raven-testexec-test-revisionId").val();

            if (!self.hasRequiredFields) {
                new RAVEN.Dialogs.createSimpleTestExecDialog(labels, summary, projectKey, assignee, fixVersions, customField, function(projectKey, summary, assignee, canRedirectToExecutePage, versions, env, revision) {
                    self._addTestExec([], projectKey, summary, assignee, canRedirectToExecutePage, versions, env, revision);
                });
            } else {
                var testExecIssueType = parseInt(this.testExecutionIssueType);
                var createForm = JIRA.Forms.createCreateIssueForm({
                    issueType: testExecIssueType
                });

                // select trigger for create issue in global context
                var dialog = createForm.asDialog({
                    id: 'raven-create-exec-dialog'
                });

                createForm.bind("sessionComplete", function(e, data) {
                    if (data && data.length > 0) {
                        var issue = data[data.length - 1]; //get the last issue created
                        if (issue) {
                            self._buildSuccessMessageJiraDialog(issue.issueKey, issue.summary);

                            self._updateHistory();
                            self._updateDates();

                            // refresh the test runs table
                            self.refresh();
                        }
                    }
                });

                dialog.show();

                JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {

                    var dialog = $('#raven-create-exec-dialog.jira-dialog-open, #raven-create-exec-dialog.aui-dialog-open');

                    context.find('.qf-create-another').hide();

                    if (dialog.length > 0 && dialog.find("#summary").val() !== summary) {

                        // add Summary
                        context.find("#summary").val(summary);

                        // add Test
                        context.find('select[id=' + self.testExecCustomField + ']').trigger('selectOption', [{ label: testKey, value: testKey, title: testKey }]);

                        // Fixed Versions
                        $(fixVersions).each(function(index, version) {
                            context.find('select[id="fixVersions"] option').each(function() {
                                $(dialog).find("#fixVersions").trigger('selectOption', [{ value: version.value, title: version.name, label: version.name }]);
                            });
                        });

                        // add assignee
                        var assignee;
                        $.ajax({
                            type: "GET",
                            url: contextPath + '/rest/api/2/user?key=' + self.assignee,
                            async: false,
                            contentType: "application/json",
                            success: function(data) {
                                assignee = data;
                            }
                        });

                        context.find("#assignee-field").val(assignee.displayName);
                        context.find("#assignee-single-select .aui-ss-entity-icon").attr("src", assignee.avatarUrls['16x16']);

                        context.find("#assignee").append('<option value="' + assignee.key + '" title="undefined" selected="selected" style="background-image: url("' + assignee.avatarUrls['16x16'] + '");">' + assignee.displayName + '</option>')
                        context.find("#assignee").val(assignee.key);
                        context.find('#assignee').trigger('change');

                        // add project needs to be last because it refreshes(locks) dialog.
                        context.find('input[id=project]').val(self.projectId);
                        context.find('input[id=project]').trigger('change');
                    }
                });
            }
        },

        _processExistingTestExecution: function() {
            var self = this;
            var testKey = self.testKey;
            var issueTypeId = self.testExecutionIssueType;
            var labels = {};
            labels.label = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.associate-existing.label');
            labels.title = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.associate-existing.title', testKey);
            labels.description = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.associate-existing.description');
            labels.enter = AJS.I18n.getText('view-issue-testrun-section.action.associate-test-execution.associate-existing.enter');

            new RAVEN.Dialogs.createIssueSelectorDialog('TESTEXECUTION', 'key not in testTestExecutions(' + testKey + ')', issueTypeId, testKey, labels, function(issues, canRedirectToExecutePage) {
                if (issues && issues.length > 0) {
                    self._addTestExec(issues, null, null, null, canRedirectToExecutePage);
                }
            }, null, null, true);
        },

        _addTestExec: function(issues, projectKey, summary, assignee, canRedirectToExecutePage, versions, env, revision) {
            var self = this;

            var testKey = self.testKey;

            $('#raven-test-testruns').block();

            var path;
            if (issues.length === 0) {
                path = contextPath + '/rest/raven/1.0/test/' + testKey + '/createtestexec?projectKey=' + projectKey + "&summary=" + summary + "&assignee=" + assignee + "&fixVersion=" + versions + "&environment=" + env + "&revision=" + revision;
            } else {
                path = contextPath + '/rest/raven/1.0/test/' + testKey + '/addtestexec';
            }

            var value = JSON.stringify(issues);
            $.ajax({
                type: "POST",
                url: path,
                processData: true,
                async: true,
                timeout: 60000, // sets timeout to 60 seconds
                data: value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    self._clearMessage();
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-testrun-section.action.add.error', testKey), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self._clearMessage();
                    if (data.tests && data.tests.length > 0) {
                        if (canRedirectToExecutePage) {
                            window.location.href = self._getExecuteTestHref(data.tests[0]);
                            return;
                        }
                    }
                    if (data.errors && data.errors.length > 0) {
                        var errorMsg = '';
                        $.each($(data.errors), function(i, msg) {
                            errorMsg = errorMsg + msg;
                            if (i < data.errors.length) {
                                errorMsg = errorMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-testrun-section.action.add.controlled.error', testKey), errorMsg, "error");
                    }

                    if (data.warnings && data.warnings.length > 0) {
                        var warningMsg = '';
                        $.each($(data.warnings), function(i, msg) {
                            warningMsg = warningMsg + msg;
                            if (i < data.warnings.length) {
                                warningMsg = warningMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-testrun-section.action.add.controlled.warning', testKey), warningMsg, "warning");
                    }

                    // refresh view
                    if (data.tests && data.tests.length > 0) {
                        self._buildsuccessMessage(data.tests, data.summaries);
                        self.refresh();
                    }
                },
                complete: function() {
                    self._updateHistory();
                    self._updateDates();
                    $('#raven-test-testruns').unblock();
                }
            });
        },

        _buildsuccessMessage: function(tests, summaries) {
            var self = this;
            var successMsg = '';
            $.each(tests, function(i, testExec) {
                successMsg = successMsg + AJS.I18n.getText('view-issue-testrun-section.action.add.success', contextPath, testExec, self.testKey, self._getExecuteTestHref(testExec), summaries[i]);
                if (i < tests.length) {
                    successMsg = successMsg + '<br/>';
                }
            });

            self.message(AJS.I18n.getText('view-issue-testrun-section.action.add.controlled.success', self.testKey), successMsg, "success");
        },

        _buildSuccessMessageJiraDialog: function(testExec, summary) {
            var self = this;
            self._clearMessage();
            self.message(AJS.I18n.getText('view-issue-testrun-section.action.add.controlled.success', self.testKey), AJS.I18n.getText('view-issue-testrun-section.action.add.success', contextPath, testExec, self.testKey, self._getExecuteTestHref(testExec), summary), "success");
        },

        _onSelectDateStart: function(inst, dateText) {
            this.milisecStart = null;
            this.clearFilter = false;
            if (inst.date != undefined && inst.date != null) {
                this.milisecStart = this._convertToMilisec(inst.date);
                if (this.milisecStart != null && !isNaN(this.milisecStart)) {
                    $('#raven-initialization-error').empty()
                    if (this.milisecEnd != null && !isNaN(this.milisecEnd) && this.milisecEnd < this.milisecStart) {
                        this._showInitializationError(AJS.I18n.getText('filter.date.end.greater.start'));
                        $('#date-start-filter').val(null);
                        this.milisecStart = null;
                    } else {
                        $('#date-start-filter').val(dateText);
                    }
                }
            }
            if (inst.dateClicked == true) {
                inst.hide();
            }
            $('#testrun-entries-table').xrayDataTable().fnDraw(false);
        },

        _onSelectDateEnd: function(inst, dateText) {
            this.milisecEnd = null;
            this.clearFilter = false;

            if (inst.date != undefined && inst.date != null) {
                this.milisecEnd = this._convertToMilisec(inst.date);
                $('#raven-initialization-error').empty()
                if (this.milisecEnd != null && !isNaN(this.milisecEnd) && this.milisecEnd < this.milisecStart) {
                    this._showInitializationError(AJS.I18n.getText('filter.date.end.greater.start'));
                    $('#date-end-filter').val(null);
                    this.milisecEnd = null;
                } else {
                    $('#date-end-filter').val(dateText);
                }
            }
            if (inst.dateClicked == true) {
                inst.hide();
            }
            $('#testrun-entries-table').xrayDataTable().fnDraw(false);
        },

        _convertToMilisec: function(dateTime) {
            var milisec = new Date(dateTime);
            if (milisec != null || !isNaN(milisec)) {
                return milisec.valueOf();
            }

            return null;
        },

        _initRavenCalendar: function() {
            var self = this;

            self.datePickerStart = Calendar.setup({
                firstDay: 1,
                inputField: 'date-start-filter',
                align: 'Br',
                onSelect: $.proxy(self._onSelectDateStart, self),
                singleClick: true,
                showsTime: true,
                useISO8601WeekNumbers: false,
                ifFormat: '%d-%m-%Y %H:%M'
            });

            self.datePickerEnd = Calendar.setup({
                firstDay: 1,
                inputField: 'date-end-filter',
                align: 'Br',
                onSelect: $.proxy(self._onSelectDateEnd, self),
                singleClick: true,
                showsTime: true,
                useISO8601WeekNumbers: false,
                ifFormat: '%d-%m-%Y %H:%M'
            });
        },

        //XRAY-312
        _inlineExecuteTestRun: function($row, statusId) {
            var self = this;
            $('#raven-test-testruns').block();

            var rowData = self.dataTable.fnGetData($row[0]);

            if (rowData.status.id !== statusId) {
                var testKey = self.testKey;
                var testExecKey = rowData.key;

                var path = contextPath + '/rest/raven/1.0/test/' + testKey + '/execute/' + testExecKey;

                $.ajax({
                    type: "POST",
                    url: path + "?status=" + statusId,
                    dataType: "json",
                    processData: true,
                    contentType: "application/json",
                    async: true,
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }

                        self.message(AJS.I18n.getText('view-issue-testexec-tests-section.inline-execute.error', testExecKey), errorThrown)
                    },
                    success: function(data, textStatus, jqXHR) {
                        self._clearMessage();
                        if (data.obj) {
                            self.dataTable.fnUpdate(data.obj, $row[0], undefined, false, false);
                        }
                        if (data.errors.length > 0) {
                            var finalErrors = "";
                            for (var i = 0; i < data.errors.length; i++) {
                                finalErrors += data.errors[i] + "\n";
                            }

                            self.message(AJS.I18n.getText('test-execution.workflow-status.error-message.title'), finalErrors, "warning");
                        }
                    },
                    complete: function() {
                        $('#raven-test-testruns').unblock();
                    }
                });
            }
        },

        _updateHistory: function() {
            if (AJS.$('#changehistory-tabpanel').attr('class') === 'active') {
                var alt_token = AJS.$('input[name="atl_token"]').val();
                $.ajax({
                    type : "GET",
                    url : contextPath + '/browse/' + this.testKey +"?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="+alt_token,
                    contentType : "text/plain",
                    success : function (data, status) {
                        var tab = AJS.$(data).find('#activitymodule div.mod-content #issue_actions_container');
                        var jiraTab = AJS.$('#activitymodule div.mod-content').find('#issue_actions_container');
                        $(jiraTab).html(tab);
                    }
                });
            }
        },

        _updateDates: function() {

            var updatedDate = null;

            if(AJS.$('#datesmodule #updated-date').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
            } else if(AJS.$('#datesmodule #updated-val').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
            } else {
                return null;
            }

            $.ajax({
                type : "GET",
                url : AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
                contentType : "application/json",
                success : function (data) {
                    var title = data.title;
                    var dateTimeIso8601 = data.dateTimeIso8601;
                    updatedDate.attr("datetime", dateTimeIso8601);
                    $.each(AJS.$('#datesmodule dd.date'), function() {
                        if($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                            $(this).attr("title", title);
                        }
                    });
                    updatedDate.livestamp();
                }
            });
        },

        _clearMessage: function() {
            $('#raven-test-testruns-message-bar').empty();
        },

        message: function(title, message, type) {
            type = (type === undefined) ? "error" : type;
            this._clearMessage();

            if (type === "error") {
                AJS.messages.error('#raven-test-testruns-message-bar', {
                    title: title,
                    body: message
                });
            } else if (type === "info") {
                AJS.messages.info('#raven-test-testruns-message-bar', {
                    title: title,
                    body: message
                });
            } else if (type === "warning") {
                AJS.messages.warning('#raven-test-testruns-message-bar', {
                    title: title,
                    body: message
                });
            } else if (type === "success") {
                AJS.messages.success('#raven-test-testruns-message-bar', {
                    title: title,
                    body: message
                });
            }
        },

        _noneItem: function(string) {
            return new AJS.ItemDescriptor({
                value: 'none',
                label: string,
                icon: null,
                allowDuplicate: false,
                selected: true
            });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.test');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.test');
        },

        enableFixVersionInput: function () {
            $("#versions-filter-multi-select :input").prop("disabled", false);
            $("#versions-filter-multi-select .icon.noloading.drop-menu").css("display","block");
            $("#versions-filter-multi-select textarea").removeAttr("placeholder");
        },

        disableFixVersionInput: function () {
            $("#versions-filter-multi-select :input").prop("disabled", true);
            $("#versions-filter-multi-select .icon.noloading.drop-menu").css("display","none");
            $("#versions-filter-multi-select textarea").attr("placeholder", AJS.I18n.getText('view-issue-test-testruns-section.label.project-dependent-version.placeholder'));
        },

        _onChangeProjectsFilter: function() {
            var self = this;

            var newSelectedValue = $projectField.find('option:selected').attr('value');
            if (this.selectedProjectKey !== newSelectedValue) {
                this.selectedProjectKey = newSelectedValue;
                if (newSelectedValue === "none") {
                    $("#projects-filter-single-select").removeClass("aui-ss-has-entity-icon");
                    self.disableFixVersionInput();
                } else {
                    self.enableFixVersionInput();
                }
                // update version picker
                if (versionPicker) {
                    versionPicker.clearLozenges();
                    $('#versions-filter').empty();
                    self._updateVersionPicker($('#versions-filter'), newSelectedValue);
                }
            }
            if ($('#projects-filter').val() != "") {
                self.clearFilter = false;
                $('#testrun-entries-table').xrayDataTable().fnDraw(false);
            }
        },

        _onChangeVersionsFilter: function() {
            var self = this;

            AJS.$("#versions-filter-multi-select .item-delete").bind("click", function(e) {
                e.preventDefault();
                e.stopPropagation();
                $('#testrun-entries-table').xrayDataTable().fnDraw(false);
            });
            $('#testrun-entries-table').xrayDataTable().fnDraw(false);

            self.clearFilter = false;
            return false;
        },

        _onChangeStatusFilter: function() {
            var self = this;

            if ($('#status-filter').val() != "") {
                self.clearFilter = false;
                $('#testrun-entries-table').xrayDataTable().fnDraw(false);
            }
        },

        _onClickCleanFilter: function(e) {
            var self = this;

            e.preventDefault();
            e.stopPropagation();

            /* Clear Filter default values */
            self.clearFilter = true;
            self._cleanFilter();
        },

        _onClickProcessNewTestExecution: function() {
            var self = this;

            self._clearMessage();
            self._processNewTestExecution();
        },

        _onClickProcessExistingTestExecution: function() {
            var self = this;

            self._clearMessage();
            self._processExistingTestExecution();
        },

        _onClickInlineExecuteTestRun: function(e) {
            var self = this;

            e.preventDefault();
            e.stopPropagation();
            console.log("hello");
            if (self.currentDropdownTrigger) {
                var $row = $(self.currentDropdownTrigger).closest('tr');
                var statusId = $(e.currentTarget).data('status-id');
                console.log($row);

                if ($row.length > 0) {
                    self._inlineExecuteTestRun($row, statusId);
                }
            }

            self.currentDropdownTrigger = null;

            return false;
        },

        /**
         * Save current dropdown trigger
         */
        _onDropDownTriggerClick: function(e) {
            this.currentDropdownTrigger = $(e.target);
            this.currentDropdownTrigger.closest('tr').addClass('active');
        },

        /**
         * Clear current dropdown trigger
         */
        _onDownDownHide: function() {
            if (this.currentDropdownTrigger) {
                this.currentDropdownTrigger.closest('tr').removeClass('active');
            }
        }
    });

    RAVEN.Test.TestRunsView.REFRESH_VIEW = "refreshTestRunsView";

})(AJS.$);