(function($) {

	var canEditTestExec = false;
	var milisecStart = null;
	var milisecEnd = null;
	var oldStart = null;
	var oldEnd = null;

	window.RAVEN = window.RAVEN || {};

	RAVEN.InlineEdit = RAVEN.InlineEdit || {};

	/*
	 * Date Field
	 */
	RAVEN.InlineEdit.DateField = RAVEN.InlineEdit.TextField.extend({
		initialize: function(elem, callbacks) {
			// call parent constructor
			RAVEN.InlineEdit.DateField.__super__.initialize.apply(this, [elem, callbacks]);

			// do not propagate click event on calendar
			$(document).on("click.inline-edit", '.calendar.active', function(e) {
				e.stopPropagation();
			});
		},
		
		getSubmitValue: function() {
			var value = this.getEditingValue();
            
            //When cleaning the value in the field
            if(value == ""){
                return null;
            }
            
            if(!validateDateTimeValueInserted(value)){
                this.cancel(); // cancel changes
                value = this.getValue(); // get previously setted value
                //if previous value was empty
                if(value == ""){
                    return null;
                }
            }
            
            var ms = convertToMilisec(value);
            //JIRA saves all dates in UTC, so we must get the offset between the
            //jira user prefered timezone ant UTC and  then calculate the accurate time
            //var offsetInMilliseconds = $('#user-time-zone-off-set').val();
            //ms = ms - offsetInMilliseconds;
                        
			if (ms && ms > 0) { // this means no date has been selected on the UI component
				var milisec = new Date(ms);
                return moment(milisec).format("YYYY-MM-DDTHH:mm:ss.SSSZZ");
			}
            
			return null;
		}
	});
	
	function getTestExecKey () {
		return $('#raven-testexec-details-issue-key').val();
	}
	
	function updateHistory() {
		if (AJS.$('#changehistory-tabpanel').attr('class') == 'active') {
			var alt_token = AJS.$('input[name="atl_token"]').val();
			$.ajax({
				type : "GET",
				url : contextPath + '/browse/' + getTestExecKey() +"?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="+alt_token,
				contentType : "text/plain",
				success : function (data, status) {
					var tab = AJS.$(data).find('#activitymodule div.mod-content #issue_actions_container');
					var jiraTab = AJS.$('#activitymodule div.mod-content').find('#issue_actions_container');
					$(jiraTab).html(tab);
				}
			});
		}
	}

    
    function validateDateTimeValueInserted(dateTime){
        if(dateTime == "" || dateTime == undefined) {
			return false;
		}
		
		dateTime = dateTime.replace(/[-,:\s]/g, ",");
		var parts = dateTime.split(",");
        
        if(parts == null){
            return false;   
        }
 
        if(parts.length == 0){
            return false;   
        }
        
        if(parts.length > 0 && ( parts[0]<1 || parts[0]>31)){
            return false;
        }  

        if(parts.length > 1 && ( parts[1]<1 ||  parts[1]>12)){
            return false;
        }
        
        if(parts.length > 2 && parts[2]>9999){
            return false;
        }
 
        if(parts.length > 3 && ( parts[3]<0 ||  parts[3]>23)){
            return false;
        }

        if(parts.length > 4 && ( parts[4]<0 ||  parts[4]>59)){
            return false;
        }
        
         //check leap years
        isLeap = new Date(parts[2], 1, 29).getMonth() == 1;

        if(isLeap && parts[0]>29){
            return false;
        }
             
        if(!isLeap && parts[0]>28 && parts[1]==2){
            return false;
        }
        
        return true;
    }
    

    function convertToMilisec (dateTime) {
		var userTimeZone = $('#user-time-zone').val();		
		
		if(dateTime == "" || dateTime == undefined) {
			return false;
		}
		
		dateTime = dateTime.replace(/[-,:\s]/g, ",");
		var parts = dateTime.split(",");
		moment.tz.setDefault(userTimeZone);
		var milisec = moment();
		milisec.year(parts[2]);
		milisec.month(parts[1]-1);
		milisec.date(parts[0]);
		if (parts[3]) {
			milisec.hours(parts[3]);
			if (parts[4]) {
				milisec.set('minute', parts[4]);
			} else {
				milisec.set('minute', 0);
			}
		} else {
			milisec.hours(0);
			milisec.set('minute', 0);
		}
				
		if (milisec != null && !isNaN(milisec)) {
			return milisec.valueOf();
		}
		return null;
	}
    
    Date.prototype.stdTimezoneOffset = function() {
        var jan = new Date(this.getFullYear(), 0, 1);
        var jul = new Date(this.getFullYear(), 6, 1);

        return Math.max(jan.getTimezoneOffset(), jul.getTimezoneOffset());
    }

    Date.prototype.dst = function() {
        return this.getTimezoneOffset() < this.stdTimezoneOffset();
    }
  
	function convertToMilisecFromString (dateTime) {
		var milisec = Date.parse(dateTime);
		if (milisec != null || !isNaN(milisec)) {
			return milisec.valueOf();
		}
		
		return null;
	}
	
	function clearMessages (id) {
		var element = $('#'+id);
		if ($(element) != undefined) {
			$(element).empty();
		}
	}
	
	function validateDate(e){
		var objDate = $(e).parent().parent().find('input');
		if (objDate) {
			if ($('#testexec-start-date-field-date-id').val() != "" && $('#testexec-end-date-field-date-id').val() != "") {
				milisecStart = convertToMilisec ($('#testexec-start-date-field-date-id').val());
				milisecEnd = convertToMilisec ($('#testexec-end-date-field-date-id').val());
				$("#raven-testexec-details-error-message").empty();
				if(milisecStart != null && !isNaN(milisecStart) && milisecStart > milisecEnd) {
					showInitializationError(AJS.I18n.getText('filter.date.end.greater.start'));
					milisecStart = null;
					milisecEnd = null;
					$('#testexec-start-date-field-date-id').val(null);
					return false;
				}else if(milisecEnd != null && !isNaN(milisecEnd) && milisecEnd < milisecStart) { 
					showInitializationError(AJS.I18n.getText('filter.date.end.greater.start'));
					milisecEnd = null;
					milisecStart = null;
					$('#testexec-end-date-field-date-id').val(null);
					return false;
				}
				
				return true;
			}
		}
		return true;
	}
	
	function initComponentCalendar (){
		
        if ($('#testexec-start-date-field-date-id').length) {
            milisecStart = ($('#testexec-start-date-field-date-id').val() != "") ? new Date($('#testexec-start-date-field-date-id').val()) : null;

    		Calendar.setup({
    			firstDay : 1,
    			inputField : 'testexec-start-date-field-date-id',
    			button : 'testexec-start-date-field-id-date-trigger',
    			align : 'Br',
    			displayArea: 'working-icon',
    			singleClick : true,
    			showsTime : true,
    			useISO8601WeekNumbers : false,
    			ifFormat : '%d-%m-%Y %H:%M'
    		});
        }

        if ($('#testexec-end-date-field-date-id').length) {
            milisecEnd = ($('#testexec-end-date-field-date-id').val() != "") ? new Date($('#testexec-end-date-field-date-id').val()) : null;
    		Calendar.setup({
    			firstDay : 1,
    			inputField : 'testexec-end-date-field-date-id',
    			button : 'testexec-end-date-field-id-date-trigger',
    			align : 'Br',
    			displayArea: 'working-icon',
    			singleClick : true,
    			showsTime : true,
    			useISO8601WeekNumbers : false,
    			ifFormat : '%d-%m-%Y %H:%M'
    		});
        }
		
        $('.date_time').xrayInputMask('00-00-0000 00:00',{maxlength: false});
	}

	function validateDates () {
        // if any date field is hidden than do not validate dates
        if ($('#testexec-start-date-field-date-id').length == 0 || $('#testexec-end-date-field-date-id').length == 0) {
            return true;
        }

		var start = $('#testexec-start-date-field-date-id').val();
		var end = $('#testexec-end-date-field-date-id').val();

		startDate = convertToMilisec(start);
		endDate = convertToMilisec(end);
		
		if (startDate === false || endDate === false) {
			return true;
		}
		
		if (startDate > endDate) {
			$('#raven-testexec-details-error-message').empty(); // clear previous message
			AJS.messages.error("#raven-testexec-details-error-message", {
				body : AJS.I18n.getText('filter.date.end.greater.start')
			});

			return false;
		}
		
		return true;
	}	
	
	function initialize() {
		AJS.messages.setup();
		canEditTestExec = $('#can-edit-testexec').val() === 'true';

		// inline edit support
		initInlineEditFields(canEditTestExec);
		
		initComponentCalendar();
	}

	function initInlineEditFields(canEditTestExec) {
		// create Inline Edit
        if ($('#testexec-start-date-field').length > 0) {
    		var startDateField = new RAVEN.InlineEdit.DateField(
    			$('#testexec-start-date-field'),
    			{
    				before: validateDates,
    				success: function() {
    					updateHistory();
    					$('#raven-testexec-details-error-message').empty();
    				},
    				error: fieldSaveError
    			}
    		);
        }
        if ($('#testexec-end-date-field').length > 0) {
    		var endDateField = new RAVEN.InlineEdit.DateField(
    			$('#testexec-end-date-field'),
    			{
    				before: validateDates,
    				success: function() {
    					updateHistory();
    					$('#raven-testexec-details-error-message').empty();
    				},
    				error: fieldSaveError
    			}
    		);
        }
        if ($('#testexec-revision-field').length > 0) {
    		var revisionField = new RAVEN.InlineEdit.TextField(
    			$('#testexec-revision-field'),
    			{
    				success: function() {
    					updateHistory();
    					$('#raven-testexec-details-error-message').empty();
    				},
    				error: fieldSaveError
    			}
    		);
        }

		var inlineEdit = new RAVEN.InlineEdit();
	}

	function fieldSaveError(field, error) {
		$('#raven-testexec-details-error-message').empty(); // clear previous message

		var fieldName = $(field.el).data('field-name');
		AJS.messages.error("#raven-testexec-details-error-message", {
			title : AJS.I18n.getText('view-issue-testexec-details-section.save.error', fieldName),
			body : error
		});	
	}
	
	JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
		if ($('#raven-testexec-details:not([data-initialized="true"])').length > 0 && $(context).find('#raven-testexec-details:not([data-initialized="true"])').length > 0) {
			if (reason == JIRA.CONTENT_ADDED_REASON.pageLoad) {
				initialize();
				$('#raven-testexec-details').attr("data-initialized", "true");
			} else if (reason == JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
				initialize();
				$('#raven-testexec-details').attr("data-initialized", "true");
			}
		}
	});

})(AJS.$);
