(function($) {

    RAVEN.namespace("RAVEN.TestExecution");

    function getTestExecKey() { 
        return $('#raven-testexec-tests-issue-key').val(); 
    } 

    /**
    * This Model defines the column layout for the given user
    */
    RAVEN.TestExecution.TestRunFiltersModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testexec/' + getTestExecKey() + '/filters';
        },

        defaults: function() {
            return {
                statuses: []
            }
        },
    });

    /**
     * This is the filters view for the Test Runs table on a Test Execution issue
     */
    RAVEN.TestExecution.TestRunFiltersView = Backbone.View.extend({
        el: '#testexec-tests-filter',
        model: new RAVEN.TestExecution.TestRunFiltersModel(),

        /**
         * Parent Tests view
         */
        mainView: null,

        /**
         * The Test Execution issue key
         */
        testExecKey: null,

        /**
         * Test Set single select filter object
         */
        testSetSingleSelect: null,

        /**
         * Status single select filter object
         */
        statusMultiSelect: null,

        /**
         * Assignee single select filter
         */
        assigneeSingleSelect: null,

        /**
         * Stores the last selected Test Set key
         */
        selectedTestSetKey: null,

        /**
         * Stores the last selected Components
         */
        componentsMultiSelect: null,

        /**
         * Stores the last assignee value of the filter
         */
        lastAssigneeFilterValue: null,

        /**
         * Stores the last status value of status filter
         */
        lastStatusFilterValue: null,

        /**
         * Stores the last components value of components filter
         */
        lastComponentsValue: null,

        /**
         * Stores the last entered value on text filter
         */
        lastTextFilterValue: null,

        allDefaultItem: new AJS.ItemDescriptor({
            value: '-1',
            label: AJS.I18n.getText("common.label.all"),
            allowDuplicate: false,
            icon: null
        }),

        events: {
            'keyup #testexec-tests-filter-contains:not(.disabled)': '_onTextFilterKeyUp',
            'change #testexec-tests-filter-testsets:not(.disabled)': '_onTestSetsFilterChanged',
            'change #testexec-tests-filter-assignee:not(.disabled)': '_onAssigneeFilterChanged',
            'change #testexec-tests-filter-status:not(.disabled)': '_onStatusFilterChanged',
            'change #testexec-tests-filter-component:not(.disabled)': '_onComponentsFilterChanged',
            'click .item-delete': '_onItemRemoved',
            'click #testexec-tests-filter-clean': '_onCleanFilterClick'
        },

        initialize: function(options) {
            var self = this;

            if (options) {
                _.extend(this, _.pick(options, "mainView"));
            }

            this.testExecKey = $('#raven-testexec-tests-issue-key').val();

            self.$el.block();
            this.model.fetch({
                success: function() {
                    self.render();
                    self.$el.unblock();
                },
                error: function(model, response) {
                    if (self.mainView) {
                        self.mainView.message(AJS.I18n.getText('view-issue-testexec-tests-section.table-filter.error'));
                    }
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            this.$el.empty();

            var contents = RAVEN.TestExecution.renderTestRunFilters({ statuses: this.model.get('statuses')});
            this.$el.html(contents);

            this._initializeFilters();

            this.$el.show();

            return this;
        },

        _initializeFilters: function() {
            var self = this;

            if ($('#exec-entries-table').length > 0) {
                this._createTestSetPicker();
            }

            this.statusMultiSelect = new AJS.MultiSelect({
                element : self.$el.find('#testexec-tests-filter-status'),
                revertOnInvalid: true
            });

            this.assigneeSingleSelect = new AJS.SingleSelect({
                element: self.$el.find('#testexec-tests-filter-assignee'),
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: true,
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + "/rest/api/2/user/picker",
                    query: true, // keep going back to the sever for each keystroke
                    data: {
                        showAvatar: true
                    },
                    formatResponse: $.proxy(self._formatAssigneeFilterResponse, self)
                },
                inputText: null
            });

            this.componentsMultiSelect = new AJS.MultiSelect({
    			element: $("#testexec-tests-filter-component"),
    			showDropdownButton: true,
    			submitInputVal: true,
    			noQueryNoRequest: false,
    			revertOnInvalid: true,
    			loadOnInit: true,
    			ajaxOptions: {
    				query: true,
    				url: contextPath + '/rest/api/2/project/' + $('#testexec-project-key').val() + '/components',
    				formatResponse: function (data) {
    					var optgroup = new AJS.GroupDescriptor({
    						weight: 0,
    						id: "all-filter-opt-group",
    						replace: true
    					});

    					$(data).each(function(i, status) {
    						this.label = this.name;
    						var item = new AJS.ItemDescriptor({
    							value: this.name,
    							label: this.name,
    							title: this.description,
    							html: '<div style="overflow: hidden;text-overflow: ellipsis;" title="' + this.name +'" >' + this.name + '</div>',
    							highlighted: true,
    						});
    						optgroup.addItem(item);
    					});

    					return [optgroup];
    				}
    			}
    		});
        },

        /**
         * Create select Test Set filter
         */
        _createTestSetPicker: function() {
            var self = this;

            // prevent form submission
            this.$el.find('form.aui').submit(false);

            var $selectField = this.$el.find('#testexec-tests-filter-testsets');

            var path = contextPath + '/rest/raven/1.0/testexec/' + getTestExecKey() + '/testset/query';

            this.testSetSingleSelect = new AJS.SingleSelect({
                element: $selectField,
                revertOnInvalid: true,
                submitInputVal: true,
                itemAttrDisplayed: "label",
                ajaxOptions: {
                    query: true,
                    processData: true,
                    data: {
                    	showSubTasks: true
                    },
                    url: path,
                    suggestionsHandler: null,
                    formatResponse: function (response) {
                        if (response) {
                            var items = _.map(response, function(item) {
                                return new AJS.ItemDescriptor({
                                    value: item.key,
                                    label: item.key + " - " + item.summary,
                                    icon: contextPath + '/download/resources/com.xpandit.plugins.xray/images/test-set.png',
                                    html: '<img class="icon" alt="" src="' + contextPath + '/download/resources/com.xpandit.plugins.xray/images/test-set.png"><div style="overflow: hidden;text-overflow: ellipsis;" title="'+item.key + ' - ' + item.summary+'" >' +item.key + ' - ' + item.summary+'</div>',
                                    styleClass: "aui-iconised-link",
                                    allowDuplicate: false
                                });
                            });

                            return [self.allDefaultItem].concat(items);
                        }

                        return null;
                    }
                }
            });
            //this.testSetSingleSelect.clear();
        },

        /**
         * @Deprecated
         */
        _createTestSetOptions: function($selectField, data) {
            $selectField.append('<option value="-1" style="background-image: none;">' + AJS.I18n.getText("common.label.all") + '</option>');
            $(data).each(function(i, proj) {
                $selectField.append('<option value="' + proj.key + '">' + proj.name + '</option>');
            });
        },

        _formatAssigneeFilterResponse: function(data) {
            var ret = [];

            var suggestionsGroupDescriptor = new AJS.GroupDescriptor({
                weight: 0, // order or groups in suggestions dropdown
                label: AJS.I18n.getText("common.label.user-suggestions")
            });

            suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                value: "-1",
                label: AJS.I18n.getText("common.label.all"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                value: "\u241Eunassigned\u241E",
                label: AJS.I18n.getText("common.label.unassigned"), // title of lozenge
                icon: null,
                allowDuplicate: false,
                highlighted: true
            }));

            var currentUser = this.model.get('currentUser');
            if (currentUser) {
                suggestionsGroupDescriptor.addItem(new AJS.ItemDescriptor({
                    value: currentUser.key,
                    label: AJS.I18n.getText('common.label.current-user') + " - " + currentUser.displayName, // title of lozenge
                    icon: currentUser.avatarUrls['16x16'],
                    fieldText: currentUser.displayName,
                    allowDuplicate: false,
                    highlighted: true
                }));
            }

            ret.push(suggestionsGroupDescriptor);

            var groupDescriptor = new AJS.GroupDescriptor({
                label: data.header
            });
            $(data.users).each(function(){
                groupDescriptor.addItem(new AJS.ItemDescriptor({
                    value: this.key, // value of item added to select
                    label: this.displayName, // title of lozenge
                    html: this.html,
                    icon: this.avatarUrl,
                    allowDuplicate: false,
                    highlighted: true
                }));
            });

            ret.push(groupDescriptor);
            return ret;
        },

        /**
         * Event handler for when Test Set filter option is changed
         */
        _onTestSetsFilterChanged: function(e) {
            var $selectField = this.$el.find('#testexec-tests-filter-testsets');
            var newSelectedValue = $selectField.find('option:selected').attr('value');
            if (this.selectedTestSetKey !== newSelectedValue) {
                this.selectedTestSetKey = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
            }
        },

        /**
         * Event handler for when Assignee filter option is changed
         */
        _onAssigneeFilterChanged: function(e) {
            var $selectField = this.$el.find('#testexec-tests-filter-assignee');
            var newSelectedValue = $selectField.find('option:selected').attr('value');
            if (this.lastAssigneeFilterValue !== newSelectedValue) {
                this.lastAssigneeFilterValue = newSelectedValue ? newSelectedValue : '';
                if(newSelectedValue === "-1" || newSelectedValue === "\u241Eunassigned\u241E"){
                	$("#testexec-tests-filter-assignee-single-select").removeClass("aui-ss-has-entity-icon");
                }
                // perform search
                this._applyFilters();
            }
        },

        /**
         * Event handler for when Status filter option is changed
         */
        _onStatusFilterChanged: function(e) {
            var $selectField = this.$el.find('#testexec-tests-filter-status');
            var newSelectedValue = $selectField.val();
            if (this.lastStatusFilterValue !== newSelectedValue) {
                this.lastStatusFilterValue = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
            }
        },

        _onComponentsFilterChanged : function(e) {
        	var $selectField = this.$el.find("#testexec-tests-filter-component");
        	var newSelectedValue = $selectField.val();
        	if (this.lastComponentsValue !== newSelectedValue ) {
                this.lastComponentsValue = newSelectedValue ? newSelectedValue : '';
                // perform search
                this._applyFilters();
        	}
        },

        _onItemRemoved : function(e) {
        	var newSelectedValue = $("#testexec-tests-filter-component").val();

        	if (this.lastComponentsValue !== newSelectedValue ) {
                this.lastComponentsValue = newSelectedValue ? newSelectedValue : '';
        	}

            var newSelectedStatusValue = $  ('#testexec-tests-filter-status').val();
            if (this.lastStatusFilterValue !== newSelectedStatusValue) {
                this.lastStatusFilterValue = newSelectedStatusValue ? newSelectedStatusValue : '';
            }

            this._applyFilters();
        },

        /**
         * Event handler for when text filter is modified
         */
        _onTextFilterKeyUp: function(e) {
            var $textField = this.$el.find('#testexec-tests-filter-contains');
            if (this.lastTextFilterValue !== $textField.val()) {
                this.lastTextFilterValue = $textField.val();
                // perform search
                this._applyFilters();
            }
        },

        _applyFilters: function() {
            var data = [];

            if (this.selectedTestSetKey && this.selectedTestSetKey != '-1') {
                data.push({ "name": "filter_test_set", "value": this.selectedTestSetKey});
            }
            if (this.lastAssigneeFilterValue && this.lastAssigneeFilterValue != '-1') {
                data.push({ "name": "filter_testrun_assignee", "value": this.lastAssigneeFilterValue});
            }
            if (this.lastStatusFilterValue) {
                data.push({ "name": "filter_testrun_status", "value": this.lastStatusFilterValue});
            }
            if (this.lastComponentsValue) {
                data.push({ "name": "filter_component", "value": this.lastComponentsValue});
            }
            if (this.lastTextFilterValue) {
                data.push({ "name": "filter_text", "value": this.lastTextFilterValue});
            }

            this.mainView.filter(data);
        },

        /**
         * Event handler for when clean filter button is clicked
         */
        _onCleanFilterClick: function(e) {
            var $textField = this.$el.find('#testexec-tests-filter-contains');

            $('#testexec-tests-filter-contains').addClass('disabled');
            $('#testexec-tests-filter-testsets').addClass('disabled');
            $('#testexec-tests-filter-assignee').addClass('disabled');
            $("#testexec-tests-filter-component").addClass('disabled');
            $('#testexec-tests-filter-status').addClass('disabled');

            $textField.val('');
            this.testSetSingleSelect.setSelection(this.allDefaultItem);
            this.assigneeSingleSelect.setSelection(this.allDefaultItem);
            this.statusMultiSelect.clearLozenges();
            this.componentsMultiSelect.clearLozenges();

            this.selectedTestSetKey = null;
            this.lastAssigneeFilterValue = null;
            this.lastStatusFilterValue = null;
            this.lastComponentsValue = null;
            this.lastTextFilterValue = null;

            $('#testexec-tests-filter-contains').removeClass('disabled');
            $('#testexec-tests-filter-testsets').removeClass('disabled');
            $('#testexec-tests-filter-assignee').removeClass('disabled');
            $("#testexec-tests-filter-component").removeClass('disabled');
            $('#testexec-tests-filter-status').removeClass('disabled');

            this.mainView.cleanFilters();

            $("#testexec-tests-filter-testsets-single-select").removeClass("aui-ss-has-entity-icon");
            $("#testexec-tests-filter-assignee-single-select").removeClass("aui-ss-has-entity-icon");

            return false;
        }
    });


})(AJS.$);
