(function($) {

    RAVEN.namespace("RAVEN.TestPlan.TestExec");

    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.TestPlan.TestExec.TestExecsTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testplan/' + this._getTestPlanKey() + '/testexec/columns';
        },

        defaults: function() {
            return [];
        },

        _getTestPlanKey: function () {
            return $('#raven-testplan-testexecs-issue-key').val();
        }
    });

    /**
     * This is the main view for the Test Executions table on a Test Plan issue
     */
    RAVEN.TestPlan.TestExec.TestExecsView = Backbone.View.extend({
        el: '#raven-plan-testexecs',

        /**
         * Flag that indicates whether this view has finished rendering
         */
        ready: false,

        /**
         * DataTable Instance
         */
        dataTable: null,

        /**
         * The Test Plan issue key
         */
        testPlanKey: null,

        /**
         * Permissions
         */
        canEdit: false,

        /**
         * Current dropdown trigger element
         */
        currentDropdownTrigger: null,

        /**
         * The Bulk view
         */
        bulkEditView: null,

        /**
         * The Test Execs toolbar view object
         */
        toolbarView: null,

        events: {

        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.testplan.testexec .raven-testplan-testexec-remove-command' : '_onTestExecDeleteClick',
            'click.raven.testplan.testexec .raven-bulk-testplan-testexec-remove-command' : '_onBulkTestExecDeleteClick',
            'click.raven.testplan.testexec .raven-dropdown-trigger': '_onDropDownTriggerClick',
            'aui-dropdown2-hide.raven.testplan.testexec .aui-dropdown2': '_onDownDownHide',
            'click.raven.testplan #raven-plan-add-testexecs-link': '_onAddTestExecsClick'
        },

        initialize: function() {
            var self = this;

            this.model = new RAVEN.TestPlan.TestExec.TestExecsTableMetaModel();

            this._delegateUnscopedEvents();

            AJS.messages.setup();

            this._clearMessage();

            this.testPlanKey = $('#raven-testplan-testexecs-issue-key').val();
            this.canEdit = $('#testexecs-can-edit').val() === 'true';

            self.$el.block();
            this.model.fetch({
                success: function() {
                    self.render();
                    self.$el.unblock();
                },
                error: function(model, response) {
                    self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.column-layout.error'), null, 'error');
                    self.$el.unblock();
                }
            });
        },

        render: function() {
            this._clearMessage();
            
            // Render table skeleton
            var $table = RAVEN.TestPlan.TestExecs.renderTestExecsTable({'columns': _.values(this.model.attributes)});
            var $tableWrapper = this.$el.find('#testplan-testexecs-table-wrapper');
            $tableWrapper.empty();
            $tableWrapper.html($table);

            // create the dataTable
            this._createTestsDataTable();


            // render bulk mode button
            if (this.canEdit === true && !this.bulkEditView) {
                this._renderBulkModeButton();
            }

            // render test runs toolbar view
            this._renderTestsToolbarView();

            $tableWrapper.show();

            this.$el.show();

            this.ready = true;

            return this;
        },

        destroy: function() {
            this._cleanView();

            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();

            this.$el.removeData().unbind();

            this.unbind();

            this.remove();
        },

        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.ready = false;
            this.model = null;
            this.bulkEditView = null;

            var $tableWrapper = this.$el.find('#testplan-testexecs-table-wrapper');
            $tableWrapper.empty();
        },

        /**
         * Redraws the datatable
         */
        refresh: function() {
            this._cleanView();
            this.initialize();
        },

        /**
         * Handles the creation of the datatable.
         */
        _createTestsDataTable: function() {
            var self = this;

            var path = contextPath + '/rest/raven/1.0/testplan/' + this.testPlanKey + "/testexecution";

            if(this.dataTable) {
                this.dataTable.fnDestroy();
            }

            this.dataTable = $('#testplan-testexecs-entries-table').xrayDataTable({
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('view-issue-testplan-testexecs-section.infoempty'),
                    "sEmptyTable": AJS.I18n.getText('view-issue-testplan-testexecs-section.emptytable'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch":  AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "sDom" : "l<t>ip",
                "bServerSide" : true,
                "sAjaxSource": path,
                "sAjaxDataProp": "entries",
                "bProcessing": false,
                "bAutoWidth": false,
                "sPaginationType": "full_numbers",
                "bSort": true,
                "aaSorting": [],
                "aoColumnDefs": this._createDataTableColumnDefs(),
                "fnServerData": function( sSource, aoData, fnCallback, oSettings){
                    self._fnServerData( sSource, aoData, fnCallback, oSettings );
                    if (self.bulkEditView) {
                        self.bulkEditView.setCheckBoxRowsSelectionValue(false);
                    }
                },
                "fnRowCallback": function( nRow, aData, iDisplayIndex, iDisplayIndexFull ) {
                    nRow.id = aData["key"];

                    return nRow;
                },
                "fnDrawCallback": function( oSettings ) {
                    // set initial overflow to allow overflow columns to be visible
                    self.$el.css("overflow", "initial");

                    // hack for drag&drop to work
                    $('#testplan-testexecs-entries-table').closest('div').addClass('datatable-drag-overflow');

                    self._toogleTestExecsTable();
                    if (self.bulkEditView) {
                        self.bulkEditView.showHideBulkActions();
                    }
                },
                "fnInitComplete": function(settings, json) {
                }
            });

            this._initSortableColumns();
        },

        /**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createDataTableColumnDefs: function() {
            var self = this;
            var columns = _.values(this.model.attributes);

            var bulkToogleColumnTitle = null;
            var bulkActionColumnTitle = null;
            if (self.canEdit === true) {
                bulkToogleColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkSelectAllInput();

                var bulkActions = RAVEN.TestPlan.TestExecs.testExecsBulkMenuActions({ 'canEdit' : self.canEdit});
                bulkActionColumnTitle = RAVEN.DataTable.BulkEdit.renderBulkActionTrigger({id: "testplan-testexec-bulk-action", actions: bulkActions});
            }

            var firstColumnDefs = [
                {
                    "mData": null,
                    "sTitle": bulkToogleColumnTitle,
                    "mDataProp": null,
                    "sWidth": "20px",
                    "sDefaultContent": "<input type='checkbox' ></input>",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        if (self.canEdit === true) {
                            return RAVEN.DataTable.BulkEdit.renderBulkSelectInput({id: row["key"]});
                        } else {
                            return '';
                        }
                    },
                    "aTargets": [ 0 ]
                },
                {
                    "mData": "key",
                    "sTitle": "Key",
                    "sWidth": "110px",
                    "bSortable": true,
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        return '<a href=' + contextPath + '/browse/' + row["key"] + ' class="link-title">' + data + '</a>';
                    },
                    "aTargets": [ 1 ]
                }
            ];

            var userColumnDefs = this._createDataTableUserColumn(columns);

            var lastColumnDefs = [
                {
                    "mData": "status",
                    "sTitle": "Status",
                    "sWidth": "110px",
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        var htmlProgressBar = '<ul class="testexec-progress-bar">';

                        if(data.count === 0) {
                            htmlProgressBar=htmlProgressBar+'<li class="pb-todo" style="width:100%;" title="'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+' 100%" data-status="TODO">'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+'</li>';
                        } else {
                            for (var i = 0; i < data.statuses.length; i++) {
                                var status = data.statuses[i];
                                htmlProgressBar = htmlProgressBar + '<li style="background-color:' + status.color + ';width:'+ status.statusPercent +'%;" title="' + status.name + ':' + status.statusCount + ' (' + status.statusPercent + '%)" data-status-id="' + status.id + '">' + status.name + '</li>';
                            }
                        }

                        htmlProgressBar=htmlProgressBar+'</ul>';

                        return htmlProgressBar;
                    },
                    "aTargets": [ 2 + columns.length ]
                },
                {
                    "mData": null,
                    "sTitle": bulkActionColumnTitle,
                    "bSortable": false,
                    "bSearchable": false,
                    "mRender": function ( data, type, row ) {
                        if (self.canEdit === true) {
                            return RAVEN.TestPlan.TestExecs.testExecsMenu({
                                key: row["key"]
                            });
                        }
                        return null;
                    },
                    "aTargets": [ 3 + columns.length]
                }
            ];

            return firstColumnDefs.concat(userColumnDefs).concat(lastColumnDefs);
        },

        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumn: function(columns) {
            return _.map(columns, function(column, index) {
                return {
                    "mData": 'userColumns.' + column.id,
                    "bSortable": column.sortable,
                    "bSearchable": false,
                    "aTargets": [ 2 + index ]
                };
            });
        },

        _addTestExecs: function(testExecKeys, assignee) {
            var self = this;

            $('#raven-plan-testexecs').block();
            var path = contextPath + '/rest/raven/1.0/testplan/' + this.testPlanKey + '/testexec';

            if (!assignee || assignee == "-1") {
                assignee = null;
            }

            var value = {
                "keys": testExecKeys,
                "assignee": assignee
            };
            $.ajax({
                type: "POST",
                url: path,
                processData: true,
                async: true,
                timeout: 60000, // sets timeout to 60 seconds
                data: JSON.stringify(value),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.add.error', self.testPlanKey), errorThrown);
                    $('#raven-plan-testexecs').unblock();
                },
                success: function(data, textStatus, jqXHR) {
                    if (data.errors && data.errors.length > 0) {
                        var errorMsg = '';
                        $.each($(data.errors), function(i, msg) {
                            errorMsg = errorMsg + msg;
                            if (i < data.errors.length) {
                                errorMsg = errorMsg + '<br/>';
                            }
                        });
                        self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.add.controlled.error', self.testPlanKey), errorMsg);
                    }

                    $('#raven-plan-tests').trigger('update-tests', data); //refresh tests (tests table)
                    $('#raven-plan-testexecs').unblock();
                },
                complete: function() {
                    self.refresh();
                    self._updateHistory();
                    self._updateDates();
                }
            });
        },

        message: function (title, message, type) {
            type = (type === undefined || type == "") ? "error" : type;

            if (!message && title) {
                message = title;
                title = null;
            }

            if (type === "error") {
                this._clearMessage();
                AJS.messages.error('#raven-plan-testexecs-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "warning") {
                this._clearMessage();
                AJS.messages.warning('#raven-plan-testexecs-message-bar', {
                    title : title,
                    body : message
                });
            }

            if (type === "success") {
                AJS.messages.success('#raven-plan-testexecs-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "info") {
                AJS.messages.info('#raven-plan-testexecs-message-bar', {
                    title: title,
                    body: message
                });
            }
        },

        _clearMessage: function() {
            $('#raven-plan-testexecs-message-bar').empty();
        },

        _fnServerData: function( sSource, aoData, fnCallback, oSettings ) {
            var self = this;

            if (this.filters) {
                aoData = aoData.concat(this.filters);
            }

            $('#raven-plan-testexecs').block();
            oSettings.jqXHR = $.ajax( {
                "dataType": 'json',
                "type": "GET",
                "url": sSource,
                "data": aoData,
                "error": function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.processingerror'), errorThrown, 'error');
                },
                "complete": function() {
                    $('#raven-plan-testexecs').unblock();
                },
                "success": fnCallback
            });
        },

        _updateHistory: function() {
            if ($('#changehistory-tabpanel').attr('class') == 'active') {
                var alt_token = $('input[name="atl_token"]').val();
                $.ajax({
                    type : "GET",
                    url : contextPath + '/browse/' + this.testPlanKey +"?page=com.atlassian.jira.plugin.system.issuetabpanels:changehistory-tabpanel&_pjax=true&_="+alt_token,
                    contentType : "text/plain",
                    success : function (data, status) {
                        var tab = $(data).find('#activitymodule div.mod-content #issue_actions_container');
                        var jiraTab = $('#activitymodule div.mod-content').find('#issue_actions_container');
                        $(jiraTab).html(tab);
                    }
                });
            }
        },

        _updateDates: function () {
            var updatedDate = null;

            if(AJS.$('#datesmodule #updated-date').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-date time.livestamp');
            } else if(AJS.$('#datesmodule #updated-val').length === 1) {
                updatedDate = AJS.$('#datesmodule #updated-val time.livestamp');
            } else {
                return null;
            }

            $.ajax({
                type : "GET",
                url : AJS.contextPath() + '/rest/raven/1.0/date/' + JIRA.Issue.getIssueKey() + "/updated",
                contentType : "application/json",
                success : function (data) {
                    var title = data.title;
                    var dateTimeIso8601 = data.dateTimeIso8601;
                    updatedDate.attr("datetime", dateTimeIso8601);
                    $.each(AJS.$('#datesmodule dd.date'), function() {
                        if($(this).find('#updated-date time.livestamp').length > 0 || $(this).find('#updated-val time.livestamp').length > 0) {
                            $(this).attr("title", title);
                        }
                    });
                    updatedDate.livestamp();
                }
            });
        },

        _toogleTestExecsTable: function() {
            if (this.dataTable && this.dataTable.fnRecordsTotal() > 0) {
                $('#raven-plan-testexecs .notestexecs').hide();
                $('#plan-testexecs-section').show();
            } else {
                $('#plan-testexecs-section').hide();
                $('#raven-plan-testexecs .notestexecs').show();
            }
        },

        _initSortableColumns: function() {
            var self = this;

            $('#testplan-testexecs-entries-table thead tr').sortable({
                items: 'th:not(.fixed)',
                placeholder: "dragging-placeholder",
                update: function (event, ui) {
                    var ancorNode = ui.item.prev();
                    var columnId = $(ui.item).data('column-id');
                    if (columnId) {
                        var anchorColumnId = null;
                        if (!$(ancorNode).hasClass('fixed')) {
                            anchorColumnId = $(ancorNode).data('column-id');
                        }

                        self.columnLayoutView.moveColumn(columnId, anchorColumnId);
                    }
                }
            });
        },

        _confirmationDeleteTest: function(el, yesFunction, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                content: function (callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('view-issue-testplan-tests-section.bulk.delete.confirm.title');
                        message = AJS.I18n.getText('view-issue-testplan-tests-section.bulk.delete.confirm.message', self.testPlanKey);
                    } else {
                        title = AJS.I18n.getText('view-issue-testplan-tests-section.delete.confirm.title', el);
                        message = AJS.I18n.getText('view-issue-testplan-tests-section.delete.confirm.message', self.testPlanKey);
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({title:title, message:message, severity:severity});
                    callback(innerHtml);
                },
                submitHandler: function (e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(el);
                    dialog.hide();
                },
                autoClose : true
            });
            dialog.show();
        },

        _renderBulkModeButton: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#bulk-edit-action').length > 0) {
                return;
            }

            this.bulkEditView = new RAVEN.DataTable.BulkEditView({
                el: $wrapper
            });
        },

        _renderTestsToolbarView: function() {
            var $wrapper = this.$el.find('.xrayDataTables_wrapper');
            // if filter already exists then return
            if ($wrapper.find('#toolbar-wrapper').length > 0) {
                return;
            }

            var toolbarHtml = RAVEN.TestPlan.TestExecs.renderTestExecsToolBar();
            $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.TestPlan.TestExec.TestExecsColumnLayoutView({
                mainView: this
            });
        },

        _confirmationDeleteTestExec: function(el, yesFunction, bulk) {
            var self = this;

            var dialog = new JIRA.FormDialog({
                content: function (callback) {
                    var title = null;
                    var message = null;
                    if (bulk === true) {
                        title = AJS.I18n.getText('view-issue-testplan-testexecs-section.bulk.delete.confirm.title');
                        message = AJS.I18n.getText('view-issue-testplan-testexecs-section.bulk.delete.confirm.message', self.testPlanKey);
                    } else {
                        title = AJS.I18n.getText('view-issue-testplan-testexecs-section.delete.confirm.title', el);
                        message = AJS.I18n.getText('view-issue-testplan-testexecs-section.delete.confirm.message', self.testPlanKey);
                    }
                    var severity = "info";
                    var innerHtml = RAVEN.Dialog.deleteConfirmationDialog({title:title, message:message, severity:severity});
                    callback(innerHtml);
                },
                submitHandler: function (e) {
                    e.preventDefault();
                    var proxyFunc = $.proxy(yesFunction, self);
                    proxyFunc(el);
                    dialog.hide();
                },
                autoClose : true
            });
            dialog.show();
        },

        _deleteTestExec: function(testExecKeys) {
            var self = this;

            $('#raven-plan-testexecs').block();

            //if we have records to delete
            if(testExecKeys && testExecKeys.length > 0) {
                //Build query param
                var testExecKeysParam = this._buildQueryParamsFromSelectedRows(testExecKeys);

                $.ajax({
                    type: "DELETE",
                    url: contextPath + '/rest/raven/1.0/testplan/' + self.testPlanKey + "/testexecution?" + testExecKeysParam + "" ,
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() != '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }
                        if (testExecKeys.length == 1) {
                            self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.delete.error', testExecKeys[0]), errorThrown);
                        } else {
                            self.message(AJS.I18n.getText('view-issue-testplan-testexecs-section.bulk.delete.error'), errorThrown);
                        }
                        $('#raven-plan-testexecs').unblock();
                    },
                    success: function(data, textStatus, jqXHR) {
                        $('#raven-plan-testexecs').unblock();
                        self._deleteTestExecsCallback(testExecKeys);
                    },
                    complete: function() {
                        self._updateHistory();
                        self._updateDates();
                    }
                });
            }
        },

        _deleteTestExecsCallback: function(testExecKeys, data) {
            var self = this;
            if (testExecKeys && testExecKeys.length > 0) {
                var allData = this.dataTable.fnGetData();
                if (allData.length >= 1) {
                    _.each(_.initial(testExecKeys), function(testExecKey) {
                        self.dataTable.fnDeleteRow(testExecKey, null, false);
                    });
                    this.dataTable.fnDeleteRow(_.last(testExecKeys));
                } else {
                    // don't refresh the table if the last element is removed because it will request more data from server
                    this.dataTable.fnDeleteRow(_.last(testExecKeys), null, false);
                }
            }

        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.testplan.testexec');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.testplan.testexec');
        },

        /**
         * Delete test execution click handler
         */
        _onTestExecDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            var testExecKey = $(e.target).closest('a').data('key');
            this._confirmationDeleteTestExec([ testExecKey ], this._deleteTestExec);
        },

        /**
         * Delete Bulk test exec click handler
         */
        _onBulkTestExecDeleteClick: function(e) {
            e.stopPropagation();
            e.preventDefault();

            this._clearMessage();

            if(this.bulkEditView.getSelectedRowIds().length > 0) {
                this._confirmationDeleteTestExec(this.bulkEditView.getSelectedRowIds(), this._deleteTestExec, true);
            }
        },

        /**
         * Build query param
         */
        _buildQueryParamsFromSelectedRows: function(selectedIds) {
            var testExecKeys = "";
            var i;
            for (i = 0; i < selectedIds.length; ++i) {
                if(testExecKeys.length > 0){
                    testExecKeys+="&";
                }
                testExecKeys += "testExecKeys=" + selectedIds[i];
            }

            return testExecKeys;
        },

        /**
         * Save current dropdown trigger
         */
        _onDropDownTriggerClick: function(e) {
            this.currentDropdownTrigger = $(e.target);
            this.currentDropdownTrigger.closest('tr').addClass('active');
        },

        /**
         * Clear current dropdown trigger
         */
        _onDownDownHide: function() {
            if (this.currentDropdownTrigger) {
                this.currentDropdownTrigger.closest('tr').removeClass('active');
            }
        },

        /**
         * Add test sets button click handler
         */
        _onAddTestExecsClick: function() {
            var self = this;

            this._clearMessage();

            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.testexec.label');
            labels.title = AJS.I18n.getText('dialogs.issueselector.testexec.title', this.testPlanKey);
            labels.description = AJS.I18n.getText('dialogs.issueselector.testexec.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.testexec.enter');

            var issueTypeId = [$('#testexec-issue-type-id').val(), $('#substask-testexec-issue-type-id').val()];
            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog(
                'TESTEXECUTION',
                'key not in testPlanTestExecutions(' + this.testPlanKey + ')',
                issueTypeId,
                this.testPlanKey,
                labels,
                function (issues, assignee) {
                    if (issues && issues.length > 0) {
                        self._addTestExecs(issues, assignee);
                    }
                },
                null,
                self.canAssign
            );
        },
    });

    RAVEN.TestPlan.TestExec.TestExecsView.REFRESH_VIEW = "refreshTestExecsView";

})(AJS.$);
