require(['aui/inline-dialog2']);

(function($) {

    var OVERALL = "overall";
    var HISTORIC = "daily";

    var OK_STATUS = 'OK';
    var NOK_STATUS = 'NOK';
    var NOTRUN_STATUS = 'Not Run';
    var UNKNOWN_STATUS = 'Unknown';
    var UNCOVERED_STATUS = 'Uncovered';

    var FILTERS_MODE = 1;
    var SAVED_FILTER_MODE = 2;

    var NO_ANALISYS_VERSION = 0;

    var overallChart = null;
    var absHistoricChart = null;
    var relHistoricChart = null;
    var requirementsDataTable = null;

    var dialog = null;

    var chartDetailsData = null;

    var contentAdded = false;

    var execScope = null;
    var DEFAULT_EXEC_SCOPE = "version";

    var currentReport = OVERALL;
    var filterValues = {
        "savedFilter": null,
        "fixVersion": null,
        "includePreviousVersions": null,
        "status": null,
        "resolution": null,
        "assignee": null,
        "contains": null
    }

    var reportFieldPicker = null;
    var execScopeFieldPicker = null;
    var versionPicker = null;
    var testPlanPicker = null;
    var environmentPicker = null;
    var groupingFieldPicker = null;


    var fixVersionFieldPicker = null;
    var statusFieldPicker = null;
    var resolutionFieldPicker = null;
    var assigneeFieldPicker = null;
    var componentFieldPicker = null;

    var graphContentRatio = 0.75;

    function message(title, message, type) {
        clearMessage();
        if (!type) {
            type = 'error';
        }
        AJS.messages[type]('#raven-requirement-coverage-message-bar', {
            title: title,
            body: message
        });
    }

    function clearMessage() {
        $('#raven-requirement-coverage-message-bar').empty();
    }

    function getStatusFromName(statusName) {
        if (statusName) {
            if (statusName === OK_STATUS) {
                return 'OK';
            } else if (statusName === NOK_STATUS) {
                return 'NOK';
            } else if (statusName === NOTRUN_STATUS) {
                return 'NOTRUN';
            } else if (statusName === UNKNOWN_STATUS) {
                return 'UNKNOWN';
            } else if (statusName === UNCOVERED_STATUS) {
                return 'UNCOVERED';
            }
        }
        return null;
    }

    function createCompletenessBar(completeness) {
        return '<ul class="completeness-bar" title="' + completeness + '%"><li style="width:' + completeness + '%;"><span>' + completeness + '%</span></li><li></li></ul>';
    }

    function getProjectKey() {
        return $('#raven-project-key').val();
    }

    function getDefaultExecScope() {
        var value = $('#raven-exec-scope').val();
        if (!value) {
            return DEFAULT_EXEC_SCOPE;
        } else {
            return value;
        }
    }

    function includePreviousVersions() {
        return $('#include-previous-versions-option').is(':checked');
    }

    function flattenRequirements() {
        return $('#flat-requirements-option').hasClass('checked');
    }

    function fetchOverallCoverageChartData(projectKey, execScope, version, testPlan, environment, groupingField, filters) {
        $('#raven-requirement-coverage-wrapper').block();
        $('#raven-requirement-coverage-content-panel').spin("large");

        var params = '?execScope=' + execScope + '&project=' + projectKey + '&withDetails=true';
        if (version && version != '-1') {
            params = params + '&version=' + version;
        }
        if (testPlan) {
            params = params + '&testPlan=' + testPlan;
        }
        if (environment && environment != '-1') {
            params = params + '&environment=' + environment;
        }
        params = params + '&flattenRequirements=' + flattenRequirements() + '&groupingField=' + groupingField + filters;

        $.ajax({
            type: "GET",
            url: contextPath + '/rest/raven/1.0/chart/requirement/coverage' + params,
            async: true,
            timeout: 120000,
            contentType: "application/json",
            error: function(qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                message(AJS.I18n.getText('projectpanels.requirement-coverage.error.server'), errorThrown)
            },
            success: function(data, textStatus, jqXHR) {
                createOverallCoverageChart(data);
            },
            complete: function() {
                $('#raven-requirement-coverage-wrapper').unblock();
                $('#raven-requirement-coverage-content-panel').spinStop();
            }
        });
    }

    function fetchHistoricChartData(projectKey, execScope, version, testPlan, environment, groupingField, filters, from, to) {
        $('#raven-requirement-coverage-wrapper').block();
        $('#raven-requirement-coverage-content-panel').spin("large");

        var params = '?execScope=' + execScope + '&project=' + projectKey;
        if (version && version != '-1') {
            params = params + '&version=' + version;
        }
        if (testPlan) {
            params = params + '&testPlan=' + testPlan;
        }
        if (environment && environment != '-1') {
            params = params + '&environment=' + environment;
        }

        params = params + '&flattenRequirements=' + flattenRequirements() + '&groupingField=' + groupingField + '&fromDate=' + from + '&toDate=' + to + filters;

        $.ajax({
            type: "GET",
            url: contextPath + '/rest/raven/1.0/chart/requirement/historic' + params,
            async: true,
            timeout: 120000,
            contentType: "application/json",
            error: function(qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                message(AJS.I18n.getText('projectpanels.requirement-coverage.error.server'), errorThrown)
            },
            success: function(data, textStatus, jqXHR) {
                createHistoricCoverageCharts(data);
            },
            complete: function() {
                $('#raven-requirement-coverage-wrapper').unblock();
                $('#raven-requirement-coverage-content-panel').spinStop();
            }
        });
    }

    function refreshCoverageCharts() {
        var groupingField = $('#raven-grouping-field-select').val();
        if (groupingField === -1) {
            return;
        }

        var selectedEnvItem = environmentPicker.getSelectedItem();
        var environment = null;

        if (selectedEnvItem) {
            environment = selectedEnvItem.id;
        }

        if (execScope === 'version') {
            var selectedItem = versionPicker.getSelectedItem();
            var version = null;

            if (selectedItem) {
                version = selectedItem.id;
            }

            if (version == NO_ANALISYS_VERSION) {
                version = null;
            }

            if (currentReport === OVERALL) {
                loadOverallCoverageChart(execScope, version, null, environment, groupingField);
            } else {
                var from = $('#historic-chart-from').val();
                var to = $('#historic-chart-to').val();
                var valid = isHistoricChartPeriodValid();
                if (valid === true) {
                    loadHistoricCoverageChart(execScope, version, null, environment, groupingField, from, to);
                }
            }
        } else if (execScope === 'testPlan') {
            var testPlan = $('#raven-analysis-test-plan-select').val();

            if (testPlan) {
                if (currentReport === OVERALL) {
                    loadOverallCoverageChart(execScope, null, testPlan, environment, groupingField);
                } else {
                    var from = $('#historic-chart-from').val();
                    var to = $('#historic-chart-to').val();
                    var valid = isHistoricChartPeriodValid();
                    if (valid === true) {
                        loadHistoricCoverageChart(execScope, null, testPlan, environment, groupingField, from, to);
                    }
                }
            }
        }
    }

    function createFilterParams() {
        var filters = "";

        if ($('.filter-title.selected').data('mode') == SAVED_FILTER_MODE) {
            var savedFilter = filterValues.savedFilter;
            if (savedFilter) {
                filters = filters + "&savedFilter=" + savedFilter;
            }
        } else {
            var fixVersion = filterValues.fixVersion;
            if (fixVersion && fixVersion != -1) {
                filters = filters + "&fixVersion=" + fixVersion;
                filters = filters + "&previousVersions=" + filterValues.includePreviousVersions;
            }
            var status = filterValues.status;
            if (status) {
                filters = filters + "&status=" + status;
            }
            var resolution = filterValues.resolution;
            if (resolution && resolution != -1) {
                filters = filters + "&resolution=" + resolution;
            }
            var assignee = filterValues.assignee;
            if (assignee && assignee != -1) {
                filters = filters + "&assignee=" + assignee;
            }
            var contains = filterValues.contains;
            if (contains) {
                filters = filters + "&contains=" + contains;
            }

            var components = filterValues.components;
            if(components) {
            	filters = filters + "&components=" + components;
            }
        }

        return filters;
    }

    function loadOverallCoverageChart(execScope, version, testPlan, environment, groupingField) {
        disposeOverallCoverageChart();

        var filters = createFilterParams();
        fetchOverallCoverageChartData(getProjectKey(), execScope, version, testPlan, environment, groupingField, filters);
    }

    function disposeOverallCoverageChart() {
        if (overallChart) {
            // don't know how to correclty dispose the chart.
            // Current solution is to remove the element and append it again to clear CCC state which caused popup errors.
            overallChart = null;
            $('#overall-coverage-chart').remove();
            $('#overall-coverage-chart-section').prepend('<div id="overall-coverage-chart"></div>');
        }

        hideOverallChartDetailsDataTable();
    }

    function loadHistoricCoverageChart(execScope, version, testPlan, environment, groupingField, from, to) {
        disposeHistoricCoverageChart();

        var filters = createFilterParams();
        fetchHistoricChartData(getProjectKey(), execScope, version, testPlan, environment, groupingField, filters, from, to);
    }

    function disposeHistoricCoverageChart() {
        if (absHistoricChart || relHistoricChart) {
            // don't know how to correclty dispose the chart.
            // Current solution is to remove the element and append it again to clear CCC state which caused popup errors.
            absHistoricChart = null;
            relHistoricChart = null;
            $('#historic-coverage-chart-absolute').remove();
            $('#historic-coverage-chart-relative').remove();
            $('#historic-coverage-chart-section')
                .append('<div id="historic-coverage-chart-absolute"></div>')
                .append('<div id="historic-coverage-chart-relative"></div>');
        }
    }

    function createOverallCoverageChart(data) {
        if (data && data.resultset && data.resultset.length > 0) {
            chartDetailsData = data.details;

            var height = 0;

            // The size of the gadget will be calculated according to the number of rows it has
            if (data.rows && data.rows > 0) {
                height = 40 * data.rows;
            }

            if (height < 300) {
                height = 300;
            } else if (height > 1000) {
                height = 1000;
            }

            clearMessage();

            overallChart = new pvc.NormalizedBarChart({
                canvas: "overall-coverage-chart",
                width: calculateGraphicWidth(),
                height: height,
                orientation: 'horizontal',
                stacked: true,
                animate: true,
                legend: true,
                interactive: true,
                selectable: false,
                clickable: true,
                hoverable: true,
                legendPosition: 'top',
                legendAlign: 'right',
                legendFont: '12px sans-serif',
                valuesFont: 'bold 12px sans-serif',
                orthoAxisLabelSpacingMin: 6,
                valuesVisible: true,
                valuesOptimizeLegibility: true,
                colors: ['#95C160', '#D45D52', '#F1E069', '#9AA1A8', '#3671A3'],
                clickAction: overallChartDatumClickAction,
                selectionChangedAction: overallSelectionChanged,
                extensionPoints: {
                    axisLabel_font: '12px sans-serif',
                    barLabel_text: function() {
                        // don't show percentages less than 8% because the label is too big...
                        var percent = this.scene.vars.value.percent.value;
                        if (percent > 0.07) {
                            return this.scene.vars.value.percent.label + ' (' + this.scene.vars.value.value + ')';
                        } else {
                            return null;
                        }
                    }
                }
            });
            overallChart.setData(data, {
                crosstabMode: false
            });
            overallChart.render();

        } else {
            message(AJS.I18n.getText('projectpanels.requirement-coverage.resultset.empty'), null, 'info')
        }
    }

    function createHistoricCoverageCharts(data) {
        clearMessage();

        if (data && data.resultset && data.resultset.length > 0) {
            absHistoricChart = createHistoricCoverageChart(data, 'absolute');
            relHistoricChart = createHistoricCoverageChart(data, 'relative');
        } else {
            message(AJS.I18n.getText('projectpanels.requirement-coverage.historic.resultset.empty'), null, 'info')
        }
    }

    function createHistoricCoverageChart(data, type) {
        var title = type === 'absolute' ?
            AJS.I18n.getText('projectpanels.requirement-coverage.historic.absolute.title') :
            AJS.I18n.getText('projectpanels.requirement-coverage.historic.relative.title')
        var options = {
            canvas: "historic-coverage-chart-" + type,
            width: calculateGraphicWidth(),
            height: 400,
            title: title,
            titlePosition: 'left',
            stacked: true,
            animate: true,
            legend: true,
            selectable: false,
            clickable: true,
            hoverable: true,
            legendPosition: 'right',
            legendAlign: 'left',
            legendFont: '12px sans-serif',
            valuesFont: 'bold 12px sans-serif',
            baseAxisTicks: true,
            valuesVisible: true,
            valuesOptimizeLegibility: true,
            ignoreNulls: false,

            colors: createColors(data.dimensions),

            extensionPoints: {
                xAxisLabel_textAngle: -Math.PI / 3,
                xAxisLabel_textAlign: 'right',
                xAxisLabel_textBaseline: 'top',
                barLabel_text: function() {
                    // don't show percentages less than 5% because the label is too big...
                    var categoriesCount = this.chart.data.getVisibleCategories().length;
                    if (categoriesCount < 7) {
                        var percent = this.scene.vars.value.percent.value;
                        if (percent > 0.05) {
                            if (type === 'absolute') {
                                return this.scene.vars.value.value;
                            } else {
                                return this.scene.vars.value.percent.label;
                            }
                        }
                    }
                    return null;
                }
            }
        };

        var historicChart;
        if (type === 'relative') {
            historicChart = new pvc.NormalizedBarChart(options);
        } else {
            historicChart = new pvc.BarChart(options);
        }

        historicChart.setData(data, {
            crosstabMode: false
        });
        historicChart.render();

        return historicChart;
    }

    function createColors(dimensions) {
        if (dimensions === 3) {
            return [
                '#A4BFD6', '#3671A3', '#1F426C', // Uncovered
                '#F9F1BB', '#F1E069', '#CAB63D', // Not run
                '#D1D5D8', '#9AA1A8', '#616B73', // Unknown
                '#ECB6B1', '#D45D52', '#A73730', // NOK
                '#CFE3B7', '#95C160', '#5C9038' // OK
            ];
        } else if (dimensions === 2) {
            return [
                '#A4BFD6', '#3671A3', // Uncovered
                '#F9F1BB', '#F1E069', // Not run
                '#D1D5D8', '#9AA1A8', // Unknown
                '#ECB6B1', '#D45D52', // NOK
                '#CFE3B7', '#95C160' // OK
            ];
        } else if (dimensions === 1) {
            return [
                '#A4BFD6', // Uncovered
                '#F9F1BB', // Not run
                '#D1D5D8', // Unknown
                '#ECB6B1', // NOK
                '#CFE3B7', // OK
            ];
        } else {
            return [];
        }
    }

    function overallChartDatumClickAction(scene) {
        // Replace any selected datums with the datums participating in the scene

        if (scene.datums().first().isSelected) {
            this.chart.data.replaceSelected([]);
        } else {
            this.chart.data.replaceSelected(scene.datums());
        }

        // Re-renders if any changed
        this.chart.updateSelections();
    }

    function overallSelectionChanged(selectedDatums, selectedChangedDatums) {
        if (selectedDatums && selectedDatums.length > 0) {
            var datum = selectedDatums[0];
            var group = datum.atoms.category.value;
            var status = datum.atoms.series.value;
            showOverallChartDetailsDataTable(group, status);
        } else {
            hideOverallChartDetailsDataTable();
        }
    }

    function navigateToOverallChart() {
        clearMessage();
        currentReport = OVERALL;
        $('#historic-coverage-chart-section').hide();
        $('#overall-coverage-chart-section').show();

        disposeHistoricCoverageChart();

        refreshCoverageCharts();
    }

    function navigateToHistoricChart() {
        clearMessage();
        currentReport = HISTORIC;
        $('#overall-coverage-chart-section').hide();
        $('#historic-coverage-chart-section').show();

        setHistoricPeriodFieldPickerPosition();

        disposeOverallCoverageChart();

        refreshCoverageCharts();
    }

    function showOverallChartDetailsDataTable(group, status) {
        // set details section width
        $('#overall-coverage-chart-details').css({ "max-width": calculateGraphicWidth() });

        // dispose previous datatable
        disposeOverallCoverageChartDetails();

        var version = $('#raven-analysis-version-select').val();
        if (version == NO_ANALISYS_VERSION) {
            version = null;
        }
        var groupingField = $('#raven-grouping-field-select').val();
        if (version != -1 && groupingField != -1) {
            $('#overall-coverage-chart-details').show();
            var statusCode = getStatusFromName(status);

            // create header
            var groupingName = $('#raven-grouping-field-select').find(":selected").text();

            $('#overall-coverage-chart-details-title')
                .append('<h3 class="requirements-title">' + AJS.I18n.getText('common.label.requirements-with') + ' ' + groupingName + ' "' + group + '" ' + AJS.I18n.getText('common.label.and') + ' status "' + status + '"</h3>')
                .append('<p><a id="view-issues-link" style="display: none;" href="">' + AJS.I18n.getText('common.label.view-issues') + '</a></p>');

            var filters = createFilterParams();

            createOverallChartDetailsDataTable(getProjectKey(), version, groupingField, group, statusCode, filters);
        }
    }

    function createViewIssuesLink(jql) {
        var jql = encodeURIComponent(jql);
        return contextPath + '/secure/IssueNavigator.jspa?reset=true&mode=hide&jqlQuery=' + jql;
    }

    function hideOverallChartDetailsDataTable() {
        // dispose previous datatable
        disposeOverallCoverageChartDetails();

        $('#overall-coverage-chart-details').hide();
    }

    function createOverallChartDetailsDataTable(projectKey, version, groupingField, group, status, filters) {
        if (!chartDetailsData) {
            message(AJS.I18n.getText('projectpanels.requirement-coverage.details.error.server'));
            return;
        }

        var data = _.find(chartDetailsData, function(details) {
            return details.group.name === group && details.status.toUpperCase() === status.toUpperCase();
        });

        if (!data) {
            message(AJS.I18n.getText('projectpanels.requirement-coverage.details.error.server'));
            return;
        }

        if (data.jqlQuery) {
            var viewIssuesLink = createViewIssuesLink(data.jqlQuery);
            $('#view-issues-link').attr('href', viewIssuesLink).show();
        }

        requirementsDataTable = $('#requirements-datatable').xrayDataTable({
            "bServerSide": false,
            "bProcessing": false,
            "bAutoWidth": false,
            "iDisplayLength": 10,
            "sPaginationType": "full_numbers",
            "bSort": true,
            "oLanguage": {
                "oPaginate": {
                    "sFirst": AJS.I18n.getText('common.label.first'),
                    "sLast": AJS.I18n.getText('common.label.last'),
                    "sNext": AJS.I18n.getText('common.label.next'),
                    "sPrevious": AJS.I18n.getText('common.label.previous')
                },
                "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                "sEmptyTable": AJS.I18n.getText('projectpanels.requirement-coverage.details.empty'),
                "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                "sSearch": AJS.I18n.getText('common.table.label.search'),
                "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
            },
            "aaSorting": [],
            "aoColumns": [{
                "mData": "subRequirements"
            }, {
                "mData": "key"
            }, {
                "mData": "summary"
            }, {
                "mData": "total"
            }, {
                "mData": "ok"
            }, {
                "mData": "nok"
            }, {
                "mData": "notRun"
            }, {
                "mData": "completeness"
            }],
            "aoColumnDefs": [{
                "bSearchable": false,
                "bSortable": false,
                "mRender": function(data, type, row) {
                    if (data && data.length > 0) {
                        return '<span class="toogle-sub-requirements expand-icon disabled"></span>';
                    } else {
                        return '';
                    }
                },
                "aTargets": [0]
            }, {
                "mRender": function(data, type, row) {
                    return '<a href="' + contextPath + '/browse/' + row["key"] + '" class="link-title">' + $('<span></span>').text(data).html() + '</a>';
                },
                "aTargets": [1, 2]
            }, {
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<span class="tests-value">' + data + '</span>';
                },
                "aTargets": [3, 4, 5, 6]
            }, {
                "mRender": function(data, type, row) {
                    return createCompletenessBar(data);
                },
                "aTargets": [7]
            }],
            "aaData": data.requirements
        });
    }

    function createOverallChartDetailsDataTableFromServer(projectKey, version, resolution, groupingField, group, status, filters) {
        var path = contextPath + '/rest/raven/1.0/chart/requirement/coverage/details';

        path += "?project=" + projectKey + "&group=" + group + "&requirementStatus=" + status;

        if (version) {
            path += '&version=' + version;
        }

        if (resolution) {
            path += '&resolution=' + version;
        }

        path += '&flattenRequirements=' + flattenRequirements() + '&groupingField=' + groupingField + filters;

        requirementsDataTable = $('#requirements-datatable').xrayDataTable({
            "bServerSide": true,
            "sAjaxSource": path,
            "sAjaxDataProp": "entries",
            "bProcessing": false,
            "bAutoWidth": false,
            "iDisplayLength": 10,
            "sPaginationType": "full_numbers",
            "bSort": true,
            "oLanguage": {
                "oPaginate": {
                    "sFirst": AJS.I18n.getText('common.label.first'),
                    "sLast": AJS.I18n.getText('common.label.last'),
                    "sNext": AJS.I18n.getText('common.label.next'),
                    "sPrevious": AJS.I18n.getText('common.label.previous')
                },
                "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                "sEmptyTable": AJS.I18n.getText('projectpanels.requirement-coverage.details.empty'),
                "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                "sSearch": AJS.I18n.getText('common.table.label.search'),
                "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
            },
            "aaSorting": [],
            "aoColumns": [{
                "mData": "subRequirements"
            }, {
                "mData": "key"
            }, {
                "mData": "summary"
            }, {
                "mData": "total"
            }, {
                "mData": "ok"
            }, {
                "mData": "nok"
            }, {
                "mData": "notRun"
            }, {
                "mData": "completeness"
            }],
            "aoColumnDefs": [{
                "bSearchable": false,
                "bSortable": false,
                "mRender": function(data, type, row) {
                    if (data && data.length > 0) {
                        return '<span class="toogle-sub-requirements expand-icon disabled"></span>';
                    } else {
                        return '';
                    }
                },
                "aTargets": [0]
            }, {
                "mRender": function(data, type, row) {
                    return '<a href="' + contextPath + '/browse/' + row["key"] + '" class="link-title">' + data + '</a>';
                },
                "aTargets": [1, 2]
            }, {
                "bSearchable": false,
                "mRender": function(data, type, row) {
                    return '<span class="tests-value">' + data + '</span>';
                },
                "aTargets": [3, 4, 5, 6]
            }, {
                "mRender": function(data, type, row) {
                    return createCompletenessBar(data);
                },
                "aTargets": [7]
            }],
            "fnServerData": function(sSource, aoData, fnCallback, oSettings) {
                return fnServerData(sSource, aoData, fnCallback, oSettings, group, status);
            }
        });
    }

    function toogleExpandRow(trigger) {
        var row = $(trigger).parents('tr')[0];
        if (row) {
            if (requirementsDataTable.fnIsOpen(row)) {
                // This row is already open - close it
                $(trigger).removeClass('enabled').addClass('disabled');
                requirementsDataTable.fnClose(row);
            } else {
                // Open this row
                $(trigger).removeClass('disabled').addClass('enabled');
                requirementsDataTable.fnOpen(row, fillSubRequirementsDetails(row), 'details');
            }
        }
    }

    function fillSubRequirementsDetails(row) {
        var data = requirementsDataTable.fnGetData(row);
        if (data && data.subRequirements && data.subRequirements.length > 0) {
            var table = $('<table class="detail aui data-table">\
<colgroup>\
<col span="1" style="width: 40px;">\
<col span="1" style="width: 120px;">\
<col span="1">\
<col span="1" style="width: 80px;">\
<col span="1" style="width: 80px">\
<col span="1" style="width: 80px;">\
<col span="1" style="width: 100px;">\
<col span="1" style="width: 130px;">\
</colgroup>\
<tbody>\
</tbody>\
</table>');
            $.each(data.subRequirements, function() {
                table.append(renderSubRequirementRow(this));
            });

            return table.prop('outerHTML');
        }
        return '';
    }

    function renderSubRequirementRow(req) {
        var row = '<tr>\
<td></td>\
<td><a href="' + contextPath + '/browse/' + req.key + '" style="padding-left:10px;" class="link-title" >' + req.key + '</a></td>\
<td><a href="' + contextPath + '/browse/' + req.key + '" class="link-title">' + req.summary + '</a></td>\
<td><span class="tests-value">' + req.total + '</span></td>\
<td><span class="tests-value">' + req.ok + '</span></td>\
<td><span class="tests-value">' + req.nok + '</span></td>\
<td><span class="tests-value">' + req.notRun + '</span></td>\
<td>' + createCompletenessBar(req.completeness) + '</td>\
</tr>';
        return row;
    }

    function disposeOverallCoverageChartDetails() {
        if (requirementsDataTable) {
            requirementsDataTable.fnClearTable(false);
            requirementsDataTable.fnDestroy();
            requirementsDataTable = null;
        }

        $('#overall-coverage-chart-details-title').empty();
    }

    function fnServerData(sSource, aoData, fnCallback, oSettings, group, status) {
        $('#overall-coverage-chart-details').block();
        oSettings.jqXHR = $.ajax({
            "dataType": 'json',
            "type": "GET",
            "timeout": 120000,
            "url": sSource,
            "data": aoData,
            "error": function(qXHR, textStatus, errorThrown) {
                if (errorThrown && errorThrown.trim() != '') {
                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                } else {
                    errorThrown = qXHR.responseText;
                }
                message(AJS.I18n.getText('projectpanels.requirement-coverage.details.error.server'), errorThrown)
            },
            "complete": function() {
                $('#overall-coverage-chart-details').unblock();
            },
            "success": function(data, textStatus, jqXHR) {
                if (data.meta) {
                    var viewIssuesLink = createViewIssuesLink(data.meta);
                    $('#view-issues-link').attr('href', viewIssuesLink).show();
                }
                fnCallback(data, textStatus, jqXHR);
            }
        });
    }

    function onHistoricChartPeriodUpdate(e) {
        var from = $('#historic-chart-from').val();
        var to = $('#historic-chart-to').val();
        var valid = isHistoricChartPeriodValid();
        if (valid === true && $(e.activeDiv).hasClass("day")) {
            refreshCoverageCharts();
        }
    }

    function isHistoricChartPeriodValid(from, to) {
        return true;
    }

    function createSubtleSelect($selectField) {
        return new RAVEN.SubtleSelect({
            element: $selectField,
            showDropdownButton: false,
            revertOnInvalid: true,
        });
    }

    function createSubtleSingleSelect($selectField) {
        var items = _.map($selectField.find('option'), function(item) {
            return {
                'id' : item.value,
                'name' : item.text,
            };
        });

        var selectedItem = _.first(_.map($selectField.find('option:selected'), function(item) {
            return {
                'id' : item.value,
                'name' : item.text,
            };
        }));

        var subtlePicker = new RAVEN.Field.SubtleSingleSelect(items, selectedItem, $selectField.closest('#raven-subtle-container'));
        subtlePicker.render();

        // remove select element. This is only for retrieving the initial items.
        $selectField.remove();

        return subtlePicker;
    }

    function createSingleSelectPicker($selectField) {
        $selectField.on('initialized', function(e, field) {
            var version = $('body').data('version');
            if (!version || version.toString().charAt(0) === '5') {
                $('input', $selectField.parent()).height('23px');
            }
        });

        return new AJS.SingleSelect({
            element: $selectField,
            showDropdownButton: true,
            revertOnInvalid: true,
        });
    }

    function createStatusPicker($selectField) {
        return new AJS.MultiSelect({
            element: $selectField,
            showDropdownButton: true,
            revertOnInvalid: true,
        });
    }

    function createComponentPicker($selectField) {

    	return new AJS.MultiSelect({
			element: $selectField,
			showDropdownButton: true,
			submitInputVal: true,
			noQueryNoRequest: false,
			revertOnInvalid: true,
			loadOnInit: true,
			ajaxOptions: {
				query: true,
				url: contextPath + '/rest/api/2/project/' + getProjectKey() + '/components',
				formatResponse: function (data) {
					var optgroup = new AJS.GroupDescriptor({
						weight: 0,
						id: "all-filter-opt-group",
						replace: true
					});

					$(data).each(function(i, status) {
						this.label = this.name;
						var item = new AJS.ItemDescriptor({
							value: this.name,
							label: this.name,
							title: this.description,
							highlighted: true,
						});
						optgroup.addItem(item);
					});

					return [optgroup];
				}
			}
		});
    }

    function initSelectPickers() {
        // Parameter pickers
        execScopeFieldPicker = renderExecScopePicker();

        versionPicker = createSubtleSingleSelect($('#raven-analysis-version-select'));
        $(versionPicker).on('selectedItemChanged', function(e) {
            refreshCoverageCharts();
        });

        /*
        $(document).on('change.version.raven', '#raven-analysis-version-select', function(e) {
            $('#raven-analysis-version-select-single-select input').removeClass('option-hint');
            $('#raven-analysis-version-select option[value="-1"]').remove();
            refreshCoverageCharts();
        });
        */

        testPlanPicker = createTestPlanSingleSelectPicker($('#raven-analysis-test-plan-select'));
        $("#raven-analysis-test-plan-select-field").attr("placeholder", AJS.I18n.getText("view-issue-requirement-test-coverage-section.placeholer.strategy.test-plan"));
/*        $(document).on('change.test-plan.raven', '#raven-analysis-test-plan-select', function(e) {
            refreshCoverageCharts();
        });
        */
        $('#raven-analysis-test-plan-select').on("change", function(e, field) {
           refreshCoverageCharts();
        });

        environmentPicker = createSubtleSingleSelect($('#raven-analysis-environment-select'));
        $(environmentPicker).on('selectedItemChanged', function(e) {
            refreshCoverageCharts();
        });

        groupingFieldPicker = createSubtleSelect($('#raven-grouping-field-select'));
        $(document).on('change.grouping-field.raven', '#raven-grouping-field-select-select', function(e) {
            $('#raven-grouping-field-select-single-select input').removeClass('option-hint');
            $('#raven-grouping-field-select option[value="-1"]').remove();
            refreshCoverageCharts();
        });

        execScope = getDefaultExecScope();
        toogleExecScopeValuePicker(false);

        // Filter pickers
        fixVersionFieldPicker = createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-fixversion'));
        statusFieldPicker = createStatusPicker($('#raven-filter-inline-dialog #raven-requirement-filter-status'));
        resolutionFieldPicker = createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-resolution'));
        assigneeFieldPicker = initAssigneeFilterPicker();
        componentFieldPicker = createComponentPicker($('#raven-requirement-filter-component'));
    }

    function renderExecScopePicker() {
        if($("#exec-scope-picker-field").html() === null) {
            var $element = $('#raven-requirement-coverage-parameters').find("#exec-scope-picker");
            $element.on("change", function(e, field){
                execScope = $('#raven-requirement-coverage-parameters').find("#exec-scope-picker").val()[0];

                toogleExecScopeValuePicker(true);
            });

            return new RAVEN.SubtleSelect({
                element: $element,
                showDropdownButton: false,
                revertOnInvalid: true,
            });
        }

        return null;
    }

    function toogleExecScopeValuePicker(refresh) {
        switch (execScope) {
            case "version":
                versionPicker.$container.show();
                testPlanPicker.$container.hide();
                break;
            case "testPlan":
                versionPicker.$container.hide();
                testPlanPicker.$container.show();
                break;
        }

        if (refresh === true) {
            refreshCoverageCharts();
        }
    }

    function createTestPlanSingleSelectPicker($element) {
        return new AJS.SingleSelect({
            element: $element,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                formatResponse: function (data) {
                    var section = data.sections[0];
                    var optgroup = new AJS.GroupDescriptor({
                        weight: 0,
                        id: section.id + "-opt-group",
                        replace: true,
                        label: section.label + "(" + section.sub + ")"
                    });

                    $(section.issues).each(function(i, issue) {
                        var item = new AJS.ItemDescriptor({
                            value: issue.key,
                            label: issue.key + ' - ' + issue.summaryText,
                            title: issue.summary,
                            icon: AJS.contextPath() + issue.img,
                            allowDuplicate: false,
                            //selected: issue.key === strategyValue[0] ? 'selected' : null
                        });
                        optgroup.addItem(item);
                    });
                    return [optgroup];
                }
            }
        });
    }

    function initAssigneeFilterPicker() {
        var $assineePicker = $('.raven-default-user-picker');

        if ($assineePicker.data("aui-ss")) return;

        var restPath = "/rest/api/1.0/users/picker";

        var data = {
                showAvatar: true
            },
            inputText = $assineePicker.data('inputValue');

        return new AJS.SingleSelect({
            element: $assineePicker,
            submitInputVal: true,
            showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
            errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
            ajaxOptions: {
                url: contextPath + restPath,
                query: true, // keep going back to the sever for each keystroke
                data: data,
                formatResponse: JIRA.UserPickerUtil.formatResponse
            },
            inputText: inputText
        });
    }

    function initFilterValues() {
        filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
        filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
        filterValues.includePreviousVersions = includePreviousVersions();
        filterValues.status = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
        filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
        filterValues.assignee = $('#raven-filter-inline-dialog #raven-requirement-filter-assignee').val();
        filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
        filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
    }

    function saveFilterValues() {
        if ($('.filter-title.selected').data('mode') == SAVED_FILTER_MODE) {
            filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
            filterValues.fixVersion = null;
            filterValues.includePreviousVersions = null;
            filterValues.status = null;
            filterValues.resolution = null;
            filterValues.assignee = null;
            filterValues.contains = null;
            filterValues.components = null;
        } else {
            filterValues.savedFilter = null;
            filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
            filterValues.includePreviousVersions = includePreviousVersions();
            filterValues.status = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
            filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
            filterValues.assignee = $('#raven-filter-inline-dialog #raven-requirement-filter-assignee').val();
            filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
            filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
        }
    }

    function resetFilterValues() {
        if (fixVersionFieldPicker) {
            fixVersionFieldPicker._setDescriptorWithValue("-1");
        }
        if (statusFieldPicker) {
            statusFieldPicker.clearLozenges();
        }
        if (resolutionFieldPicker) {
            resolutionFieldPicker._setDescriptorWithValue("-1");
        }
        if (assigneeFieldPicker) {
            assigneeFieldPicker._setDescriptorWithValue("-1");
        }
        if (componentFieldPicker) {
        	componentFieldPicker.clearLozenges();
        }

        $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val("");

        var fieldPicker = $('#raven-requirement-filter-saved').data('field-support-obj');
        if (fieldPicker) {
            fieldPicker.clear();
        }
    }

    function renderFilterSummary() {
        var summary = '';
        if (filterValues.savedFilter && filterValues.savedFilter != "-1") {
            summary = AJS.I18n.getText('projectpanels.requirement-coverage.label.saved-filter') + ': ' + $('#raven-filter-inline-dialog #raven-requirement-filter-saved option[value="' + filterValues.savedFilter + '"]').text();
        } else {
            if (filterValues.fixVersion && filterValues.fixVersion != "-1") {
                var previousVersions = filterValues.includePreviousVersions;
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.fix-version') + ': ';
                if (previousVersions === true) {
                    summary += '<= ';
                }
                summary += $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion option[value="' + filterValues.fixVersion + '"]').text();
            }
            if (filterValues.status) {
                if (summary) {
                    summary += ', ';
                }
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.status') + ': ' + filterValues.status;
            }
            if (filterValues.resolution && filterValues.resolution != "-1") {
                if (summary) {
                    summary += ', ';
                }
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.resolution') + ': ' + $('#raven-filter-inline-dialog #raven-requirement-filter-resolution option[value="' + filterValues.resolution + '"]').text();
            }
            if (filterValues.assignee && filterValues.assignee != "-1") {
                if (summary) {
                    summary += ', ';
                }
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.assignee') + ': ' + $('#raven-filter-inline-dialog #raven-requirement-filter-assignee option[value="' + filterValues.assignee + '"]').text();
            }
            if (filterValues.contains) {
                if (summary) {
                    summary += ', ';
                }
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.contains') + ': ' + filterValues.contains;
            }
            if(filterValues.components) {
            	if (summary) {
                    summary += ', ';
                }
                summary += AJS.I18n.getText('projectpanels.requirement-coverage.label.component') + ': ' + filterValues.components;
            }
        }

        if (summary) {
            $('#raven-filter-trigger').html(summary);
        } else {
            $('#raven-filter-trigger').text(AJS.I18n.getText('projectpanels.requirement-coverage.label.filter-button'));
        }
    }

    function setHistoricPeriodFieldPickerPosition() {
        $("#raven-historic-coverage-period-field-group").css('padding-left', calculateGraphicWidth() / 2 - 300);
    }

    function calculateGraphicWidth() {
        var size = $('.aui-page-panel-content').width();
        if (size < 1200) {
            return size;
        } else {
            return size * graphContentRatio;
        }
    }

    function toogleFilters(trigger) {
        $('.filter-title.selected').removeClass('selected');

        $(trigger).addClass('selected');

        if ($(trigger).data('mode') == SAVED_FILTER_MODE) {
            $('#raven-requirement-filters-form').hide();
            $('#raven-requirement-savedfilter-form').show();
        } else {
            $('#raven-requirement-filters-form').show();
            $('#raven-requirement-savedfilter-form').hide();
        }
    }

    function initView() {
        AJS.messages.setup();

        var selectedReport = $('#raven-current-report').val();
        if (!selectedReport || selectedReport === OVERALL) {
            selectedReport = OVERALL;
        } else {
            currentReport = HISTORIC;
        }

        initSelectPickers();

        Calendar.setup({
            firstDay: 1,
            inputField: 'historic-chart-from',
            button: 'historic-chart-from-trigger',
            //align : 'Br',
            //displayArea: 'working-icon',
            singleClick: true,
            showsTime: false,
            useISO8601WeekNumbers: false,
            ifFormat: '%Y-%m-%d',
            onUpdate: onHistoricChartPeriodUpdate
        });

        Calendar.setup({
            firstDay: 1,
            inputField: 'historic-chart-to',
            button: 'historic-chart-to-trigger',
            //align : 'Br',
            //displayArea: 'working-icon',
            singleClick: true,
            showsTime: false,
            useISO8601WeekNumbers: false,
            ifFormat: '%Y-%m-%d',
            onUpdate: onHistoricChartPeriodUpdate
        });

        // show the parameters section once all fields have been initialized
        $('#raven-requirement-coverage-parameters').show();

        // init filter values and render filter summary
        initFilterValues();
        if (filterValues && filterValues.savedFilter) {
            toogleFilters($('.filter-title[data-mode=2]'));
        }
        renderFilterSummary();

        // Generate selected report upon init
        if (currentReport == HISTORIC) {
            navigateToHistoricChart();
        } else {
            navigateToOverallChart();
        }

        /**********
         * EVENTS *
         **********/

        $(document).off('.raven');

        $(document).on('click.presentation.raven', '#requirement-presentation-option a', function(e) {
            refreshCoverageCharts();
        });

        $(document).on('change.parameters.raven', '#raven-analysis-version-select, #raven-grouping-field-select', function(e) {
            refreshCoverageCharts();

            return false;
        });

        $(document).on('click.nav.overall.raven', '#raven-overall-report-opt', function(e) {
            $('#raven-report-selector-form #raven-report-type').html(AJS.I18n.getText('projectpanels.requirement-coverage.label.overall'));

            $('#raven-daily-report-opt').removeClass('hidden');
            $('#raven-daily-report-opt a').removeClass('disabled aui-dropdown2-disabled').attr('aria-disabled', 'false');
            $('#raven-overall-report-opt').addClass('hidden');
            $('#raven-overall-report-opt a').addClass('disabled').attr('aria-disabled', 'true');

            navigateToOverallChart();

            return false;
        });

        $(document).on('click.nav.historic.raven', '#raven-daily-report-opt', function(e) {
            $('#raven-report-selector-form #raven-report-type').html(AJS.I18n.getText('projectpanels.requirement-coverage.label.daily'));

            $('#raven-overall-report-opt').removeClass('hidden');
            $('#raven-overall-report-opt a').removeClass('disabled aui-dropdown2-disabled').attr('aria-disabled', 'false');
            $('#raven-daily-report-opt').addClass('hidden');
            $('#raven-daily-report-opt a').addClass('disabled aui-dropdown2-disabled').attr('aria-disabled', 'true');

            navigateToHistoricChart();

            return false;
        });

        $(document).on('click.expand.raven', '.toogle-sub-requirements', function() {
            toogleExpandRow(this);
        });

        $(document).on('click', '.filter-title:not(.selected)', function(e) {
            toogleFilters(this);
            e.stopPropagation();
        });

        $(document).on('click', '#raven-requirement-filter-fixversion-suggestions, #raven-requirement-filter-status-suggestions, #raven-requirement-filter-resolution-suggestions,  #raven-requirement-filter-assignee-suggestions, #raven-requirement-filter-saved-suggestions, #raven-requirement-filter-component-suggestions, .item-delete', function(e) {
            e.stopPropagation();
        });

        // Apply button inline Dialog
        $(document).on('click', '#raven-dialog-apply', function() {
            saveFilterValues();
            refreshCoverageCharts();

            renderFilterSummary();

            if($('#raven-filter-inline-dialog').attr('open') == "open"){
            	$('#raven-filter-inline-dialog').removeAttr('open');
            } else {
            	document.querySelector('#raven-filter-inline-dialog').hide();
            }
        });

        // inline dialog submit
        $(document).on('submit.raven', '#raven-requirement-filters-form, #raven-requirement-savedfilter-form', function() {
            saveFilterValues();
            refreshCoverageCharts();

            renderFilterSummary();

            if($('#raven-filter-inline-dialog').attr('open') == "open"){
            	$('#raven-filter-inline-dialog').removeAttr('open');
            } else {
            	document.querySelector('#raven-filter-inline-dialog').hide();
            }

            return false;
        });

        // Cancel button inline Dialog
        $(document).on('click', '#raven-dialog-cancel', function() {
            if($('#raven-filter-inline-dialog').attr('open') == "open"){
            	$('#raven-filter-inline-dialog').removeAttr('open');
            } else {
            	document.querySelector('#raven-filter-inline-dialog').hide();
            }
        });

        // Clear button inline Dialog
        $(document).on('click', '#raven-dialog-clear', function() {
            resetFilterValues();
        });
    }

    // register for NEW_CONTENT_ADDED
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
        if ($(context).find('#raven-requirement-coverage-content-panel').length > 0) {
            if (reason == JIRA.CONTENT_ADDED_REASON.pageLoad || reason == JIRA.CONTENT_ADDED_REASON.tabUpdated || reason == JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
                initView();
            }
        }
    });

})(AJS.$);
