(function($) {

	RAVEN.namespace("RAVEN.Project.TestSet");
	
    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Project.TestSet.TestSetTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testset/columns';
        },

        defaults: function() {
            return [];
        }
    });
    
    /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Project.TestSet.TestSetsView = Backbone.View.extend({
    	el: '#raven-testset',
    	
    	testSetTable: null,
    	versionPicker: null,
    	testEnvironmentPicker: null,
    	
    	clearMessage: function () {
			$('#raven-testset-message-bar').empty();
		},
		
    	message: function(title, message, type) {
			clearMessage();
			AJS.messages.error('#raven-testset-message-bar', {
				title : title,
				body : message
			});
		},
		
		fnFormatDetails: function(testSetTable, nTr) {
			var oData = $('#testset-entries-table').xrayDataTable().fnGetData(nTr);
			var renderValue;
			if (typeof oData.description === 'undefined') {
				renderValue = '-';
			} else {
				renderValue = oData.description;
			}
			var sOut = '<div class="innerDetails">'
					+ '<table cellpadding="5" cellspacing="0" border="0" style="padding-left:50px;">'
					+ '<tr><td class="label" style="width: 100px;">'+AJS.I18n.getText('common.label.description')+':</td><td>' + renderValue
					+ '</td></tr>' + '</table>' + '</div>';
			return sOut;
		},
		
    	refresh: function(){
    		this._cleanView();
    		this.initialize();
    		//this.refreshDataTable();
    	},
    	
        _cleanView: function() {
            if (this.testSetTable) {
                this.testSetTable.fnDestroy(true);
                this.testSetTable = null;
            }

            this.model = null;

            var $tableWrapper = this.$el.find('#testsets-section');

          //  $('#raven-test-filter-version').remove();
          //  $('#raven-test-analysis-version').remove();
            $tableWrapper.empty();
        },
        
    	initialize: function (){
    		var self = this;
    		
    		this.model = new RAVEN.Project.TestSet.TestSetTableMetaModel();
    
	        this.model.fetch({
	            success: function() {
	                //$('#raven-test-testruns').unblock();
	                if (self.$el.closest(document.documentElement).length > 0) {
	                    self.render();
	                }
	            },
	            error: function(model, response) {
	                self.message(AJS.I18n.getText('projectpanels-section.column-layout.error'), null, 'fatal');
	            }
	        });

    	},
    	
        render: function() {
        	var self = this;
        	$(document).on('submit.raven-testset-panel', '#raven-testset-panel-form', function(e) {
                self.refreshDataTable();
                return false;
            });
        		
            $(document).on('click.raven-testset-panel', '#raven-testset-panel-clear-filters', function(e) {
	            if (self.filterVersionPicker) {
	                self.filterVersionPicker._setDescriptorWithValue("-1");
	            }
	            
	            if(self.filterComponentPicker) {
	            	self.filterComponentPicker.clearLozenges();
	            }
	            $('#raven-testset-filter-contains-text').val('');
	
	            self.refreshDataTable();
	        });
        	
	        this.initializefilters();
	        this.createDataTable();
	        
            // render test runs toolbar view
            this.renderTestSetsToolbarView();

            this.ready = true;

            return this;
        },
        
		createDataTable: function () {
			var self = this;
			
			var projectKey = $('#raven-testset-projectKey').val();
	
			var path = contextPath + '/rest/raven/1.0/project/' + projectKey + "/testset";
	
			var $table = RAVEN.Project.renderTestSetTable({'columns': _.values(this.model.attributes)});
			
    		var $tableWrapper = $('#testsets-section');
    		$tableWrapper.empty();
    		$tableWrapper.html($table);
    		
			testSetTable = $('#testset-entries-table').xrayDataTable({
	            "bServerSide" : true,
	            "fnServerData": function( sSource, aoData, fnCallback, oSettings){
                    self.fnServerData( sSource, aoData, fnCallback, oSettings );
                },
	            "sAjaxSource": path,
				"sAjaxDataProp" : "entries",
				"bProcessing": false,
				"bAutoWidth" : false,
				"bDestroy": true,
				"bDefaultFontSize": true,
				"dataType": "json",
				"type":"GET",
				"sPaginationType" : "full_numbers",
	            "bFilter": false,
				"bSort" : true,
	            "oLanguage": {
	                "sEmptyTable": AJS.I18n.getText('projectpanels.testset.datatable.empty'),
	                "oPaginate": {
	                    "sFirst": AJS.I18n.getText('common.label.first'),
	                    "sLast": AJS.I18n.getText('common.label.last'),
	                    "sNext": AJS.I18n.getText('common.label.next'),
	                    "sPrevious": AJS.I18n.getText('common.label.previous')
	                },
	                "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
	                "sEmptyTable": AJS.I18n.getText('projectpanels.testset.datatable.empty'),
	                "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
	                "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
	                "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
	                "sSearch":  AJS.I18n.getText('common.table.label.search'),
	                "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
	            },
				"aaSorting" : [],
    			"aoColumns" : this._createTestRunTableColumns(),
	            "fnInitComplete": function(oSettings, json) {
	            },
			});
		},
		
		/**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createTestRunTableColumnDefs: function() {
            var self = this;
            var userColumnDefs = this._createDataTableUserColumnDefs();

            var lastColumnDefs = [];
    		
    		
    		return userColumnDefs.concat(lastColumnDefs);
        },
        
        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumnDefs: function() {
			
    		return _.map(_.values(this.model.attributes), function(column, index) {
    			console.log('index: ' + index);
				return {
					"mData" : 'userColumns.' + column.id,
					"bSortable" : column.sortable,
					"bSearchable" : false,
					"aTargets" : [ index ]
				};
			});
    	},
		
    	/**
         * Creates the DataTable column descriptors dynamically based on the user column layout
         */
    	_createTestRunTableColumns: function() {
            var userColumnDefs = this._createTestRunUserColumns();

            var lastColumnDefs = [
            					{
            						"mDataProp" : "statusBar",
            						"bSortable" : false,
            						"bSearchable" : false,
            						"mRender" : function(data, type, row) {
            	                        var statusBar = row["statusBar"];
            	
            							var htmlProgressBar='';
            							htmlProgressBar=htmlProgressBar+ '<ul class="testexec-progress-bar">'
            	                        
            	                        if(statusBar.count === 0) {
            	                             htmlProgressBar=htmlProgressBar+'<li class="pb-todo" style="width:100%;" title="'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+' 100%" data-status="TODO">'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+'</li>';
            	                        } else {
            	                            for (var i = 0; i < statusBar.statuses.length; i++) {
            	                                var status = statusBar.statuses[i];
            	                                htmlProgressBar = htmlProgressBar + '<li style="background-color:' + status.color + ';width:'+ status.statusPercent +'%;" title="' + status.name + ':' + status.statusCount + ' (' + status.statusPercent + '%)" data-status-id="' + status.id + '">' + status.name + '</li>';
            	                            }
            	                        }
            	
            							htmlProgressBar=htmlProgressBar+'</ul>';
            					
            							return htmlProgressBar;
            						}
            					},
            					{
            						"bSortable" : false,
            						"bSearchable" : false,
            						"mDataProp" : null,
            						"sClass" : "nav issue_actions",
            						"sDefaultContent" : '',
            						"mRender" : function(data, type, row) {
            							return '<a href="#'
            									+ row["key"]
            									+ '" aria-owns="'
            									+ row["key"]
            									+ '" aria-haspopup="true" class="aui-dropdown2-trigger aui-dropdown2-trigger-arrowless aui-style-default icon-tools-small"></a>'
            									+ '<div id="'
            									+ row["key"]
            									+ '" class="aui-dropdown2 aui-style-default">'
            									+ '<ul class="aui-list-truncate">'
            									+ '<li><a id="'
            									+ row["id"]
            									+ '" href="' + contextPath + '/secure/EditIssue!default.jspa?id='
            									+ row["id"]
            									+ '&amp;returnUrl='
            									+ window.location
            									+ '" class="editTestSetAction aui-list-item-link">'+AJS.I18n.getText('common.action.edit')+'</a></li>'
            									/*+ '<li><a id="'
            									+ row["key"]
            									+ '" href="javascript:void(0);" class="addTestAction aui-list-item-link">Add Test</a></li>' */
            									+ '<li><a href="' + contextPath + '/secure/DeleteIssue!default.jspa?id='
            									+ row["id"]
            									+ '&amp;returnUrl='
            									+ window.location
            									+ '">'+AJS.I18n.getText('common.action.delete')+'</a></li>'
            									+ '</ul></div>';
            	
            						}
            					} 
            	            ];

            return userColumnDefs.concat(lastColumnDefs);
        },
    	
    	_createTestRunUserColumns:function() {
 		   
 		   return _.map(_.values(this.model.attributes), function(column, index) {
 			   return {
 	               "mData": 'userColumns.' + column.id
 	           };
 			 });
 	   },
 	   
	   fnServerData: function( sSource, aoData, fnCallback, oSettings ) {
	      $('#raven-testset').block();
	
	      // set filter
	      this.populateFilterQueryParams(aoData);
	
	      oSettings.jqXHR = $.ajax( {
	          "dataType": 'json',
	          "type": "GET",
	          "url": sSource,
	          "data": aoData,
	          "error": function(qXHR, textStatus, errorThrown) {
	              if (errorThrown && errorThrown.trim() != '') {
	                  errorThrown = errorThrown + ': ' + qXHR.responseText;
	              } else {
	                  errorThrown = qXHR.responseText;
	              }
	              message(AJS.I18n.getText('projectpanels.testset.datatable.processingerror'), errorThrown)
	          },
	          "complete": function() {
	              $('#raven-testset').unblock();
	          },
	          "success": fnCallback
	      });
	  },
	  
	  populateFilterQueryParams: function (aoData) {
	    var versionValue = $('#raven-testset-filter-version').val();
	    if (versionValue && versionValue != -1) {
	        aoData.push({ "name": "version", "value": versionValue});
	    }
	
	    var searchValue = $('#raven-testset-filter-contains-text').val();
	    if (searchValue) {
	        aoData.push({ "name": "contains", "value": searchValue});
	    }

	    var environmentAnalysisValue = $('#raven-testset-analysis-environment').val();
        if (environmentAnalysisValue && environmentAnalysisValue != -1) {
            aoData.push({ "name": "analysisEnvironment", "value": environmentAnalysisValue});
        }
	
	    var versionAnalysisValue = $('#raven-testset-analysis-version').val();
	    if (versionAnalysisValue) {
	        aoData.push({ "name": "analysisVersion", "value": versionAnalysisValue});
	    }
	    
        var componentValue = $('#raven-testset-filter-component').val();
        if(componentValue) {
        	aoData.push({ "name": "components", "value": componentValue});
        }
	},
	
	refreshDataTable: function() {
	  if ($('#testset-entries-table').length > 0) {
	      var table = $('#testset-entries-table').xrayDataTable();
	      table.fnDraw(false);
	  }
	},
	
	initializefilters: function(){
		var self = this;
		
		var projectKey = $('#raven-testset-projectKey').val();
		
		if(!this.filterVersionPicker){
			this.filterVersionPicker = this.createVersionPicker($('#raven-testset-filter-version'));
		}
		
		if(!this.analisysVersionPicker){
			this.analisysVersionPicker = this.createVersionPicker($('#raven-testset-analysis-version'));
		}
		
		if(!this.filterComponentPicker) {
			this.filterComponentPicker = this.createComponentPicker($("#raven-testset-filter-component"));
		}

		if(!this.testEnvironmentPicker){ 
            this.testEnvironmentPicker = this._createTestEnvironmentPicker($('#raven-testset-analysis-environment'));
        }

        $('#raven-testset-analysis-environment').on('change', function(e) {
            self.refreshDataTable();
        });
		
	    $('#raven-testset-analysis-version').on('change', function(e) {
	        self.refreshDataTable();
	    });

		$('#raven-testset-panel-form').show();
	},
	
	createVersionPicker: function($selectField) {
		 
		$selectField.on('initialized', function(e, field) {            
		      var version = $('body').data('version');
		      if (!version || version.toString().charAt(0) === '5') {
		          $('input', $selectField.parent()).height('23px');
		      }
		  });
		  
		  return new AJS.SingleSelect({
		      element: $selectField,
		      showDropdownButton: true,
		      revertOnInvalid: true,
		  });
		},
	
	createComponentPicker: function($selectField) {
    	return new AJS.MultiSelect({
			element: $selectField,
			itemAttrDisplayed: "label",
			showDropdownButton: true,
			submitInputVal: false,
			noQueryNoRequest: false,
			revertOnInvalid: true,
			loadOnInit: true,
			errorMessage: "",
			ajaxOptions: {
				query: true,
				url: contextPath + '/rest/api/2/project/' + $('#raven-testset-projectKey').val() + '/components',
				formatResponse: function (data) {
					var optgroup = new AJS.GroupDescriptor({
						weight: 0,
						id: "all-filter-opt-group",
						replace: true
					});

					$(data).each(function(i, status) {
						this.label = this.name;
						var item = new AJS.ItemDescriptor({
							value: this.name,
							label: this.name,
							title: this.description,
							
							highlighted: true,
						});
						optgroup.addItem(item);
					});

					return [optgroup];
				}
			}
		});
    },

    _createTestEnvironmentPicker: function ($selectField) {
        var testEnvironmentsCFID = $("#raven-testenvironments-cf-id").val();

        return new AJS.SingleSelect({
            element: $selectField,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + "/rest/api/1.0/labels/suggest?customFieldId=" + testEnvironmentsCFID,
                formatResponse: this._formatEnvironmentFilterResponse
            }
        });
    },

    _formatEnvironmentFilterResponse: function(data) {
        var ret = [];

        ret.push(new AJS.ItemDescriptor({
            value: "-1",
            label: AJS.I18n.getText("common.label.all"), // title of lozenge
            icon: null,
            allowDuplicate: false,
            highlighted: true
        }));

        ret.push(new AJS.ItemDescriptor({
            value: "__NULL__",
            label: AJS.I18n.getText("common.label.empty"), // title of lozenge
            icon: null,
            allowDuplicate: false,
            highlighted: true
        }));

        if (data && data.suggestions) {
            var suggestionsGroupDescriptor = new AJS.GroupDescriptor({
                weight: 0, // order or groups in suggestions dropdown
                label: AJS.I18n.getText("common.label.available-environment-suggestions")
            });

            $(data.suggestions).each(function(i, env) {
                var item = new AJS.ItemDescriptor({
                    value: env.label,
                    label: env.label,
                    allowDuplicate: false
                });
                suggestionsGroupDescriptor.addItem(item);
            });

            ret.push(suggestionsGroupDescriptor);
        }

        return ret;
    },
		
    renderTestSetsToolbarView: function() {
        var $wrapper = $('.xrayDataTables_wrapper');
        // if filter already exists then return
        if ($wrapper.find('#toolbar-wrapper').length > 0) {
            return;
        }

        var toolbarHtml = RAVEN.Project.renderTestRunsToolBar();
        $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

        this.columnLayoutView = new RAVEN.Project.TestSetsColumnLayoutView({
            mainView: this,
        });
    }
		
    });
    
    RAVEN.Project.TestSet.TestSetsView.REFRESH_VIEW = "refreshTestView";
    
})(AJS.$);