(function($) {
    RAVEN.namespace("RAVEN.TestRepository.Fields");

    RAVEN.TestRepository.Fields.FolderPicker = Backbone.View.extend({

        repositoryPath: null,

        entityKey: null,

        folderJQLName: null,

        $picker: null,

        $selector: null,

        $tree: null,

        siblingDropdownElements: null,

        testRepositoryView: null,

        unscopedEvents: {
            'pickFolderEvt .raven-test-repository-folder-picker': '_pickFolderEvt',

            /*** selectFolderSilentEvt
             * sets the selected option in the 'select' component without triggering the changeFolderPick event. required for silent initialization
             */
            'selectFolderSilentEvt .raven-test-repository-folder-picker': '_selectFolderSilentEvt',

            'click.raven.test-repository.picker .raven-test-repository-folder-picker': '_showFolderPicker',
            'click.raven.test-repository.picker .raven-test-repository-folder-tree': '_clickOnFolderTree',
            'click.raven.test-repository.picker html': '_hideFolderPicker'
        },

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "repositoryPath"));
                _.extend(this, _.pick(options, "entityKey"));
                _.extend(this, _.pick(options, "folderJQLName"));
                _.extend(this, _.pick(options, "$picker"));
                _.extend(this, _.pick(options, "siblingDropdownElements"));
            } else {
                console.log('Error: unable to init Test Repository Folder Picker');
                return null;
            }

            this._initializeFolderPicker();
        },

        _initializeFolderPicker: function() {
            this._delegateUnscopedEvents();

            this.render();

            this.testRepositoryView = new RAVEN.TestRepository.Fields.TestRepositoryView({
                el: this.$tree,
                repositoryPath: this.repositoryPath,
                entityKey: this.entityKey,
                testRepositoryFolderPicker: $('.raven-test-repository-folder-picker')
            });

            this._initDomChangeEvt();
        },

        render: function() {
            var pickerId = this.$picker.attr("id");

            var $inputName = $('<input/>').attr({
                "id": pickerId + "-field-name",
                "class": "text aui-ss-field ajs-dirty-warning-exempt",
                "type": "text",
                "readonly": "true"
            });

            var $inputId = $('<input/>').attr({
                "id": pickerId + "-field-id",
                "class": "text aui-ss-field ajs-dirty-warning-exempt",
                "type": "hidden"
            });

            var $dropMenu = $('<span/>')
                .attr({
                    "class": "icon aui-ss-icon noloading drop-menu"
                });

            this.$selector = $('<div/>')
                .attr({
                    "id": pickerId + "-picker",
                    "class": "aui-ss ajax-ss raven-test-repository-folder-picker"
                });

            this.$selector
                .append($inputName)
                .append($inputId)
                .append($dropMenu);

            this.$tree = $('<div/>')
                .attr({
                    "id": pickerId + "-tree",
                    "class": "raven-test-repository-folder-tree"
                })
                .css({
                    "display": "none"
                });
            this.$picker.empty();
            this.$picker
                .append(this.$selector)
                .append(this.$tree);
        },

        _isDropdownFilter: function(id) {
            return $.inArray(id, this.siblingDropdownElements) > -1;
        },

        clearPicker: function() {
            this.testRepositoryView.resetPicker();
        },

        setEntityKey: function(entityKey) {
            this.entityKey = entityKey;
        },

        resetFolderPicker: function() {
            this._undelegateUnscopedEvents();
            this._initializeFolderPicker();
        },

        buildJql: function() {
            var path = $('#' + this.$picker.attr("id") + '-field-name').val();
            var pathArr = path.split("/").slice(1);
            if(pathArr && pathArr.length > 0) {
                return 'issue in ' + this.folderJQLName + ' ("' + this.entityKey + '", "' + pathArr.join("/") + '")';
            }
            return null;
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.test-repository.picker');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.test-repository.picker');
        },

        _initDomChangeEvt: function() {
            var self = this;
            $(window).bind("DOMSubtreeModified",function(e){
                if(self._isDropdownFilter(e.target.id)) {
                    self._hideFolderPicker();
                }
            });
        },

        _selectFolderSilentEvt: function(e, folderId, path) {
            var pickerId = this.$picker.attr("id");

            var $fieldPickerName = this.$picker.find('#' + pickerId + '-field-name');
            $fieldPickerName.val(path);

            this._initTooltip($fieldPickerName, path);

            var $fieldPickerId = this.$picker.find('#' + pickerId + '-field-id');
            $fieldPickerId.val(folderId);
        },

        _pickFolderEvt: function(e, folderId, path) {
            this._selectFolderSilentEvt(e, folderId, path);
            this.$picker.trigger('changeFolderPick', folderId);

            this._hideFolderPicker();
        },

        _initTooltip: function($elem, path) {
            $elem.attr('title', path);
            AJS.$($elem).tooltip();
        },

        _showFolderPicker: function(e) {
            e.stopPropagation();

            if(this.$tree.is(":hidden")) {
                this._showPicker();
            } else {
                this._hideFolderPicker();
            }
        },

        _showPicker: function() {
            this.$tree.show();
            this.$picker.find('input').first().addClass("active");
        },

        _hideFolderPicker: function() {
            this.$tree.hide();
            this.$picker.find('input').first().removeClass("active");
        },

        _clickOnFolderTree: function(e) {
            e.stopPropagation();
        }
    });

    RAVEN.TestRepository.Fields.TestRepositoryView = Backbone.View.extend({
        el: null,

        FOLDER_ROOT: 'folder-root',

        ALL_TESTS: 'all-tests',

        FOLDER_ROOT_PATH: AJS.I18n.getText('raven.testrepository.testplan.root-folder.description'),

        folderView: null,

        entityKey: null,

        repositoryPath: null,

        testRepositoryFolderPicker: null,

        defaultFolderItemWidth: null,

        MIN_FOLDER_NAME_WIDTH: 250,

        TREE_SELECTOR_MIN_WIDTH: 450,

        TREE_DOT_LINE_SIZE: 36,

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "el"));
                _.extend(this, _.pick(options, "repositoryPath"));
                _.extend(this, _.pick(options, "testRepositoryFolderPicker"));
                _.extend(this, _.pick(options, "entityKey"));
            } else {
                console.log('Error: invalid repository path');
                return null;
            }

            var self = this;
            this.$el
                .on({
                    'folderInitializedEvt': function () {
                        self._initializeFilterPickerProperties();
                    },
                    'folderActiveEvt': function (e, id) {
                        var path = self.folderView.buildCompletePath(id);
                        self.testRepositoryFolderPicker.trigger('pickFolderEvt', [id, path]);
                    },
                    'folderSelectEvt': function (e, id) {
                        self._increaseTreeWidth(id);
                    },
                    'folderCollapseEvt': function (e, id, maxTreeLevel) {
                        self._decreaseTreeWidth(id, maxTreeLevel);
                    }
                });

            this.folderView = new RAVEN.Folder.FolderView({
                'el': this.$el,
                'mainView': this,
                'repositoryPath': this.repositoryPath,
                'entityKey': this.entityKey,
                'hasAllFolder': true,
                'hasOrphansFolder': false,
                'rootDescription': this.FOLDER_ROOT_PATH,
                'isEditable': false,
                'hasFolderActions': false
            });
        },

        resetPicker: function() {
            this.folderView.activateAllFolderAndResetPath();
            this.testRepositoryFolderPicker.trigger('pickFolderEvt', [this.ALL_TESTS, 'All']);
        },

        _initializeFilterPickerProperties: function() {
            var $testRepositoryTree = this.$el.find('#raven-test-repository-tree');

            $testRepositoryTree.children('ul').css('cssText', 'padding-left: 14px !important');
            $testRepositoryTree.children('ul').first().css('margin-top', '14px');
            $testRepositoryTree.children('ul').last().css('margin-bottom', '14px');
            $testRepositoryTree.css('overflow-x', 'auto');
            $testRepositoryTree.css('min-height', '55px');
            $testRepositoryTree.css('max-height', '500px');

            this.defaultFolderItemWidth = $testRepositoryTree.find('.raven-folder-item:not(#folder-root)').first().width();
            this.folderView.activateAllFolderAndResetPath();
            this.testRepositoryFolderPicker.trigger('selectFolderSilentEvt', [this.ALL_TESTS, 'All']);
        },

        _increaseTreeWidth: function(id) {
            var $testRepositoryTree = this.$el.find('#raven-test-repository-tree');
            var extraSpace = 0;
            if (id !== this.ALL_TESTS) {
                var $activeFolderViewMode = $testRepositoryTree.find('.raven-activeFolder .raven-folders-view-mode').first();
                var width = $activeFolderViewMode.width();
                if (width < this.MIN_FOLDER_NAME_WIDTH && width > extraSpace) {
                    extraSpace = this.MIN_FOLDER_NAME_WIDTH - width;
                }

                var $folderRoot = $testRepositoryTree.find('.' + this.ALL_TESTS);
                $folderRoot.width($folderRoot.width() + extraSpace);
            }
        },

        _decreaseTreeWidth: function(id, maxTreeLevel) {
            var $testRepositoryTree = this.$el.find('#raven-test-repository-tree');
            var $collapsedElem = $testRepositoryTree.find('#' + id);

            var self = this;
            $.each(maxTreeLevel.ids, function (i, maxLevelId) {
                if ($collapsedElem.find('#' + maxLevelId).andSelf().length > 0) {
                    var treeDotLineExtension = $collapsedElem.find('.raven-folder-item-structure').first().children('.raven-tree-dot-line-extension').length;
                    var treeDotLine = $collapsedElem.find('.raven-folder-item-structure').first().children('.raven-tree-dot-line').length;

                    var collapsedElemLevel = treeDotLineExtension + treeDotLine;

                    var $folderRoot = $testRepositoryTree.find('.' + self.ALL_TESTS);
                    var folderRootWidth = $folderRoot.width();

                    if (folderRootWidth > self.TREE_SELECTOR_MIN_WIDTH) {
                        var newFolderRootWidth = folderRootWidth - self.TREE_DOT_LINE_SIZE * (maxTreeLevel.level - collapsedElemLevel);
                        if (newFolderRootWidth <= self.TREE_SELECTOR_MIN_WIDTH) {
                            $folderRoot.width(self.TREE_SELECTOR_MIN_WIDTH);
                        } else {
                            $folderRoot.width(newFolderRootWidth);
                        }
                    }
                }
            });
        }
    });
})(AJS.$);