(function($) {

    function onNewContentAdded(e, context, reason) {
        if (reason === JIRA.CONTENT_ADDED_REASON.pageLoad || reason === JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
            var $section = $('#raven-test-repository-wrapper');
            if ($section.length > 0 && !$section.attr('data-initialized')) {
                initialize();
            }
        }
    }

    function initialize() {
        var testRepository = new RAVEN.TestRepository.TestRepositoryWrapperView({
            repositoryConfig: getRepositoryConfig()
        });

        $('#raven-test-repository-wrapper').attr("data-initialized", "true");
    }

    var getRepositoryConfig = function() {
        var projectKey = $('#selected-project-id').val();
        var filterDefinition = getFilterDefinition();
        var folderContentRenderer = createContentRenderer();
        var rootDescription = getRootFolderDescription();
        return {
            filterDefinition: filterDefinition,
            repositoryPath: 'folderStructure',
            jqlConstant: 'project = "' + projectKey + '"',
            folderJQLName: 'TestRepositoryFolderTests',
            allowDialogProjectFilter: false,
            allowDialogFolderFilter: false,
            allowContextMenuTestPlan: true,
            allowEnvironmentSelect: false,
            allowRenderEnvironment: false,
            folderContentRenderer: folderContentRenderer,
            rootFolderDescription: rootDescription,
            createTestFieldFiller: createTestFieldFiller
        }
    };

    var getFilterDefinition = function() {
        var groupAttributes = AJS.I18n.getText('common.label.attributes');
        var groupEntities = AJS.I18n.getText('common.label.related-entities');
        var groups = [groupAttributes, groupEntities];
        var filters = [
            {key: RAVEN.Filters.KEY_TEST_TYPE, group: groupAttributes},
            {key: RAVEN.Filters.KEY_LABEL, group: groupAttributes},
            {key: RAVEN.Filters.KEY_PRIORITY, group: groupAttributes},
            {key: RAVEN.Filters.KEY_COMPONENT, group: groupAttributes},
            {key: RAVEN.Filters.KEY_WORKFLOW_STATUS, group: groupAttributes},
            {key: RAVEN.Filters.KEY_CONTAINS, group: groupAttributes},

            {key: RAVEN.Filters.KEY_REQUIREMENT, group: groupEntities},
            {key: RAVEN.Filters.KEY_REQUIREMENTVERSION, group: groupEntities},
            {key: RAVEN.Filters.KEY_EMPTY, group: groupEntities},
            {key: RAVEN.Filters.KEY_TESTSET, group: groupEntities},
            {key: RAVEN.Filters.KEY_TESTSET_VERSION, group: groupEntities},
            {key: RAVEN.Filters.KEY_SAVED_FILTER, group: groupEntities},
        ];
        return {groups: groups, filters:filters};
    };

    var createContentRenderer = function() {
        return new RAVEN.TestRepository.FolderContentRenderer('path-container', getRootFolderDescription());
    };

    var getRootFolderDescription = function() {
        return AJS.I18n.getText('raven.test-repository.folders.root-tests');
    };

    var createTestFieldFiller = function(context) {
        var projectId = $('#project-id').val();

        context.find('input[id=project]').val(projectId);
        context.find('input[id=project]').trigger('change');
    };
    // register for NEW_CONTENT_ADDED
    JIRA.unbind(JIRA.Events.NEW_CONTENT_ADDED, onNewContentAdded);
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, onNewContentAdded);

})(AJS.$);