(function ($) {

    function onNewContentAdded(e, context, reason) {
        if (reason === JIRA.CONTENT_ADDED_REASON.pageLoad || reason === JIRA.CONTENT_ADDED_REASON.panelRefreshed) {
            var $section = $('#raven-test-repository-wrapper');
            if ($section.length > 0 && !$section.attr('data-initialized')) {
                initialize();
            }
        }
    }

    function initialize() {
        setSelectedBarOption();
        initializeTestPlanPicker('#raven-testplan-selector');
        initializeStatusBar();
        var testRepository = new RAVEN.TestRepository.TestRepositoryWrapperView({
            repositoryConfig: getRepositoryConfig()
        });

        $('#raven-test-repository-wrapper').attr("data-initialized", "true");
    }

    var getRepositoryConfig = function () {
        var testPlanKey = $('#selected-entity-id').val();
        var filterDefinition = getFilterDefinition();
        var folderContentRenderer = createContentRenderer();
        var rootDescription = getRootFolderDescription();
        return {
            filterDefinition: filterDefinition,
            repositoryPath: 'testPlanRepository',
            jqlConstant: 'issue in testPlanTests("' + testPlanKey + '")',
            folderJQLName: 'testPlanFolderTests',
            allowDialogProjectFilter: true,
            allowDialogFolderFilter: true,
            allowContextMenuTestPlan: false,
            allowEnvironmentSelect: true,
            allowRenderEnvironment: true,
            folderContentRenderer: folderContentRenderer,
            rootFolderDescription: rootDescription,
            createTestFieldFiller: createTestFieldFiller
        }
    };

    var setSelectedBarOption = function() {
        $(".aui-nav-selected").removeClass("aui-nav-selected");
        $('a[data-link-id="com.xpandit.plugins.xray:raven.sidebar.testplan-repository"]').parent().addClass('aui-nav-selected');
    };

    var getFilterDefinition = function () {
        var groupAttributes = AJS.I18n.getText('common.label.attributes');
        var groupEntities = AJS.I18n.getText('common.label.related-entities');
        var groups = [groupAttributes, groupEntities];
        var filters = [
            {key: RAVEN.Filters.KEY_PROJECT, group: groupAttributes},
            {key: RAVEN.Filters.KEY_EMPTY, group: groupAttributes},
            {key: RAVEN.Filters.KEY_EMPTY, group: groupAttributes},
            {key: RAVEN.Filters.KEY_TEST_TYPE, group: groupAttributes},
            {key: RAVEN.Filters.KEY_LABEL, group: groupAttributes},
            {key: RAVEN.Filters.KEY_PRIORITY, group: groupAttributes},
            {key: RAVEN.Filters.KEY_COMPONENT, group: groupAttributes},
            {key: RAVEN.Filters.KEY_WORKFLOW_STATUS, group: groupAttributes},
            {key: RAVEN.Filters.KEY_CONTAINS, group: groupAttributes},

            {key: RAVEN.Filters.KEY_REQUIREMENT, group: groupEntities},
            {key: RAVEN.Filters.KEY_REQUIREMENTVERSION, group: groupEntities},
            {key: RAVEN.Filters.KEY_EMPTY, group: groupEntities},
            {key: RAVEN.Filters.KEY_TESTSET, group: groupEntities},
            {key: RAVEN.Filters.KEY_TESTSET_VERSION, group: groupEntities},
            {key: RAVEN.Filters.KEY_SAVED_FILTER, group: groupEntities},
        ];
        return {groups: groups, filters: filters};
    };

    var initializeTestPlanPicker = function () {
        var testPlanKey = $('#selected-entity-id').val();
        var projectKey = $('#selected-project-id').val();
        var $selectField = $('#raven-testplan-selector');

        var testplanSelector = new AJS.SingleSelect({
            element: $selectField,
            showDropdownButton: true,
            submitInputVal: true,
            noQueryNoRequest: false,
            revertOnInvalid: true,
            loadOnInit: true,
            ajaxOptions: {
                query: true,
                url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN&jql=' + 'project="' + projectKey + '" and issue not in("' + testPlanKey + '")',
                formatResponse: function (data) {
                    var section = data.sections[0];
                    var optgroup = new AJS.GroupDescriptor({
                        weight: 0,
                        id: section.id + "-opt-group",
                        replace: true,
                        label: section.label + "(" + section.sub + ")"
                    });

                    $(section.issues).each(function (i, issue) {
                        var item = new AJS.ItemDescriptor({
                            value: issue.key,
                            label: issue.key,
                            title: issue.key,
                            allowDuplicate: false,
                            selected: null
                        });
                        optgroup.addItem(item);
                    });
                    return [optgroup];
                }
            }
        });

        testplanSelector.setSelection(
            new AJS.ItemDescriptor({
                value: testPlanKey,
                label: testPlanKey,
                icon: null,
                allowDuplicate: false,
                selected: true
            })
        );

        $('select[name=raven-testplan-selector]').change(function (e) {
            var val = $("select[name=raven-testplan-selector] option:selected").val();
            if (val === testPlanKey) {
                return;
            }
            $('#raven-testplan-selector-field').css('padding-left', '');
            window.location = contextPath + '/secure/XrayTestPlanRepositoryAction!default.jspa?testPlanKey=' + val;
        });

        $('#raven-testplan-selector-container input').blur(function (e) {toggleSelector()});
        $('#raven-testplan-mode-button').click(function(e){ toggleSelector();});
        toggleSelector();
    };

    var toggleSelector = function() {
        var $selector = $('#raven-testplan-selector-single-select');
        var $link = $('#raven-testplan-link-container');

        if ($selector.is(':visible')) {
            $selector.hide();
            $link.show();
        } else {
            $selector.show();
            $link.hide();
            $selector.find('input').first().focus();
        }
    };

    var initializeStatusBar = function () {
        var testPlanKey = $('#selected-entity-id').val();
        var self = this;
        $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/testplan/' + testPlanKey + "/statusBar",
                processData: true,
                contentType: "application/json",
                success: function(data) {
                    var content = renderStatusBar(data, 580, 'raven-testplan-progress-bar');
                    $('#raven-testplan-status-container').html(content);
                }
            }
        )
    };

    var createContentRenderer = function () {
        return new RAVEN.TestRepository.TestPlanFolderContentRenderer('path-container', getRootFolderDescription(), 'raven-status-container', 300, 'raven-testplan-progress-bar');
    };

    var getRootFolderDescription = function() {
        return AJS.I18n.getText('raven.testrepository.testplan.root-folder.description');
    };

    var createTestFieldFiller = function(context) {
        var testPlanKey = $('#selected-entity-id').val();
        var testPlanCustomFieldId = 'customfield_' + $('#custom-field-test-testplan-id').val();
        context.find('select[id='+testPlanCustomFieldId+']')
            .trigger('selectOption',[{label:testPlanKey,value:testPlanKey,title:testPlanKey}]);
    };
    // register for NEW_CONTENT_ADDED
    JIRA.unbind(JIRA.Events.NEW_CONTENT_ADDED, onNewContentAdded);
    JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, onNewContentAdded);

})(AJS.$);